"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
/*
 * #%L
 * %%
 * Copyright (C) 2019 BMW Car IT GmbH
 * %%
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * #L%
 */
const crypto_1 = __importDefault(require("crypto"));
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const util_1 = require("util");
const encoding = "utf8";
const LoggingManager_1 = __importDefault(require("../joynr/system/LoggingManager"));
const log = LoggingManager_1.default.getLogger("joynr.global.JoynrPersist");
const readFileAsync = util_1.promisify(fs_1.default.readFile);
const writeFileAsync = util_1.promisify(fs_1.default.writeFile);
const readDirAsync = util_1.promisify(fs_1.default.readdir);
const lstatAsync = util_1.promisify(fs_1.default.lstat);
const mkdirAsync = util_1.promisify(fs_1.default.mkdir);
const unlinkAsync = util_1.promisify(fs_1.default.unlink);
function hashFileName(key) {
    return crypto_1.default
        .createHash("md5")
        .update(key)
        .digest("hex");
}
const resolveDir = function (dir) {
    dir = path_1.default.normalize(dir);
    if (path_1.default.isAbsolute(dir)) {
        return dir;
    }
    return path_1.default.join(process.cwd(), dir);
};
async function mkdirRecursive(directory) {
    try {
        await mkdirAsync(directory);
    }
    catch (e) {
        if (e.code === "ENOENT") {
            const parentDir = path_1.default.dirname(directory);
            await mkdirRecursive(parentDir);
            try {
                await mkdirAsync(directory);
            }
            catch (err) {
                if (err.code === "EEXIST") {
                    log.info(`Directory: ${directory} already exists`);
                }
                else {
                    throw err;
                }
            }
        }
        else if (e.code === "EEXIST") {
            log.info(`Directory: ${directory} already exists`);
        }
        else {
            throw e;
        }
    }
}
class JoynrPersist {
    constructor(options) {
        this.directory = resolveDir(options.dir);
    }
    async init() {
        await lstatAsync(this.directory).catch(() => mkdirRecursive(this.directory));
        return this.getAllData().catch(e => {
            if (e.code === "EISDIR") {
                const files = fs_1.default.readdirSync(this.directory);
                const subDirectories = files.filter(file => {
                    const filePath = path_1.default.join(this.directory, file);
                    return fs_1.default.lstatSync(filePath).isDirectory();
                });
                throw new Error(`joynr configuration error: Persistency subdirectory must not include other subdirectories. Directories found: ${JSON.stringify(subDirectories)}`);
            }
            else {
                throw e;
            }
        });
    }
    getPath(key) {
        return path_1.default.join(this.directory, hashFileName(key));
    }
    getAllData() {
        return readDirAsync(this.directory)
            .then(files => {
            const promises = [];
            for (let i = 0, length = files.length; i < length; i++) {
                const currentFile = files[i];
                const filePath = path_1.default.join(this.directory, currentFile);
                /*eslint-disable promise/no-nesting*/
                promises.push(readFileAsync(filePath, encoding)
                    .then(JSON.parse)
                    .catch(e => {
                    if (e.code === "EISDIR") {
                        throw e;
                    }
                    log.warn(`corrupted files in JoynrPersist directory: unable to parse ${filePath}. Ignoring file. Error: ${e}`);
                    return null;
                }));
                /*eslint-enable promise/no-nesting*/
            }
            return Promise.all(promises);
        })
            .then(arr => {
            return arr.filter(element => element !== null);
        });
    }
    setItem(key, value) {
        const datum = { key, value };
        return writeFileAsync(this.getPath(key), JSON.stringify(datum));
    }
    removeItem(key) {
        return unlinkAsync(this.getPath(key));
    }
    async clear() {
        // unfortunately we can't spawn a child process and call rm -rf
        // TOTO: check if it's bad to delete all files here.
        return readDirAsync(this.directory).then(files => {
            const promises = [];
            for (let i = 0, length = files.length; i < length; i++) {
                const currentFile = files[i];
                promises.push(unlinkAsync(path_1.default.join(this.directory, currentFile)));
            }
            return Promise.all(promises);
        });
    }
}
module.exports = JoynrPersist;
