"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
const JoynrPersist_1 = __importDefault(require("./JoynrPersist"));
const LoggingManager_1 = __importDefault(require("../joynr/system/LoggingManager"));
const log = LoggingManager_1.default.getLogger("joynr.global.localStorageNode");
class LocalStorageWrapper {
    /**
     * LocalStorage constructor (node wrapper for LocalStorage)
     * @constructor LocalStorageWrapper
     * @classdesc node wrapper for LocalStorage
     *
     * @param settings the settings object
     * @param settings.clearPersistency localStorage is cleared if set to true
     * @param settings.location optional, passed on to node-persist LocalStorage constructor
     */
    constructor(settings) {
        settings = settings || {};
        this.storage = new JoynrPersist_1.default({
            dir: settings.location || "./localStorageStorage"
        });
        this.map = new Map();
        this.promiseChain = Promise.resolve();
        this.settings = settings;
    }
    setItem(key, value) {
        this.map.set(key, value);
        this.wrapFunction(this.storage.setItem.bind(this.storage), key, value);
    }
    getItem(key) {
        return this.map.get(key);
    }
    removeItem(key) {
        this.map.delete(key);
        this.wrapFunction(this.storage.removeItem.bind(this.storage), key);
    }
    clear() {
        this.map.clear();
        this.wrapFunction(this.storage.clear.bind(this.storage));
    }
    async init() {
        const storageData = await this.storage.init();
        if (this.settings.clearPersistency) {
            return await this.storage.clear();
        }
        for (let i = 0, length = storageData.length; i < length; i++) {
            const storageObject = storageData[i];
            if (storageObject && storageObject.key) {
                this.map.set(storageObject.key, storageObject.value);
            }
        }
    }
    async shutdown() {
        await this.promiseChain;
    }
    wrapFunction(cb, ...args) {
        this.promiseChain = this.promiseChain
            .then(() => {
            // eslint-disable-next-line promise/no-callback-in-promise
            return cb(...args);
        })
            .catch((e) => {
            log.error(`failure executing ${cb} with args ${JSON.stringify(args)} error: ${e}`);
        });
    }
}
module.exports = LocalStorageWrapper;
