"use strict";
/*eslint global-require: "off"*/
/*
 * #%L
 * %%
 * Copyright (C) 2019 BMW Car IT GmbH
 * %%
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * #L%
 */
/* istanbul ignore file */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
const JoynrObject_1 = __importDefault(require("./joynr/types/JoynrObject"));
const JoynrApi = require("./libjoynr-deps");
const UdsLibJoynrRuntime_1 = __importDefault(require("./joynr/start/UdsLibJoynrRuntime"));
const LoggingManager_1 = __importDefault(require("./joynr/system/LoggingManager"));
const log = LoggingManager_1.default.getLogger("joynr");
/**
 * copies all non private members and methods to joynr
 * methods need to be bound with this and can't be on the prototype
 *
 * @param joynr
 * @param runtime
 */
function wrapRuntime(joynr, runtime) {
    Object.keys(runtime).forEach((key) => {
        if (!key.startsWith("_")) {
            joynr[key] = runtime[key];
        }
    });
}
class Joynr extends JoynrApi {
    constructor() {
        super(...arguments);
        this.loaded = false;
        this.JoynrObject = JoynrObject_1.default;
        this.selectedRuntime = UdsLibJoynrRuntime_1.default;
    }
    /**
     * @param provisioning
     * @param onFatalRuntimeError Called in case a runtime error prevents further communication
     * @return Promise object being resolved in case all libjoynr dependencies are loaded
     */
    async load(provisioning, onFatalRuntimeError) {
        this.loaded = true;
        if (!onFatalRuntimeError) {
            onFatalRuntimeError = (error) => {
                log.fatal(`Unexpected joynr runtime error occured: ${error}`);
            };
        }
        const runtime = new this.selectedRuntime(onFatalRuntimeError);
        await runtime.start(provisioning);
        wrapRuntime(this, runtime);
        // make sure the runtime is shutdown when process.exit(...)
        // gets called since otherwise the process might not
        // terminate. Ignore any exception thrown in case shutdown
        // had already been invoked manually before reaching this
        // point.
        if (typeof process === "object" && typeof process.on === "function") {
            process.on("exit", () => {
                try {
                    // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                    this.shutdown({ clearSubscriptionsEnabled: false });
                }
                catch (error) {
                    // ignore
                }
            });
        }
        return this;
    }
    /**
     * Changes the selected default joynr runtime from websocket to the passed runtime
     *
     * @param runtime child class of JoynrRuntime
     * @returns joynr for easy chaining
     */
    selectRuntime(runtime) {
        if (this.loaded) {
            throw new Error("joynr.selectRuntime: this method must be invoked before calling joynr.load()");
        }
        this.selectedRuntime = runtime;
        return this;
    }
}
const joynr = new Joynr();
module.exports = joynr;
