import * as Address from "../../generated/joynr/system/RoutingTypes/Address";
import * as ProviderQos from "../../generated/joynr/types/ProviderQos";
import { JoynrProvider } from "../types/JoynrProvider";
import MessageRouter = require("../messaging/routing/MessageRouter");
import ParticipantIdStorage = require("./ParticipantIdStorage");
import PublicationManager = require("../dispatching/subscription/PublicationManager");
import RequestReplyManager = require("../dispatching/RequestReplyManager");
import { DiscoveryStub } from "./interface/DiscoveryStub";
interface RegistrationSettings {
    domain: string;
    provider: JoynrProvider;
    providerQos: ProviderQos;
    expiryDateMs?: number;
    loggingContext?: Record<string, any>;
    participantId?: string;
    awaitGlobalRegistration?: boolean;
}
interface RegistrationSettingsWithGbids extends RegistrationSettings {
    gbids?: string[];
}
declare class CapabilitiesRegistrar {
    private started;
    private publicationManager;
    private requestReplyManager;
    private libjoynrMessagingAddress;
    private participantIdStorage;
    private messageRouter;
    private discoveryStub;
    /**
     * The Capabilities Registrar
     *
     * @constructor
     *
     * @param dependencies
     * @param dependencies.discoveryStub connects the inProcessStub to its skeleton
     * @param dependencies.messageRouter used to register nextHop for registered provider
     * @param dependencies.participantIdStorage connects the inProcessStub to its skeleton
     * @param dependencies.libjoynrMessagingAddress address to be used by the cluster controller to send incoming requests to this
     *            libjoynr's providers
     * @param dependencies.requestReplyManager passed on to providerAttribute, providerOperation and providerEvent
     * @param dependencies.publicationManager passed on to providerAttribute
     */
    constructor(dependencies: {
        discoveryStub: DiscoveryStub;
        messageRouter: MessageRouter;
        participantIdStorage: ParticipantIdStorage;
        libjoynrMessagingAddress: Address;
        requestReplyManager: RequestReplyManager;
        publicationManager: PublicationManager;
    });
    /**
     * Internal function used to throw exception in case of CapabilitiesRegistrar
     * is not used properly
     */
    private checkIfReady;
    /**
     * Registers a provider so that it is publicly available
     *
     * If registration to local and global scope is requested by 'providerQos' parameter,
     * the provider is registered to all GBIDs configured in the cluster controller.
     *
     * The 'gbids' parameter can be provided to override the GBIDs selection in the cluster
     * controller. The global capabilities directory identified by the first selected GBID performs
     * the registration.
     *
     * @param settings the arguments object for this function call
     * @param settings.domain
     * @param settings.provider
     * @param settings.providerQos the Quality of Service parameters for provider registration
     * @param [settings.expiryDateMs] date in millis since epoch after which the discovery entry can be purged from all directories.
     *            Default value is one day.
     * @param [settings.loggingContext] optional logging context will be appended to logging messages created in the name of this proxy
     * @param [settings.participantId] optional. If not set, a globally unique UUID participantId will be generated, and persisted to
     *            localStorage. If set, the participantId must be unique in the context of the provider's scope, as set in the ProviderQos;
     *            The application setting the participantId is responsible for guaranteeing uniqueness.
     * @param [settings.awaitGlobalRegistration] optional. If provided and set to true registerProvider will wait until local and global
     *            registration succeeds or timeout is reached: otherwise registerProvider only waits for local registration.
     * @param settings.gbids Optional subset of GBIDs configured in the cluster controller for custom global
     * registration.
     *
     * @returns an A+ promise
     */
    register(settings: RegistrationSettingsWithGbids): Promise<string>;
    /**
     * Registers a provider so that it is publicly available in all backends known to the cluster controller
     *
     * @param settings the arguments object for this function call
     * @param settings.domain
     * @param settings.provider
     * @param settings.providerQos the Quality of Service parameters for provider registration
     * @param [settings.expiryDateMs] date in millis since epoch after which the discovery entry can be purged from all directories.
     *            Default value is one day.
     * @param [settings.loggingContext] optional logging context will be appended to logging messages created in the name of this proxy
     * @param [settings.participantId] optional. If not set, a globally unique UUID participantId will be generated, and persisted to
     *            localStorage. If set, the participantId must be unique in the context of the provider's scope, as set in the ProviderQos;
     *            The application setting the participantId is responsible for guaranteeing uniqueness.
     * @param [settings.awaitGlobalRegistration] optional. If provided and set to true registerProvider will wait until local and global
     *            registration succeeds or timeout is reached: otherwise registerProvider only waits for local registration.
     *
     * @returns an A+ promise
     * @deprecated Use `register` instead.
     */
    registerInAllKnownBackends(settings: RegistrationSettings): Promise<string>;
    private registerInternal;
    /**
     * Registers a provider so that it is publicly available
     *
     * If registration to local and global scope is requested by 'providerQos' parameter,
     * the provider is registered to all GBIDs configured in the cluster controller.
     *
     * The 'gbids' parameter can be provided to override the GBIDs selection in the cluster
     * controller. The global capabilities directory identified by the first selected GBID performs
     * the registration.
     *
     * @param domain
     * @param provider
     * @param provider.interfaceName
     * @param providerQos the Quality of Service parameters for provider registration
     * @param [expiryDateMs] date in millis since epoch after which the discovery entry can be purged from all directories.
     *            Default value is one day.
     * @param [loggingContext] optional logging context will be appended to logging messages created in the name of this proxy
     * @param [participantId] optional. If not set, a globally unique UUID participantId will be generated, and persisted to localStorage.
     * @param [awaitGlobalRegistration] optional. If provided and set to true registerProvider will wait until local and global
     *            registration succeeds or timeout is reached: otherwise registerProvider only waits for local registration.
     * @param gbids: Optional subset of GBIDs configured in the cluster controller for custom global
     * registration.
     *
     * @returns an A+ promise
     */
    registerProvider(domain: string, provider: JoynrProvider, providerQos: ProviderQos, expiryDateMs?: number, loggingContext?: Record<string, any>, participantId?: string, awaitGlobalRegistration?: boolean, gbids?: string[]): Promise<string>;
    /**
     * Unregister a provider from the joynr communication framework so that it
     * can no longer be called or discovered.
     *
     * If the provider is registered globally (default), i.e. ProviderScope.GLOBAL
     * has been set in ProviderQos when registering the provider:
     * This function triggers a global remove operation in the local capabilities
     * directory of the cluster controller (standalone or embedded within the same runtime)
     * and returns. It does not wait for a response from global capabilities directory and
     * does not get informed about errors or success. The cluster controller will repeat
     * the global remove operation until it succeeds or the cluster controller is shut down.
     *
     * If the provider is registered only locally, i.e. ProviderScope.LOCAL
     * has been set in ProviderQos when registering the provider:
     * This function removes the provider from the local capabilities directory
     * of the cluster controller (standalone or embedded within the same runtime)
     * and waits for the result.
     *
     * @note: This function does not guarantee a successful execution of provider's removal
     * from the global capabilities directory.
     *
     * @param domain
     * @param provider
     * @param provider.interfaceName
     * @returns an A+ promise
     */
    unregisterProvider(domain: string, provider: JoynrProvider): Promise<void>;
    /**
     * Shutdown the capabilities registrar
     */
    shutdown(): void;
    static setDefaultExpiryIntervalMs(delay: number): void;
}
export = CapabilitiesRegistrar;
