"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
const DiscoveryEntry_1 = __importDefault(require("../../generated/joynr/types/DiscoveryEntry"));
const ProviderScope_1 = __importDefault(require("../../generated/joynr/types/ProviderScope"));
const Version_1 = __importDefault(require("../../generated/joynr/types/Version"));
const LoggingManager_1 = __importDefault(require("../system/LoggingManager"));
const UtilInternal = __importStar(require("../util/UtilInternal"));
const log = LoggingManager_1.default.getLogger("joynr.capabilities.CapabilitiesRegistrar");
let defaultExpiryIntervalMs = 6 * 7 * 24 * 60 * 60 * 1000; // 6 Weeks
class CapabilitiesRegistrar {
    /**
     * The Capabilities Registrar
     *
     * @constructor
     *
     * @param dependencies
     * @param dependencies.discoveryStub connects the inProcessStub to its skeleton
     * @param dependencies.messageRouter used to register nextHop for registered provider
     * @param dependencies.participantIdStorage connects the inProcessStub to its skeleton
     * @param dependencies.libjoynrMessagingAddress address to be used by the cluster controller to send incoming requests to this
     *            libjoynr's providers
     * @param dependencies.requestReplyManager passed on to providerAttribute, providerOperation and providerEvent
     * @param dependencies.publicationManager passed on to providerAttribute
     */
    constructor(dependencies) {
        this.started = true;
        this.discoveryStub = dependencies.discoveryStub;
        this.messageRouter = dependencies.messageRouter;
        this.participantIdStorage = dependencies.participantIdStorage;
        this.libjoynrMessagingAddress = dependencies.libjoynrMessagingAddress;
        this.requestReplyManager = dependencies.requestReplyManager;
        this.publicationManager = dependencies.publicationManager;
    }
    /**
     * Internal function used to throw exception in case of CapabilitiesRegistrar
     * is not used properly
     */
    checkIfReady() {
        if (!this.started) {
            throw new Error("CapabilitiesRegistrar is already shut down");
        }
    }
    /**
     * Registers a provider so that it is publicly available
     *
     * If registration to local and global scope is requested by 'providerQos' parameter,
     * the provider is registered to all GBIDs configured in the cluster controller.
     *
     * The 'gbids' parameter can be provided to override the GBIDs selection in the cluster
     * controller. The global capabilities directory identified by the first selected GBID performs
     * the registration.
     *
     * @param settings the arguments object for this function call
     * @param settings.domain
     * @param settings.provider
     * @param settings.providerQos the Quality of Service parameters for provider registration
     * @param [settings.expiryDateMs] date in millis since epoch after which the discovery entry can be purged from all directories.
     *            Default value is one day.
     * @param [settings.loggingContext] optional logging context will be appended to logging messages created in the name of this proxy
     * @param [settings.participantId] optional. If not set, a globally unique UUID participantId will be generated, and persisted to
     *            localStorage. If set, the participantId must be unique in the context of the provider's scope, as set in the ProviderQos;
     *            The application setting the participantId is responsible for guaranteeing uniqueness.
     * @param [settings.awaitGlobalRegistration] optional. If provided and set to true registerProvider will wait until local and global
     *            registration succeeds or timeout is reached: otherwise registerProvider only waits for local registration.
     * @param settings.gbids Optional subset of GBIDs configured in the cluster controller for custom global
     * registration.
     *
     * @returns an A+ promise
     */
    register(settings) {
        return this.registerInternal(settings, { registerToAllBackends: false, gbids: settings.gbids });
    }
    /**
     * Registers a provider so that it is publicly available in all backends known to the cluster controller
     *
     * @param settings the arguments object for this function call
     * @param settings.domain
     * @param settings.provider
     * @param settings.providerQos the Quality of Service parameters for provider registration
     * @param [settings.expiryDateMs] date in millis since epoch after which the discovery entry can be purged from all directories.
     *            Default value is one day.
     * @param [settings.loggingContext] optional logging context will be appended to logging messages created in the name of this proxy
     * @param [settings.participantId] optional. If not set, a globally unique UUID participantId will be generated, and persisted to
     *            localStorage. If set, the participantId must be unique in the context of the provider's scope, as set in the ProviderQos;
     *            The application setting the participantId is responsible for guaranteeing uniqueness.
     * @param [settings.awaitGlobalRegistration] optional. If provided and set to true registerProvider will wait until local and global
     *            registration succeeds or timeout is reached: otherwise registerProvider only waits for local registration.
     *
     * @returns an A+ promise
     * @deprecated Use `register` instead.
     */
    async registerInAllKnownBackends(settings) {
        return this.registerInternal(settings, { registerToAllBackends: true });
    }
    async registerInternal({ domain, provider, providerQos, expiryDateMs, loggingContext, participantId, awaitGlobalRegistration }, gbIdSettings) {
        this.checkIfReady();
        const missingImplementations = provider.checkImplementation();
        if (missingImplementations.length > 0) {
            throw new Error(`provider: ${domain}/${provider.interfaceName}.v${provider.constructor.MAJOR_VERSION} is missing: ${missingImplementations.toString()}`);
        }
        if (participantId === undefined || participantId === null) {
            // retrieve participantId if not passed in
            participantId = this.participantIdStorage.getParticipantId(domain, provider);
        }
        else {
            // store provided participantId
            this.participantIdStorage.setParticipantId(domain, provider, participantId);
        }
        if (loggingContext !== undefined) {
            log.warn("loggingContext is currently not supported");
        }
        if (awaitGlobalRegistration === undefined) {
            awaitGlobalRegistration = false;
        }
        if (typeof awaitGlobalRegistration !== "boolean") {
            const errText = "awaitGlobalRegistration must be boolean";
            log.warn(errText);
            return Promise.reject(new Error(errText));
        }
        // register provider at RequestReplyManager
        this.requestReplyManager.addRequestCaller(participantId, provider);
        // if provider has at least one attribute, add it as publication provider
        this.publicationManager.addPublicationProvider(participantId, provider);
        // register routing address at routingTable
        const isGloballyVisible = providerQos.scope === ProviderScope_1.default.GLOBAL;
        await this.messageRouter.addNextHop(participantId, this.libjoynrMessagingAddress, isGloballyVisible);
        // TODO: Must be later provided by the user or retrieved from somewhere
        const defaultPublicKeyId = "";
        try {
            const discoveryEntry = new DiscoveryEntry_1.default({
                providerVersion: new Version_1.default({
                    majorVersion: provider.constructor.MAJOR_VERSION,
                    minorVersion: provider.constructor.MINOR_VERSION
                }),
                domain,
                interfaceName: provider.interfaceName,
                participantId,
                qos: providerQos,
                publicKeyId: defaultPublicKeyId,
                expiryDateMs: expiryDateMs || Date.now() + defaultExpiryIntervalMs,
                lastSeenDateMs: Date.now()
            });
            if (gbIdSettings.registerToAllBackends) {
                await this.discoveryStub.addToAll(discoveryEntry, awaitGlobalRegistration);
            }
            else {
                await this.discoveryStub.add(discoveryEntry, awaitGlobalRegistration, gbIdSettings.gbids || []);
            }
        }
        catch (e) {
            this.messageRouter.removeNextHop(participantId).catch(UtilInternal.emptyFunction);
            throw e;
        }
        log.info(`Provider registered: participantId: ${participantId}, domain: ${domain}, interfaceName: ${provider.interfaceName}, majorVersion: ${provider.constructor.MAJOR_VERSION}`);
        return participantId;
    }
    /**
     * Registers a provider so that it is publicly available
     *
     * If registration to local and global scope is requested by 'providerQos' parameter,
     * the provider is registered to all GBIDs configured in the cluster controller.
     *
     * The 'gbids' parameter can be provided to override the GBIDs selection in the cluster
     * controller. The global capabilities directory identified by the first selected GBID performs
     * the registration.
     *
     * @param domain
     * @param provider
     * @param provider.interfaceName
     * @param providerQos the Quality of Service parameters for provider registration
     * @param [expiryDateMs] date in millis since epoch after which the discovery entry can be purged from all directories.
     *            Default value is one day.
     * @param [loggingContext] optional logging context will be appended to logging messages created in the name of this proxy
     * @param [participantId] optional. If not set, a globally unique UUID participantId will be generated, and persisted to localStorage.
     * @param [awaitGlobalRegistration] optional. If provided and set to true registerProvider will wait until local and global
     *            registration succeeds or timeout is reached: otherwise registerProvider only waits for local registration.
     * @param gbids: Optional subset of GBIDs configured in the cluster controller for custom global
     * registration.
     *
     * @returns an A+ promise
     */
    async registerProvider(domain, provider, providerQos, expiryDateMs, loggingContext, participantId, awaitGlobalRegistration, gbids) {
        return this.registerInternal({
            domain,
            provider,
            providerQos,
            expiryDateMs,
            loggingContext,
            participantId,
            awaitGlobalRegistration
        }, { registerToAllBackends: false, gbids });
    }
    /**
     * Unregister a provider from the joynr communication framework so that it
     * can no longer be called or discovered.
     *
     * If the provider is registered globally (default), i.e. ProviderScope.GLOBAL
     * has been set in ProviderQos when registering the provider:
     * This function triggers a global remove operation in the local capabilities
     * directory of the cluster controller (standalone or embedded within the same runtime)
     * and returns. It does not wait for a response from global capabilities directory and
     * does not get informed about errors or success. The cluster controller will repeat
     * the global remove operation until it succeeds or the cluster controller is shut down.
     *
     * If the provider is registered only locally, i.e. ProviderScope.LOCAL
     * has been set in ProviderQos when registering the provider:
     * This function removes the provider from the local capabilities directory
     * of the cluster controller (standalone or embedded within the same runtime)
     * and waits for the result.
     *
     * @note: This function does not guarantee a successful execution of provider's removal
     * from the global capabilities directory.
     *
     * @param domain
     * @param provider
     * @param provider.interfaceName
     * @returns an A+ promise
     */
    async unregisterProvider(domain, provider) {
        this.checkIfReady();
        // retrieve participantId
        const participantId = this.participantIdStorage.getParticipantId(domain, provider);
        await this.discoveryStub.remove(participantId);
        // unregister routing address at routingTable
        await this.messageRouter.removeNextHop(participantId);
        // if provider has at least one attribute, remove it as publication
        // provider
        this.publicationManager.removePublicationProvider(participantId, provider);
        // unregister provider at RequestReplyManager
        this.requestReplyManager.removeRequestCaller(participantId);
        log.info(`Provider unregistered: participantId: ${participantId}, domain: ${domain}, interfaceName: ${provider.interfaceName}, majorVersion: ${provider.constructor.MAJOR_VERSION}`);
    }
    /**
     * Shutdown the capabilities registrar
     */
    shutdown() {
        this.started = false;
    }
    static setDefaultExpiryIntervalMs(delay) {
        defaultExpiryIntervalMs = delay;
    }
}
module.exports = CapabilitiesRegistrar;
