import * as DiscoveryEntry from "../../generated/joynr/types/DiscoveryEntry";
declare class CapabilitiesStore {
    private registeredCapabilitiesTime;
    private discoveryEntryStore;
    private discoveryEntryStoreByDomainInterfaceName;
    private discoveryEntryStoreByParticipantId;
    /**
     * The Local Capabilities Storage
     *
     * @constructor
     *
     * @classdesc
     * The <code>CapabilitiesStore</code> is a joynr internal interface. When a provider that is registered with joynr, the framework creates an
     * entry for that provider in the capabilities directory. These <code>{@link DiscoveryEntry}</code> entries contain access
     * information as well as supported Qos. The information is later used in the arbitration process to pick a provider for a proxy.
     *
     * @param initialCapabilities
     */
    constructor(initialCapabilities?: DiscoveryEntry[]);
    /**
     * Unregisters a discovery entry from the store.
     *
     * @param participantId - the participant ID uniquely identifying the discovery entry to be removed
     */
    private removeDiscoveryEntryFromStore;
    /**
     * Registers discovery entry to the store.
     *
     * @param discoveryEntry - the capability to register
     */
    private addDiscoveryEntryToStore;
    /**
     * Registers either a single discovery entry or a list of discovery entry to the store.
     *
     * @param settings - an object containing the required parameters
     * @param settings.discoveryEntries - an array of discovery entries to register
     * @param settings.discoveryEntry a single discovery entry to register (only if settings.discoveryEntries is not supplied)
     * @param settings.remote true if the provided entry/entries are gathered from remote
     *
     * @returns returns this for a fluent interface
     */
    add(settings: {
        discoveryEntries?: DiscoveryEntry[];
        discoveryEntry?: DiscoveryEntry;
        remote?: boolean;
    }): CapabilitiesStore;
    /**
     * checks if the provided discoveryEntry is not older than the second argument maxAge
     *
     * @param discoveryEntry - the discovery entry to check
     * @param maxAge - the maximum age of the discovery entry
     */
    private checkAge;
    /**
     * checks if the discoveryQos matches with the discoveryEntry
     *
     * @param discoveryEntry - the discovery entry to check
     * @param cacheMaxAge - the maximum age of the discovery entry
     */
    private qosMatches;
    /**
     * filters provided entries and returns only local entries
     *
     * @param entries - the discovery entries to be filtered
     * @param cacheMaxAge - the maximum age of the discovery entries
     */
    private filterEntries;
    /**
     * Looks up a list of capabilities either for a given participant ID or for a given domain / interfaceName combination
     *
     * @param settings - an object containing the required parameters
     * @param settings.domains - the name of the domains
     * @param settings.interfaceName - the interface name of the capability
     * @param settings.participantId - the participantId of the capability
     * @param settings.cacheMaxAge - max age used for filtering the store entries
     *
     * @returns a list of matching discovery entries.
     */
    lookup(settings: {
        domains: string[];
        interfaceName: string;
        cacheMaxAge?: number;
    }): DiscoveryEntry[];
    lookup(settings: {
        participantId: string;
        cacheMaxAge?: number;
    }): DiscoveryEntry[];
    /**
     * Unregisters either a list of discovery entries or a single discovery entry from the store.
     *
     * @param settings - an object containing the required parameters
     * @param settings.participantIds - an array of participant IDs uniquely identifying the discovery entry to be removed
     * @param settings.participantId -  a participant ID uniquely identifying the discovery entry to be removed
     *
     * @returns returns this for a fluent interface
     */
    remove(settings: {
        participantIds?: string[];
        participantId?: string;
    }): CapabilitiesStore;
}
export = CapabilitiesStore;
