import * as Address from "../../../generated/joynr/system/RoutingTypes/Address";
import * as DiscoveryEntry from "../../../generated/joynr/types/DiscoveryEntry";
import * as DiscoveryEntryWithMetaInfo from "../../../generated/joynr/types/DiscoveryEntryWithMetaInfo";
import DiscoveryQosGen from "../../../generated/joynr/types/DiscoveryQos";
import { DiscoveryStub } from "../interface/DiscoveryStub";
import ProxyBuilder = require("../../proxy/ProxyBuilder");
import MessageRouter = require("../../messaging/routing/MessageRouter");
import CapabilitiesStore = require("../CapabilitiesStore");
declare class CapabilityDiscovery implements DiscoveryStub {
    private globalCapabilitiesDomain;
    private proxyBuilder;
    private messageRouter;
    private globalCapabilitiesCache;
    private localCapabilitiesStore;
    private queuedGlobalLookups;
    private queuedGlobalLookupsByParticipantId;
    private queuedGlobalDiscoveryEntries;
    private globalAddressSerialized?;
    private knownGbids;
    /**
     * The CapabilitiesDiscovery looks up the local and global capabilities directory
     *
     * @constructor
     *
     * @param localCapabilitiesStore the local capabilities store
     * @param globalCapabilitiesCache the cache for the global capabilities directory
     * @param globalCapabilitiesDomain the domain to communicate with the GlobalCapablitiesDirectory
     *                                     GlobalCapab
     * @param knownGbids known global backend identifiers provided by provisioning
     */
    constructor(localCapabilitiesStore: CapabilitiesStore, globalCapabilitiesCache: CapabilitiesStore, globalCapabilitiesDomain: string, knownGbids: string[]);
    /**
     * Set necessary dependencies
     *
     * @param messageRouter the message router
     * @param proxyBuilder the proxy builder used to create the GlobalCapabilitiesDirectoryProxy
     */
    setDependencies(messageRouter: MessageRouter, proxyBuilder: ProxyBuilder): void;
    /**
     * This method create a new global capabilities proxy with the provided ttl as messaging QoS
     *
     * @param ttl time to live of joynr messages triggered by the returning proxy
     *
     * @returns the newly created proxy
     */
    private getGlobalCapabilitiesDirectoryProxy;
    private lookupGlobal;
    /**
     * expects a capabilities array which is then filled with any that are found from the proxy
     *
     * @param domains - the domains
     * @param interfaceName - the interface name
     * @param ttl - time to live of joynr messages triggered by the returning proxy
     * @param capabilities - the capabilities array to be filled
     * @param gbids - array of knownGbids in which to look for providers
     *
     * @returns - the capabilities array filled with the capabilities found in the global capabilities directory
     */
    private lookupGlobalCapabilities;
    private lookupGlobalByParticipantId;
    private lookupGlobalCapabilitiesByParticipantId;
    /**
     * @param discoveryEntry to be added to the global discovery directory
     * @param gbids global backend identifiers of backends to add discoveryEntries into
     * @returns an A+ promise
     */
    private addGlobal;
    /**
     * @param queuedDiscoveryEntry contains a discoveryEntry and the resolve
     * and reject functions from the original Promise created on add().
     */
    private addGlobalQueued;
    /**
     * This method removes a capability from the global capabilities directory.
     *
     * @param participantId to remove
     *
     * @returns an A+ promise
     */
    private removeParticipantIdFromGlobalCapabilitiesDirectory;
    /**
     * This method is called when the global address has been created
     *
     * @param newGlobalAddress the address used to register discovery entries globally
     */
    globalAddressReady(newGlobalAddress: Address): void;
    /**
     * This method queries the local and/or global capabilities directory according to the given discoveryStrategy given in the
     * DiscoveryQos object
     *
     * @param domains the domains
     * @param interfaceName the interface name
     * @param discoveryQos the DiscoveryQos giving the strategy for discovering a capability
     * @param gbids the backends in which the lookup shall be performed
     * @returns an A+ Promise object, that will provide an array of discovered capabilities, callback signatures:
     *          then({Array[GlobalDiscoveryEntry]} discoveredCaps).catch({Error} error)
     */
    lookup(domains: string[], interfaceName: string, discoveryQos: DiscoveryQosGen, gbids: string[]): Promise<DiscoveryEntryWithMetaInfo[]>;
    /**
     * This method queries the local and/or global capabilities directory according to the given discoveryStrategy given in the
     * DiscoveryQos object
     *
     * @param participantId the participantId
     * @param discoveryQos the DiscoveryQos giving the strategy for discovering a capability
     * @param gbids the backends in which the lookup shall be performed
     * @returns an A+ Promise object, that will one single discovered capabilities, callback signatures:
     *          then({GlobalDiscoveryEntry} discoveredCap).catch({Error} error)
     */
    lookupByParticipantId(participantId: string, discoveryQos: DiscoveryQosGen, gbids: string[]): Promise<DiscoveryEntryWithMetaInfo>;
    /**
     * This method adds a capability in the local and/or global capabilities directory according to
     * the given registration strategy. Global capabilities will be added to all known gbids.
     *
     * @param discoveryEntry DiscoveryEntry to add to all known backends
     * @returns Promise resolved on success.
     */
    addToAll(discoveryEntry: DiscoveryEntry): Promise<void>;
    private addInternal;
    /**
     * This method adds a capability in the local and/or global capabilities directory according to the given registration
     * strategy.
     *
     * @param discoveryEntry DiscoveryEntry of the capability
     * @param _awaitGlobalRegistration webSocket-libjoynr specific property part of the DiscoveryStub interface
     * @param gbids identifiers of global backends to add capability into
     *
     * @returns an A+ promise
     */
    add(discoveryEntry: DiscoveryEntry, _awaitGlobalRegistration?: boolean, gbids?: string[]): Promise<void>;
    /**
     * This method sends a freshness update to the global capabilities directory to update
     * the lastSeenDateMs of the entries registered via the local cluster controller.
     *
     * @param clusterControllerId the channelId of the local cluster controller
     * @param ttlMs the time to live of the freshness update message
     *
     * @returns an A+ promise
     */
    touch(clusterControllerId: string, ttlMs: number): Promise<void>;
    /**
     * This method removes a capability from the local and/or global capabilities directory according to the given registration
     * strategy.
     *
     * @param participantId to remove
     *
     * @returns an A+ promise
     */
    remove(participantId: string): Promise<void>;
}
export = CapabilityDiscovery;
