import * as DiscoveryEntryWithMetaInfo from "../../generated/joynr/types/DiscoveryEntryWithMetaInfo";
import * as MessagingQos from "../messaging/MessagingQos";
import * as Request from "./types/Request";
import * as OneWayRequest from "./types/OneWayRequest";
import BroadcastSubscriptionRequest from "./types/BroadcastSubscriptionRequest";
import MulticastSubscriptionRequest from "./types/MulticastSubscriptionRequest";
import SubscriptionRequest from "./types/SubscriptionRequest";
import SubscriptionStop from "./types/SubscriptionStop";
import * as SubscriptionPublication from "./types/SubscriptionPublication";
import JoynrMessage from "../messaging/JoynrMessage";
import PlatformSecurityManagerNode = require("../security/PlatformSecurityManagerNode");
import InProcessMessagingStub = require("../messaging/inprocess/InProcessMessagingStub");
import MessageRouter = require("../messaging/routing/MessageRouter");
import RequestReplyManager = require("./RequestReplyManager");
import SubscriptionManager = require("./subscription/SubscriptionManager");
import PublicationManager = require("./subscription/PublicationManager");
declare class Dispatcher {
    private ttlUpLiftMs?;
    private securityManager;
    private clusterControllerMessagingStub;
    private messageRouter;
    private publicationManager;
    private subscriptionManager;
    private requestReplyManager;
    /**
     * @constructor
     *
     * @param clusterControllerMessagingStub for sending outgoing joynr messages
     * @param securityManager for setting the creator user ID header
     * @param ttlUpLiftMs
     */
    constructor(clusterControllerMessagingStub: InProcessMessagingStub, securityManager: PlatformSecurityManagerNode, ttlUpLiftMs?: number);
    /**
     * @param expiryDate the expiry date in milliseconds
     * @returns the expiry date with TTL_UPLIFT added as time delta
     */
    private upLiftTtl;
    /**
     * @param subscriptionRequest the subscription request
     * @returns the subscription request with qos.expiry date with TTL_UPLIFT added as time delta
     */
    private upLiftExpiryDateInSubscriptionRequest;
    /**
     * @param joynrMessage
     * @param settings
     * @param settings.from participantId of the sender
     * @param settings.toDiscoveryEntry DiscoveryEntry of the receiver
     * @param settings.messagingQos the messaging Qos object for the ttl
     * @returns Promise resolved on success
     */
    private sendJoynrMessage;
    private getJoynrMessageType;
    /**
     * @param settings
     * @param settings.from participantId of the sender
     * @param settings.to participantId of the receiver
     * @param settings.expiryDate time-to-live
     * @param settings.customHeaders custom headers from request
     * @param settings.messageType
     * @param settings.effort
     * @param settings.compress
     * @param settings.reply the reply to be transmitted. It can either be a Reply or a SubscriptionReply object
     * @returns A+ promise object
     */
    private sendReply;
    /**
     * @param settings
     * @param settings.from participantId of the sender
     * @param settings.to participantId of the receiver
     * @param settings.expiryDate time-to-live
     * @param settings.customHeaders custom headers from request
     * @param settings.messageType
     * @param settings.effort
     * @param settings.compress
     * @param reply
     * @returns A+ promise object
     */
    private sendRequestReply;
    /**
     * @param settings
     * @param settings.from participantId of the sender
     * @param settings.to participantId of the receiver
     * @param settings.expiryDate time-to-live
     * @param settings.customHeaders custom headers from request
     * @param subscriptionReply
     */
    private sendSubscriptionReply;
    private sendPublicationInternal;
    private createReplySettings;
    /**
     * @param newRequestReplyManager handles incoming and outgoing requests and replies
     */
    registerRequestReplyManager(newRequestReplyManager: RequestReplyManager): void;
    /**
     * @param newMessageRouter
     */
    registerMessageRouter(newMessageRouter: MessageRouter): void;
    /**
     * @param newSubscriptionManager sends subscription requests; handles incoming publications
     * and incoming replies to subscription requests
     */
    registerSubscriptionManager(newSubscriptionManager: SubscriptionManager): void;
    /**
     * @param newPublicationManager sends publications; handles incoming subscription start and stop requests
     */
    registerPublicationManager(newPublicationManager: PublicationManager): void;
    /**
     * @param joynrMessage
     * @param requestReplyId
     */
    private addRequestReplyIdCustomHeader;
    /**
     * @param settings
     * @param settings.from participantId of the sender
     * @param settings.toDiscoveryEntry DiscoveryEntry of the receiver
     * @param settings.messagingQos the messaging Qos object for the ttl
     * @param settings.request
     * @returns A+ promise object
     */
    sendRequest(settings: {
        from: string;
        toDiscoveryEntry: DiscoveryEntryWithMetaInfo;
        messagingQos: MessagingQos;
        request: Request.Request;
    }): Promise<void>;
    /**
     * @param settings
     * @param settings.from participantId of the sender
     * @param settings.toDiscoveryEntry DiscoveryEntry of the receiver
     * @param settings.messagingQos the messaging Qos object for the ttl
     * @param settings.request
     * @returns A+ promise object
     */
    sendOneWayRequest(settings: {
        from: string;
        toDiscoveryEntry: DiscoveryEntryWithMetaInfo;
        messagingQos: MessagingQos;
        request: OneWayRequest.OneWayRequest;
    }): Promise<void>;
    /**
     * @param settings
     * @param settings.from participantId of the sender
     * @param settings.toDiscoveryEntry DiscoveryEntry of the receiver
     * @param settings.messagingQos the messaging Qos object for the ttl
     * @param settings.subscriptionRequest
     * @returns  promise object that is resolved when the request is sent by the messaging stub
     */
    sendSubscriptionRequest(settings: {
        from: string;
        toDiscoveryEntry: DiscoveryEntryWithMetaInfo;
        messagingQos: MessagingQos;
        subscriptionRequest: SubscriptionRequest;
    }): Promise<void>;
    /**
     * @param settings
     * @param settings.from participantId of the sender
     * @param settings.toDiscoveryEntry DiscoveryEntry of the receiver
     * @param settings.messagingQos the messaging Qos object for the ttl
     * @param settings.subscriptionRequest
     * @returns  promise object that is resolved when the request is sent by the messaging stub
     */
    sendBroadcastSubscriptionRequest(settings: {
        from: string;
        toDiscoveryEntry: DiscoveryEntryWithMetaInfo;
        messagingQos: MessagingQos;
        subscriptionRequest: MulticastSubscriptionRequest | BroadcastSubscriptionRequest;
    }): Promise<void>;
    /**
     * @param settings
     * @param settings.from participantId of the sender
     * @param settings.toDiscoveryEntry DiscoveryEntry of the receiver
     * @param settings.multicastId of the multicast
     * @param settings.subscriptionStop
     * @param settings.messagingQos the messaging Qos object for the ttl
     * @returns A+ promise object
     */
    sendMulticastSubscriptionStop(settings: {
        from: string;
        toDiscoveryEntry: DiscoveryEntryWithMetaInfo;
        multicastId: string;
        subscriptionStop: SubscriptionStop;
        messagingQos: MessagingQos;
    }): Promise<void>;
    /**
     * @param settings
     * @param settings.from participantId of the sender
     * @param settings.toDiscoveryEntry DiscoveryEntry of the receiver
     * @param settings.subscriptionStop
     * @param settings.messagingQos the messaging Qos object for the ttl
     * @returns A+ promise object
     */
    sendSubscriptionStop(settings: {
        from: string;
        toDiscoveryEntry: DiscoveryEntryWithMetaInfo;
        subscriptionStop: SubscriptionStop;
        messagingQos: MessagingQos;
    }): Promise<void>;
    /**
     * @param settings
     * @param settings.from participantId of the sender
     * @param settings.to participantId of the receiver
     * @param settings.expiryDate time-to-live
     * @param publication
     * @param [publication.response]
     * @param [publication.error]
     * @param publication.subscriptionId
     */
    sendPublication(settings: {
        from: string;
        to: string;
        expiryDate: number;
    }, publication: SubscriptionPublication.SubscriptionPublication): void;
    /**
     * @param settings
     * @param settings.from participantId of the sender
     * @param settings.multicastName name of multicast
     * @param {String[]} partitions partitions for this multicast
     * @param settings.expiryDate time-to-live
     * @param publication
     * @param [publication.response]
     * @param [publication.error]
     * @param publication.multicastId
     */
    sendMulticastPublication(settings: {
        from: string;
        multicastName?: string;
        expiryDate: number;
    }, publication: {
        response?: any;
        error?: any;
        multicastId: string;
    }): void;
    /**
     * receives a new JoynrMessage that has to be routed to one of the managers
     *
     * @param joynrMessage being routed
     */
    receive(joynrMessage: JoynrMessage): Promise<void>;
    /**
     * Shutdown the dispatcher
     */
    shutdown(): void;
}
export = Dispatcher;
