"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
const Request = __importStar(require("./types/Request"));
const Reply = __importStar(require("./types/Reply"));
const OneWayRequest = __importStar(require("./types/OneWayRequest"));
const BroadcastSubscriptionRequest_1 = __importDefault(require("./types/BroadcastSubscriptionRequest"));
const MulticastSubscriptionRequest_1 = __importDefault(require("./types/MulticastSubscriptionRequest"));
const SubscriptionRequest_1 = __importDefault(require("./types/SubscriptionRequest"));
const SubscriptionReply_1 = __importDefault(require("./types/SubscriptionReply"));
const SubscriptionStop_1 = __importDefault(require("./types/SubscriptionStop"));
const SubscriptionPublication = __importStar(require("./types/SubscriptionPublication"));
const MulticastPublication = __importStar(require("./types/MulticastPublication"));
const JoynrMessage_1 = __importDefault(require("../messaging/JoynrMessage"));
const MessagingQosEffort = __importStar(require("../messaging/MessagingQosEffort"));
const defaultMessagingSettings_1 = __importDefault(require("../start/settings/defaultMessagingSettings"));
const DiagnosticTags = __importStar(require("../system/DiagnosticTags"));
const UtilInternal = __importStar(require("../util/UtilInternal"));
const JSONSerializer = __importStar(require("../util/JSONSerializer"));
const Typing = __importStar(require("../util/Typing"));
const SubscriptionQos_1 = __importDefault(require("../proxy/SubscriptionQos"));
const LoggingManager_1 = __importDefault(require("../system/LoggingManager"));
const log = LoggingManager_1.default.getLogger("joynr.dispatching.Dispatcher");
class Dispatcher {
    /**
     * @constructor
     *
     * @param clusterControllerMessagingStub for sending outgoing joynr messages
     * @param securityManager for setting the creator user ID header
     * @param ttlUpLiftMs
     */
    constructor(clusterControllerMessagingStub, securityManager, ttlUpLiftMs) {
        this.clusterControllerMessagingStub = clusterControllerMessagingStub;
        this.securityManager = securityManager;
        this.ttlUpLiftMs = ttlUpLiftMs;
        this.sendSubscriptionReply = this.sendSubscriptionReply.bind(this);
        // bind due to InProcessMessagingSkeleton using receive as callback.
        this.receive = this.receive.bind(this);
    }
    /**
     * @param expiryDate the expiry date in milliseconds
     * @returns the expiry date with TTL_UPLIFT added as time delta
     */
    upLiftTtl(expiryDate) {
        expiryDate += this.ttlUpLiftMs !== undefined ? this.ttlUpLiftMs : defaultMessagingSettings_1.default.TTL_UPLIFT;
        if (expiryDate > UtilInternal.getMaxLongValue()) {
            expiryDate = UtilInternal.getMaxLongValue();
        }
        return expiryDate;
    }
    /**
     * @param subscriptionRequest the subscription request
     * @returns the subscription request with qos.expiry date with TTL_UPLIFT added as time delta
     */
    upLiftExpiryDateInSubscriptionRequest(subscriptionRequest) {
        // if expiryDateMs == SubscriptionQos.NO_EXPIRY_DATE (=0), expiryDateMs must not be changed
        if (subscriptionRequest.qos.expiryDateMs !== SubscriptionQos_1.default.NO_EXPIRY_DATE) {
            subscriptionRequest.qos.expiryDateMs = this.upLiftTtl(subscriptionRequest.qos.expiryDateMs);
        }
        return subscriptionRequest;
    }
    /**
     * @param joynrMessage
     * @param settings
     * @param settings.from participantId of the sender
     * @param settings.toDiscoveryEntry DiscoveryEntry of the receiver
     * @param settings.messagingQos the messaging Qos object for the ttl
     * @returns Promise resolved on success
     */
    sendJoynrMessage(joynrMessage, settings) {
        // set headers
        joynrMessage.creator = this.securityManager.getCurrentProcessUserId();
        joynrMessage.from = settings.from;
        joynrMessage.to = settings.toDiscoveryEntry.participantId;
        joynrMessage.expiryDate = this.upLiftTtl(Date.now() + settings.messagingQos.ttl);
        const effort = settings.messagingQos.effort;
        if (effort !== MessagingQosEffort.NORMAL) {
            joynrMessage.effort = effort.value;
        }
        if (settings.messagingQos.compress) {
            joynrMessage.compress = true;
        }
        joynrMessage.isLocalMessage = settings.toDiscoveryEntry.isLocal;
        if (log.isDebugEnabled()) {
            log.debug(`sendJoynrMessage, message = ${JSON.stringify(joynrMessage)}`);
        }
        // send message
        return this.clusterControllerMessagingStub.transmit(joynrMessage);
    }
    getJoynrMessageType(subscriptionRequest) {
        const type = Typing.getObjectType(subscriptionRequest);
        switch (type) {
            case "BroadcastSubscriptionRequest":
                return JoynrMessage_1.default.JOYNRMESSAGE_TYPE_BROADCAST_SUBSCRIPTION_REQUEST;
            case "MulticastSubscriptionRequest":
                return JoynrMessage_1.default.JOYNRMESSAGE_TYPE_MULTICAST_SUBSCRIPTION_REQUEST;
            default:
                return JoynrMessage_1.default.JOYNRMESSAGE_TYPE_SUBSCRIPTION_REQUEST;
        }
    }
    /**
     * @param settings
     * @param settings.from participantId of the sender
     * @param settings.to participantId of the receiver
     * @param settings.expiryDate time-to-live
     * @param settings.customHeaders custom headers from request
     * @param settings.messageType
     * @param settings.effort
     * @param settings.compress
     * @param settings.reply the reply to be transmitted. It can either be a Reply or a SubscriptionReply object
     * @returns A+ promise object
     */
    sendReply(settings) {
        // reply with the result in a JoynrMessage
        const joynrMessage = new JoynrMessage_1.default({
            type: settings.messageType,
            payload: settings.reply
        });
        joynrMessage.from = settings.from;
        joynrMessage.to = settings.to;
        joynrMessage.expiryDate = settings.expiryDate;
        // set custom headers
        joynrMessage.setCustomHeaders(settings.customHeaders);
        if (settings.effort && settings.effort !== MessagingQosEffort.NORMAL) {
            joynrMessage.effort = settings.effort.value;
        }
        if (settings.compress) {
            joynrMessage.compress = true;
        }
        if (log.isDebugEnabled()) {
            log.debug(`sendReply, message = ${JSON.stringify(joynrMessage)}`);
        }
        return this.clusterControllerMessagingStub.transmit(joynrMessage);
    }
    /**
     * @param settings
     * @param settings.from participantId of the sender
     * @param settings.to participantId of the receiver
     * @param settings.expiryDate time-to-live
     * @param settings.customHeaders custom headers from request
     * @param settings.messageType
     * @param settings.effort
     * @param settings.compress
     * @param reply
     * @returns A+ promise object
     */
    sendRequestReply(settings, reply) {
        const toParticipantId = settings.to;
        log.info("replying", DiagnosticTags.forReply({
            reply,
            to: toParticipantId,
            from: settings.from
        }));
        settings.reply = JSONSerializer.stringifyOptional(reply, reply.error !== undefined);
        settings.messageType = JoynrMessage_1.default.JOYNRMESSAGE_TYPE_REPLY;
        return this.sendReply(settings);
    }
    /**
     * @param settings
     * @param settings.from participantId of the sender
     * @param settings.to participantId of the receiver
     * @param settings.expiryDate time-to-live
     * @param settings.customHeaders custom headers from request
     * @param subscriptionReply
     */
    sendSubscriptionReply(settings, subscriptionReply) {
        const toParticipantId = settings.to;
        log.info("replying", DiagnosticTags.forSubscriptionReply({
            subscriptionReply,
            to: toParticipantId,
            from: settings.from
        }));
        settings.reply = JSONSerializer.stringifyOptional(subscriptionReply, subscriptionReply.error !== undefined);
        settings.messageType = JoynrMessage_1.default.JOYNRMESSAGE_TYPE_SUBSCRIPTION_REPLY;
        this.sendReply(settings);
    }
    sendPublicationInternal(settings, type, publication) {
        // create JoynrMessage for the publication
        const publicationMessage = new JoynrMessage_1.default({
            type,
            payload: JSONSerializer.stringify(publication)
        });
        // set reply headers
        const toParticipantId = settings.to;
        publicationMessage.from = settings.from;
        publicationMessage.to = toParticipantId;
        publicationMessage.expiryDate = this.upLiftTtl(settings.expiryDate);
        this.addRequestReplyIdCustomHeader(publicationMessage, publication.subscriptionId);
        if (log.isDebugEnabled()) {
            log.debug(`sendPublicationInternal, message = ${JSON.stringify(publicationMessage)}`);
        }
        this.clusterControllerMessagingStub.transmit(publicationMessage);
    }
    createReplySettings(joynrMessage) {
        return {
            from: joynrMessage.to,
            to: joynrMessage.from,
            expiryDate: joynrMessage.expiryDate,
            customHeaders: joynrMessage.getCustomHeaders()
        };
    }
    /**
     * @param newRequestReplyManager handles incoming and outgoing requests and replies
     */
    registerRequestReplyManager(newRequestReplyManager) {
        this.requestReplyManager = newRequestReplyManager;
    }
    /**
     * @param newMessageRouter
     */
    registerMessageRouter(newMessageRouter) {
        this.messageRouter = newMessageRouter;
    }
    /**
     * @param newSubscriptionManager sends subscription requests; handles incoming publications
     * and incoming replies to subscription requests
     */
    registerSubscriptionManager(newSubscriptionManager) {
        this.subscriptionManager = newSubscriptionManager;
    }
    /**
     * @param newPublicationManager sends publications; handles incoming subscription start and stop requests
     */
    registerPublicationManager(newPublicationManager) {
        this.publicationManager = newPublicationManager;
    }
    /**
     * @param joynrMessage
     * @param requestReplyId
     */
    addRequestReplyIdCustomHeader(joynrMessage, requestReplyId) {
        joynrMessage.setCustomHeaders({ z4: requestReplyId });
    }
    /**
     * @param settings
     * @param settings.from participantId of the sender
     * @param settings.toDiscoveryEntry DiscoveryEntry of the receiver
     * @param settings.messagingQos the messaging Qos object for the ttl
     * @param settings.request
     * @returns A+ promise object
     */
    sendRequest(settings) {
        // Create a JoynrMessage with the Request
        const requestMessage = new JoynrMessage_1.default({
            type: JoynrMessage_1.default.JOYNRMESSAGE_TYPE_REQUEST,
            payload: JSONSerializer.stringify(settings.request)
        });
        if (settings.messagingQos.customHeaders) {
            requestMessage.setCustomHeaders(settings.messagingQos.customHeaders);
        }
        this.addRequestReplyIdCustomHeader(requestMessage, settings.request.requestReplyId);
        log.info(`calling ${settings.request.methodName}.`, DiagnosticTags.forRequest({
            request: settings.request,
            to: settings.toDiscoveryEntry.participantId,
            from: settings.from
        }));
        return this.sendJoynrMessage(requestMessage, settings);
    }
    /**
     * @param settings
     * @param settings.from participantId of the sender
     * @param settings.toDiscoveryEntry DiscoveryEntry of the receiver
     * @param settings.messagingQos the messaging Qos object for the ttl
     * @param settings.request
     * @returns A+ promise object
     */
    sendOneWayRequest(settings) {
        // Create a JoynrMessage with the OneWayRequest
        const oneWayRequestMessage = new JoynrMessage_1.default({
            type: JoynrMessage_1.default.JOYNRMESSAGE_TYPE_ONE_WAY,
            payload: JSONSerializer.stringify(settings.request)
        });
        if (settings.messagingQos.customHeaders) {
            oneWayRequestMessage.setCustomHeaders(settings.messagingQos.customHeaders);
        }
        log.info(`calling ${settings.request.methodName}.`, DiagnosticTags.forOneWayRequest({
            request: settings.request,
            to: settings.toDiscoveryEntry.participantId,
            from: settings.from
        }));
        return this.sendJoynrMessage(oneWayRequestMessage, settings);
    }
    /**
     * @param settings
     * @param settings.from participantId of the sender
     * @param settings.toDiscoveryEntry DiscoveryEntry of the receiver
     * @param settings.messagingQos the messaging Qos object for the ttl
     * @param settings.subscriptionRequest
     * @returns  promise object that is resolved when the request is sent by the messaging stub
     */
    sendSubscriptionRequest(settings) {
        log.info(`subscription to ${settings.subscriptionRequest.subscribedToName}`, DiagnosticTags.forSubscriptionRequest({
            subscriptionRequest: settings.subscriptionRequest,
            to: settings.toDiscoveryEntry.participantId,
            from: settings.from
        }));
        const requestMessage = new JoynrMessage_1.default({
            type: JoynrMessage_1.default.JOYNRMESSAGE_TYPE_SUBSCRIPTION_REQUEST,
            payload: JSONSerializer.stringify(settings.subscriptionRequest)
        });
        if (settings.messagingQos.customHeaders) {
            requestMessage.setCustomHeaders(settings.messagingQos.customHeaders);
        }
        this.addRequestReplyIdCustomHeader(requestMessage, settings.subscriptionRequest.subscriptionId);
        return this.sendJoynrMessage(requestMessage, settings);
    }
    /**
     * @param settings
     * @param settings.from participantId of the sender
     * @param settings.toDiscoveryEntry DiscoveryEntry of the receiver
     * @param settings.messagingQos the messaging Qos object for the ttl
     * @param settings.subscriptionRequest
     * @returns  promise object that is resolved when the request is sent by the messaging stub
     */
    sendBroadcastSubscriptionRequest(settings) {
        const type = this.getJoynrMessageType(settings.subscriptionRequest);
        const requestMessage = new JoynrMessage_1.default({
            type,
            payload: JSONSerializer.stringify(settings.subscriptionRequest)
        });
        if (type === JoynrMessage_1.default.JOYNRMESSAGE_TYPE_MULTICAST_SUBSCRIPTION_REQUEST) {
            log.info(`multicast subscription to ${settings.subscriptionRequest.subscribedToName}`, DiagnosticTags.forMulticastSubscriptionRequest({
                subscriptionRequest: settings.subscriptionRequest,
                to: settings.toDiscoveryEntry.participantId,
                from: settings.from
            }));
            return this.messageRouter.addMulticastReceiver({
                multicastId: settings.subscriptionRequest.multicastId,
                subscriberParticipantId: settings.from,
                providerParticipantId: settings.toDiscoveryEntry.participantId
            });
        }
        log.info(`broadcast subscription to ${settings.subscriptionRequest.subscribedToName}`, DiagnosticTags.forBroadcastSubscriptionRequest({
            subscriptionRequest: settings.subscriptionRequest,
            to: settings.toDiscoveryEntry.participantId,
            from: settings.from
        }));
        if (settings.messagingQos.customHeaders) {
            requestMessage.setCustomHeaders(settings.messagingQos.customHeaders);
        }
        this.addRequestReplyIdCustomHeader(requestMessage, settings.subscriptionRequest.subscriptionId);
        return this.sendJoynrMessage(requestMessage, settings);
    }
    /**
     * @param settings
     * @param settings.from participantId of the sender
     * @param settings.toDiscoveryEntry DiscoveryEntry of the receiver
     * @param settings.multicastId of the multicast
     * @param settings.subscriptionStop
     * @param settings.messagingQos the messaging Qos object for the ttl
     * @returns A+ promise object
     */
    sendMulticastSubscriptionStop(settings) {
        this.sendSubscriptionStop(settings);
        return this.messageRouter.removeMulticastReceiver({
            multicastId: settings.multicastId,
            subscriberParticipantId: settings.from,
            providerParticipantId: settings.toDiscoveryEntry.participantId
        });
    }
    /**
     * @param settings
     * @param settings.from participantId of the sender
     * @param settings.toDiscoveryEntry DiscoveryEntry of the receiver
     * @param settings.subscriptionStop
     * @param settings.messagingQos the messaging Qos object for the ttl
     * @returns A+ promise object
     */
    sendSubscriptionStop(settings) {
        log.info(`subscription stop ${settings.subscriptionStop.subscriptionId}`, DiagnosticTags.forSubscriptionStop({
            subscriptionId: settings.subscriptionStop.subscriptionId,
            to: settings.toDiscoveryEntry.participantId,
            from: settings.from
        }));
        const message = new JoynrMessage_1.default({
            type: JoynrMessage_1.default.JOYNRMESSAGE_TYPE_SUBSCRIPTION_STOP,
            payload: JSONSerializer.stringify(settings.subscriptionStop)
        });
        if (settings.messagingQos.customHeaders) {
            message.setCustomHeaders(settings.messagingQos.customHeaders);
        }
        this.addRequestReplyIdCustomHeader(message, settings.subscriptionStop.subscriptionId);
        return this.sendJoynrMessage(message, settings);
    }
    /**
     * @param settings
     * @param settings.from participantId of the sender
     * @param settings.to participantId of the receiver
     * @param settings.expiryDate time-to-live
     * @param publication
     * @param [publication.response]
     * @param [publication.error]
     * @param publication.subscriptionId
     */
    sendPublication(settings, publication) {
        log.info("publication", DiagnosticTags.forPublication({
            publication,
            to: settings.to,
            from: settings.from
        }));
        this.sendPublicationInternal(settings, JoynrMessage_1.default.JOYNRMESSAGE_TYPE_PUBLICATION, publication);
    }
    /**
     * @param settings
     * @param settings.from participantId of the sender
     * @param settings.multicastName name of multicast
     * @param {String[]} partitions partitions for this multicast
     * @param settings.expiryDate time-to-live
     * @param publication
     * @param [publication.response]
     * @param [publication.error]
     * @param publication.multicastId
     */
    sendMulticastPublication(settings, publication) {
        const multicastId = publication.multicastId;
        log.info("publication", DiagnosticTags.forMulticastPublication({
            publication,
            from: settings.from
        }));
        // Reply with the result in a JoynrMessage
        const publicationMessage = new JoynrMessage_1.default({
            type: JoynrMessage_1.default.JOYNRMESSAGE_TYPE_MULTICAST,
            payload: JSONSerializer.stringify(publication)
        });
        publicationMessage.from = settings.from;
        publicationMessage.to = multicastId;
        publicationMessage.expiryDate = this.upLiftTtl(settings.expiryDate);
        if (log.isDebugEnabled()) {
            log.debug(`sendMulticastPublication, message = ${JSON.stringify(publicationMessage)}`);
        }
        this.clusterControllerMessagingStub.transmit(publicationMessage);
    }
    /**
     * receives a new JoynrMessage that has to be routed to one of the managers
     *
     * @param joynrMessage being routed
     */
    receive(joynrMessage) {
        log.debug(`received message with id "${joynrMessage.msgId}"`);
        if (log.isDebugEnabled()) {
            log.debug(`receive, message = ${JSON.stringify(joynrMessage)}`);
        }
        let payload;
        try {
            payload = JSON.parse(joynrMessage.payload);
        }
        catch (error) {
            log.error(`error parsing joynrMessage: ${error} payload: ${joynrMessage.payload}`);
            return Promise.resolve();
        }
        switch (joynrMessage.type) {
            case JoynrMessage_1.default.JOYNRMESSAGE_TYPE_REQUEST:
                try {
                    const request = Request.create(payload);
                    log.info(`received request for ${request.methodName}.`, DiagnosticTags.forRequest({
                        request,
                        to: joynrMessage.to,
                        from: joynrMessage.from
                    }));
                    const handleReplySettings = this.createReplySettings(joynrMessage);
                    const effort = joynrMessage.effort;
                    if (effort && MessagingQosEffort[effort]) {
                        handleReplySettings.effort = MessagingQosEffort[effort];
                    }
                    if (joynrMessage.compress) {
                        handleReplySettings.compress = true;
                    }
                    return this.requestReplyManager.handleRequest(joynrMessage.to, request, (handleReplySettings, reply) => this.sendRequestReply(handleReplySettings, reply), handleReplySettings);
                }
                catch (errorInRequest) {
                    // TODO handle error in handling the request
                    log.error(`error handling request: ${errorInRequest}`);
                }
                break;
            case JoynrMessage_1.default.JOYNRMESSAGE_TYPE_REPLY:
                try {
                    const reply = Reply.create(payload);
                    log.info("received reply ", DiagnosticTags.forReply({
                        reply,
                        to: joynrMessage.to,
                        from: joynrMessage.from
                    }));
                    this.requestReplyManager.handleReply(reply);
                }
                catch (errorInReply) {
                    // TODO handle error in handling the reply
                    log.error(`error handling reply: ${errorInReply}`);
                }
                break;
            case JoynrMessage_1.default.JOYNRMESSAGE_TYPE_ONE_WAY:
                try {
                    const oneWayRequest = OneWayRequest.create(payload);
                    log.info(`received one way request for ${oneWayRequest.methodName}.`, DiagnosticTags.forOneWayRequest({
                        request: oneWayRequest,
                        to: joynrMessage.to,
                        from: joynrMessage.from
                    }));
                    this.requestReplyManager.handleOneWayRequest(joynrMessage.to, oneWayRequest);
                }
                catch (errorInOneWayRequest) {
                    log.error(`error handling one way: ${errorInOneWayRequest}`);
                }
                break;
            case JoynrMessage_1.default.JOYNRMESSAGE_TYPE_SUBSCRIPTION_REQUEST:
                try {
                    const subscriptionRequest = this.upLiftExpiryDateInSubscriptionRequest(new SubscriptionRequest_1.default(payload));
                    log.info(`received subscription to ${subscriptionRequest.subscribedToName}`, DiagnosticTags.forSubscriptionRequest({
                        subscriptionRequest,
                        to: joynrMessage.to,
                        from: joynrMessage.from
                    }));
                    this.publicationManager.handleSubscriptionRequest(joynrMessage.from, joynrMessage.to, subscriptionRequest, this.sendSubscriptionReply, this.createReplySettings(joynrMessage));
                }
                catch (errorInSubscriptionRequest) {
                    // TODO handle error in handling the subscriptionRequest
                    log.error(`error handling subscriptionRequest: ${errorInSubscriptionRequest}`);
                }
                break;
            case JoynrMessage_1.default.JOYNRMESSAGE_TYPE_BROADCAST_SUBSCRIPTION_REQUEST:
                try {
                    const broadcastSubscriptionRequest = this.upLiftExpiryDateInSubscriptionRequest(new BroadcastSubscriptionRequest_1.default(payload));
                    log.info(`received broadcast subscription to ${broadcastSubscriptionRequest.subscribedToName}`, DiagnosticTags.forBroadcastSubscriptionRequest({
                        subscriptionRequest: broadcastSubscriptionRequest,
                        to: joynrMessage.to,
                        from: joynrMessage.from
                    }));
                    this.publicationManager.handleBroadcastSubscriptionRequest(joynrMessage.from, joynrMessage.to, broadcastSubscriptionRequest, this.sendSubscriptionReply, this.createReplySettings(joynrMessage));
                }
                catch (errorInBroadcastSubscriptionRequest) {
                    // TODO handle error in handling the subscriptionRequest
                    log.error(`error handling broadcastSubscriptionRequest: ${errorInBroadcastSubscriptionRequest}`);
                }
                break;
            case JoynrMessage_1.default.JOYNRMESSAGE_TYPE_MULTICAST_SUBSCRIPTION_REQUEST:
                try {
                    const multicastSubscriptionRequest = this.upLiftExpiryDateInSubscriptionRequest(new MulticastSubscriptionRequest_1.default(payload));
                    log.info(`received broadcast subscription to ${multicastSubscriptionRequest.subscribedToName}`, DiagnosticTags.forMulticastSubscriptionRequest({
                        subscriptionRequest: multicastSubscriptionRequest,
                        to: joynrMessage.to,
                        from: joynrMessage.from
                    }));
                    this.publicationManager.handleMulticastSubscriptionRequest(joynrMessage.from, joynrMessage.to, multicastSubscriptionRequest, this.sendSubscriptionReply, this.createReplySettings(joynrMessage));
                }
                catch (errorInMulticastSubscriptionRequest) {
                    // TODO handle error in handling the subscriptionRequest
                    log.error(`error handling multicastSubscriptionRequest: ${errorInMulticastSubscriptionRequest}`);
                }
                break;
            case JoynrMessage_1.default.JOYNRMESSAGE_TYPE_SUBSCRIPTION_REPLY:
                try {
                    const subscriptionReply = new SubscriptionReply_1.default(payload);
                    log.info("received subscription reply", DiagnosticTags.forSubscriptionReply({
                        subscriptionReply,
                        to: joynrMessage.to,
                        from: joynrMessage.from
                    }));
                    this.subscriptionManager.handleSubscriptionReply(subscriptionReply);
                }
                catch (errorInSubscriptionReply) {
                    // TODO handle error in handling the subscriptionReply
                    log.error(`error handling subscriptionReply: ${errorInSubscriptionReply}`);
                }
                break;
            case JoynrMessage_1.default.JOYNRMESSAGE_TYPE_SUBSCRIPTION_STOP:
                try {
                    const subscriptionStop = new SubscriptionStop_1.default(payload);
                    log.info(`received subscription stop ${subscriptionStop.subscriptionId}`, DiagnosticTags.forSubscriptionStop({
                        subscriptionId: subscriptionStop.subscriptionId,
                        to: joynrMessage.to,
                        from: joynrMessage.from
                    }));
                    this.publicationManager.handleSubscriptionStop(subscriptionStop);
                }
                catch (errorInSubscriptionStop) {
                    // TODO handle error in handling the subscriptionStop
                    log.error(`error handling subscriptionStop: ${errorInSubscriptionStop}`);
                }
                break;
            case JoynrMessage_1.default.JOYNRMESSAGE_TYPE_PUBLICATION:
                try {
                    const subscriptionPublication = SubscriptionPublication.create(payload);
                    log.info("received publication", DiagnosticTags.forPublication({
                        publication: subscriptionPublication,
                        to: joynrMessage.to,
                        from: joynrMessage.from
                    }));
                    this.subscriptionManager.handlePublication(subscriptionPublication);
                }
                catch (errorInPublication) {
                    // TODO handle error in handling the publication
                    log.error(`error handling publication: ${errorInPublication}`);
                }
                break;
            case JoynrMessage_1.default.JOYNRMESSAGE_TYPE_MULTICAST:
                try {
                    const multicastPublication = MulticastPublication.create(payload);
                    log.info("received publication", DiagnosticTags.forMulticastPublication({
                        publication: multicastPublication,
                        from: joynrMessage.from
                    }));
                    this.subscriptionManager.handleMulticastPublication(multicastPublication);
                }
                catch (errorInMulticastPublication) {
                    // TODO handle error in handling the multicast publication
                    log.error(`error handling multicast publication: ${errorInMulticastPublication}`);
                }
                break;
            default:
                log.error(`unknown JoynrMessage type : ${joynrMessage.type}. Discarding message: ${JSONSerializer.stringify(joynrMessage)}`);
                break;
        }
        return Promise.resolve();
    }
    /**
     * Shutdown the dispatcher
     */
    shutdown() {
        log.debug("Dispatcher shut down");
        /* do nothing, as either the managers on the layer above (RRM, PM, SM) or
         * the message router on the layer below are implementing the
         * correct handling when the runtime is shut down
         */
    }
}
module.exports = Dispatcher;
