import * as DiscoveryEntryWithMetaInfo from "../../generated/joynr/types/DiscoveryEntryWithMetaInfo";
import * as MessagingQos from "../messaging/MessagingQos";
import { OneWayRequest } from "./types/OneWayRequest";
import * as Reply from "./types/Reply";
import * as Request from "./types/Request";
import Dispatcher = require("./Dispatcher");
declare type ReplyCallback = (replySettings: any, reply: Reply.Reply) => void;
declare type PromisifyCallback = (error: any, success?: any) => void;
interface ReplyCaller {
    callback: PromisifyCallback;
    callbackSettings: any;
    expiresAt?: number;
}
declare class RequestReplyManager {
    private dispatcher;
    /**
     * @name RequestReplyManager#sendRequest
     * @function
     *
     * @param settings
     * @param settings.from participantId of the sender
     * @param settings.toDiscoveryEntry DiscoveryEntry of the receiver
     * @param settings.messagingQos quality-of-service parameters such as time-to-live
     * @param settings.request the Request to send
     * @param callbackSettings additional settings to handle the reply.
     * @returns the Promise for the Request
     */
    sendRequest: (settings: {
        from: string;
        toDiscoveryEntry: DiscoveryEntryWithMetaInfo;
        messagingQos: MessagingQos;
        request: Request.Request;
    }, callbackSettings: Record<string, any>) => Promise<{
        response: any[];
        settings: {
            outputParameter: {
                name: string;
                type: string;
            }[];
        };
    }>;
    private cleanupInterval;
    private started;
    private replyCallers;
    private providers;
    /**
     * The RequestReplyManager is responsible maintaining a list of providers that wish to
     * receive incoming requests, and also a list of requestReplyIds which is used to match
     * an incoming message with an expected reply.
     *
     * @constructor
     *
     * @param dispatcher
     */
    constructor(dispatcher: Dispatcher);
    private checkIfReady;
    private sendRequestInternal;
    private createReplyFromError;
    /**
     * @param settings
     * @param settings.from participantId of the sender
     * @param settings.toDiscoveryEntry DiscoveryEntry of the receiver
     * @param settings.messagingQos quality-of-service parameters such as time-to-live
     * @param settings.request the Request to send
     * @returns the Promise for the Request
     */
    sendOneWayRequest(settings: {
        from: string;
        toDiscoveryEntry: DiscoveryEntryWithMetaInfo;
        messagingQos: MessagingQos;
        request: OneWayRequest;
    }): Promise<any>;
    /**
     * The function addRequestCaller is called when a provider wishes to receive
     * incoming requests for the given participantId
     *
     * @param participantId of the provider receiving the incoming requests
     * @param provider
     */
    addRequestCaller(participantId: string, provider: Record<string, any>): void;
    /**
     * The function addReplyCaller is called when a proxy get/set or rpc is called and
     * is waiting for a reply The reply caller is automatically
     * removed when the ttl expires.
     *
     * @param requestReplyId
     * @param replyCaller
     * @param ttlMs relative number of milliseconds to wait for the reply.
     *            The replycaller will be removed in ttl_ms and and Error will be passed
     *            to the replyCaller
     */
    addReplyCaller(requestReplyId: string, replyCaller: ReplyCaller, ttlMs: number): void;
    /**
     * The function removeRequestCaller is called when a provider no longer wishes to
     * receive incoming requests
     *
     * @param participantId
     */
    removeRequestCaller(participantId: string): void;
    /**
     * @param providerParticipantId
     * @param request
     * @param handleReplyCallback callback for handling the reply
     * @param replySettings settings for handleReplyCallback to avoid unnecessary function object creation
     * @returns
     */
    handleRequest(providerParticipantId: string, request: Request.Request, handleReplyCallback: ReplyCallback, replySettings: Record<string, any>): Promise<void>;
    /**
     * @param providerParticipantId
     * @param request
     */
    handleOneWayRequest(providerParticipantId: string, request: OneWayRequest): void;
    /**
     * @param reply
     */
    handleReply(reply: Reply.Reply): void;
    /**
     * Shutdown the request reply manager
     */
    shutdown(): void;
}
export = RequestReplyManager;
