"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
const Reply = __importStar(require("./types/Reply"));
const Typing = __importStar(require("../util/Typing"));
const UtilInternal = __importStar(require("../util/UtilInternal"));
const JSONSerializer = __importStar(require("../util/JSONSerializer"));
const MethodInvocationException_1 = __importDefault(require("../exceptions/MethodInvocationException"));
const ProviderRuntimeException_1 = __importDefault(require("../exceptions/ProviderRuntimeException"));
const Version_1 = __importDefault(require("../../generated/joynr/types/Version"));
const LoggingManager_1 = __importDefault(require("../system/LoggingManager"));
const util_1 = __importDefault(require("util"));
const CLEANUP_CYCLE_INTERVAL = 1000;
const log = LoggingManager_1.default.getLogger("joynr.dispatching.RequestReplyManager");
class RequestReplyManager {
    /**
     * The RequestReplyManager is responsible maintaining a list of providers that wish to
     * receive incoming requests, and also a list of requestReplyIds which is used to match
     * an incoming message with an expected reply.
     *
     * @constructor
     *
     * @param dispatcher
     */
    constructor(dispatcher) {
        this.started = true;
        this.providers = {};
        this.replyCallers = new Map();
        this.cleanupInterval = setInterval(() => {
            const currentTime = Date.now();
            for (const [id, caller] of this.replyCallers) {
                // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                if (caller.expiresAt <= currentTime) {
                    caller.callback(new Error(`Request with id "${id}" failed: ttl expired`));
                    this.replyCallers.delete(id);
                }
            }
        }, CLEANUP_CYCLE_INTERVAL);
        this.sendRequestInternal = this.sendRequestInternal.bind(this);
        this.sendRequest = util_1.default.promisify(this.sendRequestInternal);
        this.dispatcher = dispatcher;
    }
    checkIfReady() {
        if (!this.started) {
            throw new Error("RequestReplyManager is already shut down");
        }
    }
    sendRequestInternal(settings, callbackSettings, callback) {
        try {
            this.checkIfReady();
        }
        catch (e) {
            callback(e);
            return;
        }
        this.addReplyCaller(settings.request.requestReplyId, {
            callback,
            callbackSettings
        }, settings.messagingQos.ttl);
        this.dispatcher.sendRequest(settings);
    }
    createReplyFromError(exception, requestReplyId, handleReplyCallback, replySettings) {
        const reply = Reply.create({
            error: exception,
            requestReplyId
        });
        return handleReplyCallback(replySettings, reply);
    }
    /**
     * @param settings
     * @param settings.from participantId of the sender
     * @param settings.toDiscoveryEntry DiscoveryEntry of the receiver
     * @param settings.messagingQos quality-of-service parameters such as time-to-live
     * @param settings.request the Request to send
     * @returns the Promise for the Request
     */
    sendOneWayRequest(settings) {
        this.checkIfReady();
        return this.dispatcher.sendOneWayRequest(settings);
    }
    /**
     * The function addRequestCaller is called when a provider wishes to receive
     * incoming requests for the given participantId
     *
     * @param participantId of the provider receiving the incoming requests
     * @param provider
     */
    addRequestCaller(participantId, provider) {
        this.checkIfReady();
        this.providers[participantId] = provider;
    }
    /**
     * The function addReplyCaller is called when a proxy get/set or rpc is called and
     * is waiting for a reply The reply caller is automatically
     * removed when the ttl expires.
     *
     * @param requestReplyId
     * @param replyCaller
     * @param ttlMs relative number of milliseconds to wait for the reply.
     *            The replycaller will be removed in ttl_ms and and Error will be passed
     *            to the replyCaller
     */
    addReplyCaller(requestReplyId, replyCaller, ttlMs) {
        this.checkIfReady();
        replyCaller.expiresAt = Date.now() + ttlMs;
        this.replyCallers.set(requestReplyId, replyCaller);
    }
    /**
     * The function removeRequestCaller is called when a provider no longer wishes to
     * receive incoming requests
     *
     * @param participantId
     */
    removeRequestCaller(participantId) {
        this.checkIfReady();
        try {
            delete this.providers[participantId];
        }
        catch (error) {
            log.error(`error removing provider with participantId: ${participantId} error: ${error}`);
        }
    }
    /**
     * @param providerParticipantId
     * @param request
     * @param handleReplyCallback callback for handling the reply
     * @param replySettings settings for handleReplyCallback to avoid unnecessary function object creation
     * @returns
     */
    async handleRequest(providerParticipantId, request, handleReplyCallback, replySettings) {
        let exception;
        try {
            this.checkIfReady();
        }
        catch (error) {
            exception = new MethodInvocationException_1.default({
                detailMessage: `error handling request: ${JSONSerializer.stringify(request)} for providerParticipantId ${providerParticipantId}. Joynr runtime already shut down.`
            });
            return this.createReplyFromError(exception, request.requestReplyId, handleReplyCallback, replySettings);
        }
        const provider = this.providers[providerParticipantId];
        if (!provider) {
            // TODO error handling request
            // TODO what if no provider is found in the mean time?
            // Do we need to add a task to handleRequest later?
            exception = new MethodInvocationException_1.default({
                detailMessage: `error handling request: ${JSONSerializer.stringify(request)} for providerParticipantId ${providerParticipantId}`
            });
            return this.createReplyFromError(exception, request.requestReplyId, handleReplyCallback, replySettings);
        }
        // if there's an operation available to call
        let result;
        if (provider[request.methodName] && provider[request.methodName].callOperation) {
            // may throw an immediate exception when callOperation checks the
            // arguments, in this case exception must be caught.
            // If final customer provided method implementation gets called,
            // that one may return either promise (preferred) or direct result
            // and may possibly also throw exception in the latter case.
            try {
                result = await provider[request.methodName].callOperation(request.params, request.paramDatatypes);
            }
            catch (internalException) {
                exception = internalException;
            }
            // otherwise, check whether request is an attribute get, set or an operation
        }
        else {
            const match = request.methodName.match(/([gs]et)?(\w+)/);
            const getSet = match[1];
            if (getSet) {
                const attributeName = match[2];
                const attributeObject = provider[attributeName] || provider[UtilInternal.firstLower(attributeName)];
                // if the attribute exists in the provider
                if (attributeObject && !attributeObject.callOperation) {
                    try {
                        if (getSet === "get") {
                            result = await attributeObject.get();
                        }
                        else if (getSet === "set") {
                            // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                            result = await attributeObject.set(request.params[0]);
                        }
                    }
                    catch (internalGetterSetterException) {
                        if (internalGetterSetterException instanceof ProviderRuntimeException_1.default) {
                            exception = internalGetterSetterException;
                        }
                        else {
                            exception = new ProviderRuntimeException_1.default({
                                detailMessage: `getter/setter method of attribute ${attributeName} reported an error ${internalGetterSetterException}`
                            });
                        }
                    }
                }
                else {
                    // if neither an operation nor an attribute exists in the
                    // provider => deliver MethodInvocationException
                    exception = new MethodInvocationException_1.default({
                        detailMessage: `Could not find an operation "${request.methodName}" or an attribute "${attributeName}" in the provider`,
                        providerVersion: new Version_1.default({
                            majorVersion: provider.constructor.MAJOR_VERSION,
                            minorVersion: provider.constructor.MINOR_VERSION
                        })
                    });
                }
            }
            else {
                // if no operation was found and methodName didn't start with "get"
                // or "set" => deliver MethodInvocationException
                exception = new MethodInvocationException_1.default({
                    detailMessage: `Could not find an operation "${request.methodName}" in the provider`,
                    providerVersion: new Version_1.default({
                        majorVersion: provider.constructor.MAJOR_VERSION,
                        minorVersion: provider.constructor.MINOR_VERSION
                    })
                });
            }
        }
        /*
          both ProviderOperation.callOperation and ProviderAttribute.get/set have
          have asynchronous API. Therefore the result is always a promise and thus
          it's possible to await for its result.
        */
        if (exception) {
            return this.createReplyFromError(exception, request.requestReplyId, handleReplyCallback, replySettings);
        }
        const reply = Reply.create({
            response: result,
            requestReplyId: request.requestReplyId
        });
        return handleReplyCallback(replySettings, reply);
    }
    /**
     * @param providerParticipantId
     * @param request
     */
    handleOneWayRequest(providerParticipantId, request) {
        this.checkIfReady();
        const provider = this.providers[providerParticipantId];
        if (!provider) {
            throw new MethodInvocationException_1.default({
                detailMessage: `error handling one-way request: ${JSONSerializer.stringify(request)} for providerParticipantId ${providerParticipantId}`
            });
        }
        // if there's an operation available to call
        if (provider[request.methodName] && provider[request.methodName].callOperation) {
            // If final customer provided method implementation gets called,
            // that one may return either promise (preferred) or direct result
            // and may possibly also throw exception in the latter case.
            provider[request.methodName].callOperation(request.params, request.paramDatatypes);
        }
        else {
            throw new MethodInvocationException_1.default({
                detailMessage: `Could not find an operation "${request.methodName}" in the provider`,
                providerVersion: new Version_1.default({
                    majorVersion: provider.constructor.MAJOR_VERSION,
                    minorVersion: provider.constructor.MINOR_VERSION
                })
            });
        }
    }
    /**
     * @param reply
     */
    handleReply(reply) {
        const replyCaller = this.replyCallers.get(reply.requestReplyId);
        if (replyCaller === undefined) {
            log.error(`error handling reply resolve, because replyCaller could not be found: ${JSONSerializer.stringify(reply)}`);
            return;
        }
        try {
            if (reply.error) {
                log.info(`RequestReplyManager::handleReply got reply with exception ${util_1.default.inspect(reply.error)}`);
                if (reply.error instanceof Error) {
                    replyCaller.callback(reply.error);
                }
                else {
                    replyCaller.callback(Typing.augmentTypes(reply.error));
                }
            }
            else {
                replyCaller.callback(undefined, {
                    response: reply.response,
                    settings: replyCaller.callbackSettings
                });
            }
            this.replyCallers.delete(reply.requestReplyId);
        }
        catch (e) {
            log.error(`exception thrown during handling reply ${JSONSerializer.stringify(reply)}:\n${e.stack}`);
        }
    }
    /**
     * Shutdown the request reply manager
     */
    shutdown() {
        clearInterval(this.cleanupInterval);
        for (const replyCaller of this.replyCallers.values()) {
            if (replyCaller) {
                replyCaller.callback(new Error("RequestReplyManager is already shut down"));
            }
        }
        this.replyCallers.clear();
        this.started = false;
    }
}
module.exports = RequestReplyManager;
