import SubscriptionStop from "../types/SubscriptionStop";
import Dispatcher = require("../Dispatcher");
import SubscriptionRequest = require("../types/SubscriptionRequest");
import BroadcastSubscriptionRequest = require("../types/BroadcastSubscriptionRequest");
import MulticastSubscriptionRequest = require("../types/MulticastSubscriptionRequest");
declare class PublicationManager {
    private dispatcher;
    private started;
    private multicastSubscriptions;
    private onChangeProviderEventToSubscriptions;
    private onChangeProviderAttributeToSubscriptions;
    private queuedProviderParticipantIdToSubscriptionRequestsMapping;
    private queuedSubscriptionInfos;
    private subscriptionInfos;
    private eventObserverFunctions;
    private attributeObserverFunctions;
    private participantIdToProvider;
    /**
     * The PublicationManager is responsible for handling subscription requests.
     *
     * @param dispatcher
     */
    constructor(dispatcher: Dispatcher);
    private isReady;
    /**
     * Helper function to get attribute object based on provider's participantId and
     * attribute name
     *
     * @param participantId the participantId of the provider
     * @param attributeName the attribute name
     * @returns the provider attribute
     */
    private getAttribute;
    /**
     * Helper function to get attribute value. In case the attribute getter function of
     * the provider returns a promise object this function waits until the promise object
     * is resolved to resolve the promise object return by this function
     *
     * @param subscriptionInfo the subscription information
     * @param subscriptionInfo.providerParticipantId the participantId of the provider
     * @param subscriptionInfo.subscribedToName the attribute to be published
     * @returns an A+ promise object which provides attribute value upon success and an
     *            error upon failure
     */
    private getAttributeValue;
    private sendPublication;
    private getPeriod;
    private prepareAttributePublication;
    private prepareBroadcastPublication;
    private triggerPublication;
    /**
     * This functions waits the delay time before publishing the value of the attribute
     * specified in the subscription information
     *
     * @param subscriptionInfo the subscription information
     * @param subscriptionInfo.providerParticipantId the participantId of the provider
     * @param subscriptionInfo.subscribedToName the attribute to be published
     * @param delay the delay to wait for the publication
     */
    private triggerPublicationTimer;
    private getProviderIdAttributeKey;
    private getProviderIdEventKey;
    /**
     * Gives the list of subscriptions for the given providerId and attribute name
     *
     * @param providerId
     * @param attributeName
     * @returns a list of subscriptions
     e.g. [{ participantId : "...", attributeName : "..." }]
     */
    private getSubscriptionsForProviderAttribute;
    private resetSubscriptionsForProviderAttribute;
    /**
     * Checks whether the provider attribute is notifiable (=is even interesting to the
     * PublicationManager)
     *
     * @param providerAttribute the provider attribute to check for Notifiability
     * @returns if the provider attribute is notifiable
     */
    private providerAttributeIsNotifiable;
    /**
     * Checks whether the provider property is an event (=is even interesting to the
     * PublicationManager)
     *
     * @param providerProperty the provider attribute to check for Notifiability
     * @returns if the provider attribute is notifiable
     */
    private propertyIsProviderEvent;
    /**
     * @param providerId
     * @param attributeName
     * @param _attribute
     * @param value
     */
    private publishAttributeValue;
    /**
     * Adds a notifiable provider attribute to the internal map if it does not exist =>
     * onChange subscriptions can be registered here
     *
     * @param providerId
     * @param attributeName
     * @param attribute
     */
    private addPublicationAttribute;
    /**
     * Gives the list of subscriptions for the given providerId and event name
     * @name PublicationManager#getSubscriptionsForProviderEvent
     * @private
     *
     * @param providerId
     * @param eventName
     * @returns a list of subscriptions e.g. [{ participantId : "...", eventName : "..." }]
     */
    private getSubscriptionsForProviderEvent;
    private prepareMulticastPublication;
    /**
     * @param providerId
     * @param eventName
     * @param event
     * @param data
     */
    private publishEventValue;
    /**
     * Adds a provider event to the internal map if it does not exist =>
     * onChange subscriptions can be registered here
     *
     * @param providerId
     * @param eventName
     * @param event
     */
    private addPublicationEvent;
    private resetSubscriptionsForProviderEvent;
    private addRequestToMulticastSubscriptions;
    private removeRequestFromMulticastSubscriptions;
    /**
     * Removes a subscription, stops scheduled timers
     *
     * @param subscriptionId
     * @param silent suppress log outputs if subscription cannot be found
     */
    private removeSubscription;
    /**
     * Removes a notifiable provider attribute from the internal map if it exists
     *
     * @param providerId
     * @param attributeName
     * @param attribute
     */
    private removePublicationAttribute;
    /**
     * Removes a provider event from the internal map if it exists
     *
     * @param providerId
     * @param eventName
     */
    private removePublicationEvent;
    /**
     * the parameter "callbackDispatcher" is optional, as in case of restoring
     * subscriptions, no reply must be sent back via the dispatcher
     */
    private optionalCallbackDispatcher;
    private handleBroadcastSubscriptionRequestInternal;
    /**
     * @param subscriptionStop incoming subscriptionStop
     */
    handleSubscriptionStop(subscriptionStop: SubscriptionStop): void;
    /**
     * @param providerId
     * @param eventName
     * @returns true if a subscription exists for the given event
     */
    hasSubscriptionsForProviderEvent(providerId: string, eventName: string): boolean;
    /**
     * @param providerId
     * @param attributeName
     * @returns true if a subscription exists for the given attribute
     */
    hasSubscriptionsForProviderAttribute(providerId: string, attributeName: string): boolean;
    /**
     * Handles SubscriptionRequests
     *
     * @param proxyParticipantId - participantId of proxy consuming the attribute publications
     *            providerParticipantId - participantId of provider producing the attribute publications
     *            subscriptionRequest incoming subscriptionRequest
     *            callbackDispatcher callback function to inform the caller about the handling result
     * @param providerParticipantId
     * @param subscriptionRequest
     * @param callbackDispatcher
     * @param callbackDispatcherSettings
     * @throws {Error} when no provider exists or the provider does not have the attribute
     */
    handleSubscriptionRequest(proxyParticipantId: string, providerParticipantId: string, subscriptionRequest: SubscriptionRequest, callbackDispatcher?: Function, callbackDispatcherSettings?: any): void;
    /**
     * Handles BroadcastSubscriptionRequests
     *
     * @param proxyParticipantId - participantId of proxy consuming the broadcast
     *            providerParticipantId - participantId of provider producing the broadcast
     *            subscriptionRequest incoming subscriptionRequest
     *            callbackDispatcher callback function to inform the caller about the handling result
     * @param providerParticipantId
     * @param subscriptionRequest
     * @param callbackDispatcher
     * @param callbackDispatcherSettings
     * @throws {Error} when no provider exists or the provider does not have the event
     */
    handleBroadcastSubscriptionRequest(proxyParticipantId: string, providerParticipantId: string, subscriptionRequest: BroadcastSubscriptionRequest, callbackDispatcher?: Function, callbackDispatcherSettings?: any): void;
    /**
     * Handles MulticastSubscriptionRequests
     *
     * @param proxyParticipantId
     * @param providerParticipantId - participantId of provider producing the multicast
     *            subscriptionRequest incoming subscriptionRequest
     *            callbackDispatcher callback function to inform the caller about the handling result
     * @param subscriptionRequest
     * @param callbackDispatcher
     * @param callbackDispatcherSettings
     * @throws {Error} when no provider exists or the provider does not have the event
     */
    handleMulticastSubscriptionRequest(proxyParticipantId: string, providerParticipantId: string, subscriptionRequest: MulticastSubscriptionRequest, callbackDispatcher?: Function, callbackDispatcherSettings?: any): void;
    /**
     * Unregisters all attributes of a provider to handle subscription requests
     *
     * @param participantId of the provider handling the subsription
     * @param provider
     */
    removePublicationProvider(participantId: string, provider: any): void;
    /**
     * Registers all attributes of a provider to handle subscription requests
     *
     * @param participantId of the provider handling the subsription
     * @param provider
     */
    addPublicationProvider(participantId: string, provider: any): void;
    hasMulticastSubscriptions(): boolean;
    hasSubscriptions(): boolean;
    /**
     * Shutdown the publication manager
     */
    shutdown(): void;
}
export = PublicationManager;
