import * as DiscoveryEntryWithMetaInfo from "../../../generated/joynr/types/DiscoveryEntryWithMetaInfo";
import MessagingQos from "../../messaging/MessagingQos";
import SubscriptionQos from "../../proxy/SubscriptionQos";
import { MulticastPublication } from "../types/MulticastPublication";
import { SubscriptionPublication } from "../types/SubscriptionPublication";
import Dispatcher = require("../Dispatcher");
import SubscriptionReply = require("../types/SubscriptionReply");
import BroadcastFilterParameters = require("../../proxy/BroadcastFilterParameters");
interface SubscriptionSettings {
    proxyId: string;
    providerDiscoveryEntry: DiscoveryEntryWithMetaInfo;
    attributeType: string;
    attributeName: string;
    qos: SubscriptionQos;
    subscriptionId?: string;
    onReceive: (value: any) => void;
    onError?: (e: Error) => void;
    onSubscribed?: (participantId: string) => void;
}
interface BroadcastSubscriptionSettings {
    proxyId: string;
    providerDiscoveryEntry: DiscoveryEntryWithMetaInfo;
    broadcastName: string;
    broadcastParameter: {
        name: string;
        type: string;
    }[];
    subscriptionQos: SubscriptionQos;
    filterParameters: BroadcastFilterParameters;
    selective?: boolean;
    partitions: string[];
    subscriptionId?: string;
    onReceive: Function;
    onSubscribed?: Function;
    onError?: Function;
}
declare class SubscriptionManager {
    private dispatcher;
    /**
     * @name SubscriptionManager#registerBroadcastSubscription
     * @function
     * @param parameters
     * @param parameters.proxyId participantId of the sender
     * @param parameters.providerDiscoveryEntry DiscoveryEntry of the receiver
     * @param parameters.broadcastName the name of the broadcast being subscribed to
     * @param parameters.broadcastParameter the parameter meta information of the broadcast being subscribed to
     * @param [parameters.subscriptionQos] the subscriptionQos
     * @param [parameters.filterParameters] filter parameters used to indicate interest in
     *            only a subset of broadcasts that might be sent.
     * @param parameters.selective true if broadcast is selective
     * @param [parameters.partitions] partitions for multicast requests
     * @param parameters.subscriptionId optional parameter subscriptionId to reuse a
     *            pre-existing identifier for this concrete subscription request
     * @param parameters.onReceive is called when a broadcast is received.
     * @param parameters.onError is called when an error occurs with the broadcast
     * @param parameters.onSubscribed the callback to inform once the subscription request has
     *            been delivered successfully
     * @returns a promise object which provides the subscription token upon success and an error
     *          upon failure
     */
    registerBroadcastSubscription: (parameters: BroadcastSubscriptionSettings) => Promise<string>;
    /**
     * This callback is called when a publication is received
     * @callback SubscriptionManager~onReceive
     * @param {Object} publication received
     */
    /**
     * This callback is called if there was an error with the subscription
     * @callback SubscriptionManager~onError
     * @param {Error} error
     */
    /**
     * @name SubscriptionManager#registerSubscription
     * @function
     * @param settings
     * @param settings.proxyId participantId of the sender
     * @param settings.providerDiscoveryEntry DiscoveryEntry of the receiver
     * @param settings.attributeType the type of the subscribing attribute
     * @param settings.attributeName the attribute name to subscribe to
     * @param [settings.qos] the subscriptionQos
     * @param settings.subscriptionId optional parameter subscriptionId to reuse a
     *            preexisting identifier for this concrete subscription request
     * @param settings.onReceive the callback for received publications.
     * @param settings.onError the callback for missing publication alerts or when an
     *            error occurs.
     * @param settings.onSubscribed the callback to inform once the subscription request has
     *            been delivered successfully
     * @returns an A promise object which provides the subscription token upon success and
     *          an error upon failure
     */
    registerSubscription: (settings: SubscriptionSettings) => Promise<string>;
    private multicastSubscribers;
    private started;
    private subscriptionReplyCallers;
    private publicationCheckTimerIds;
    private subscriptionListeners;
    private subscriptionInfos;
    private multicastWildcardRegexFactory;
    /**
     * @constructor
     * @param dispatcher
     */
    constructor(dispatcher: Dispatcher);
    private isReady;
    /**
     * @param subscriptionId Id of the subscription to check
     * @returns time of last received publication
     */
    private getLastPublicationTime;
    private setLastPublicationTime;
    /**
     * @param subscriptionId Id of the subscription to check
     * @param delayMs Delay to the next publication check.
     * @returns true if subscription is expired, false if end date is not reached.
     */
    private subscriptionEnds;
    /**
     * @param subscriptionId Id of the subscription to check
     * @param alertAfterIntervalMs maximum delay between two incoming publications
     */
    private checkPublication;
    private calculateTtl;
    private storeSubscriptionRequest;
    private removeRequestFromMulticastSubscribers;
    private cleanupSubscription;
    private registerSubscriptionInternal;
    private addRequestToMulticastSubscribers;
    private createBroadcastSubscriptionRequest;
    /**
     * @name SubscriptionManager#registerBroadcastSubscription
     * @function
     * @param parameters
     * @param parameters.proxyId participantId of the sender
     * @param parameters.providerDiscoveryEntry DiscoveryEntry of the receiver
     * @param parameters.broadcastName the name of the broadcast being subscribed to
     * @param parameters.broadcastParameter the parameter meta information of the broadcast being subscribed to
     * @param [parameters.subscriptionQos] the subscriptionQos
     * @param [parameters.filterParameters] filter parameters used to indicate interest in
     *            only a subset of broadcasts that might be sent.
     * @param parameters.selective true if broadcast is selective
     * @param [parameters.partitions] partitions for multicast requests
     * @param parameters.subscriptionId optional parameter subscriptionId to reuse a
     *            pre-existing identifier for this concrete subscription request
     * @param {SubscriptionManager~onReceive} parameters.onReceive is called when a broadcast is received.
     * @param {SubscriptionManager~onError} parameters.onError is called when an error occurs with the broadcast
     * @param {SubscriptionManager~onSubscribed} parameters.onSubscribed the callback to inform once the subscription request has
     *            been delivered successfully
     * @param cb
     */
    private registerBroadcastSubscriptionInternal;
    /**
     * @param subscriptionReply incoming subscriptionReply
     */
    handleSubscriptionReply(subscriptionReply: SubscriptionReply): void;
    /**
     * @param publication incoming multicast publication
     */
    handleMulticastPublication(publication: MulticastPublication): void;
    /**
     * @param publication incoming publication
     */
    handlePublication(publication: SubscriptionPublication): void;
    /**
     * @param settings
     * @param settings.messagingQos the messaging Qos object for the ttl
     * @param settings.subscriptionId of the subscriptionId to stop
     * @returns A promise object
     */
    unregisterSubscription(settings: {
        messagingQos: MessagingQos;
        subscriptionId: string;
    }): Promise<void>;
    hasMulticastSubscriptions(): boolean;
    hasOpenSubscriptions(): boolean;
    /**
     * This method is meant to be called by the runtime before shutdown is called.
     * It turns out that there is a necessary shutdown order and SubscriptionManager can't be shutdown first.
     *
     * @param timeoutMs timeout in ms after which this operation shall timeout. 0 defaults to no timeout.
     * @returns - resolves if subscriptionStop message has been sent for each active subscription
     * - rejects in case of any issues or timeout occurs
     */
    terminateSubscriptions(timeoutMs: number): Promise<any>;
    /**
     * Shutdown the subscription manager
     */
    shutdown(): void;
}
export = SubscriptionManager;
