"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
const MessagingQos_1 = __importDefault(require("../../messaging/MessagingQos"));
const MulticastWildcardRegexFactory_1 = __importDefault(require("../../messaging/util/MulticastWildcardRegexFactory"));
const defaultMessagingSettings_1 = __importDefault(require("../../start/settings/defaultMessagingSettings"));
const SubscriptionQos_1 = __importDefault(require("../../proxy/SubscriptionQos"));
const SubscriptionStop_1 = __importDefault(require("../types/SubscriptionStop"));
const SubscriptionRequest_1 = __importDefault(require("../types/SubscriptionRequest"));
const MulticastSubscriptionRequest_1 = __importDefault(require("../types/MulticastSubscriptionRequest"));
const BroadcastSubscriptionRequest_1 = __importDefault(require("../types/BroadcastSubscriptionRequest"));
const SubscriptionListener_1 = __importDefault(require("./SubscriptionListener"));
const SubscriptionUtil = __importStar(require("./util/SubscriptionUtil"));
const LongTimer_1 = __importDefault(require("../../util/LongTimer"));
const LoggingManager_1 = __importDefault(require("../../system/LoggingManager"));
const nanoid_1 = require("nanoid");
const UtilInternal = __importStar(require("../../util/UtilInternal"));
const Typing = __importStar(require("../../util/Typing"));
const PublicationMissedException_1 = __importDefault(require("../../exceptions/PublicationMissedException"));
const JSONSerializer = __importStar(require("../../util/JSONSerializer"));
const util_1 = __importDefault(require("util"));
const log = LoggingManager_1.default.getLogger("joynr.dispatching.subscription.SubscriptionManager");
class SubscriptionManager {
    /**
     * @constructor
     * @param dispatcher
     */
    constructor(dispatcher) {
        this.multicastSubscribers = {};
        this.started = true;
        // stores the object which is returned by setTimeout mapped to the subscriptionId
        this.publicationCheckTimerIds = {};
        // stores subscriptionId - SubscriptionListener
        this.subscriptionListeners = {};
        // stores subscriptionId - SubscriptionInfo pairs
        this.subscriptionInfos = {};
        this.multicastWildcardRegexFactory = new MulticastWildcardRegexFactory_1.default();
        this.subscriptionReplyCallers = new Map();
        this.registerSubscription = util_1.default.promisify(this.registerSubscriptionInternal);
        this.registerBroadcastSubscription = util_1.default.promisify(this.registerBroadcastSubscriptionInternal);
        this.dispatcher = dispatcher;
        this.checkPublication = this.checkPublication.bind(this);
    }
    isReady() {
        return this.started;
    }
    /**
     * @param subscriptionId Id of the subscription to check
     * @returns time of last received publication
     */
    getLastPublicationTime(subscriptionId) {
        return this.subscriptionInfos[subscriptionId].lastPublicationTime_ms;
    }
    setLastPublicationTime(subscriptionId, timeMs) {
        // eslint-disable-next-line @typescript-eslint/camelcase
        this.subscriptionInfos[subscriptionId].lastPublicationTime_ms = timeMs;
    }
    /**
     * @param subscriptionId Id of the subscription to check
     * @param delayMs Delay to the next publication check.
     * @returns true if subscription is expired, false if end date is not reached.
     */
    subscriptionEnds(subscriptionId, delayMs) {
        if (this.subscriptionInfos[subscriptionId] === undefined) {
            log.warn(`subscriptionEnds has been called with unresolved subscriptionId "${subscriptionId}"`);
            return true;
        }
        const expiryDateMs = this.subscriptionInfos[subscriptionId].qos.expiryDateMs;
        // log.debug("Checking subscription end for subscriptionId: " + subscriptionId + "
        // expiryDateMs: " + expiryDateMs + "
        // current time: " + Date.now());
        const ends = expiryDateMs <= Date.now() + delayMs;
        // if (ends === true) {
        // log.info("Subscription end date reached for id: " + subscriptionId);
        // }
        return ends;
    }
    /**
     * @param subscriptionId Id of the subscription to check
     * @param alertAfterIntervalMs maximum delay between two incoming publications
     */
    checkPublication(subscriptionId, alertAfterIntervalMs) {
        const subscriptionListener = this.subscriptionListeners[subscriptionId];
        const timeSinceLastPublication = Date.now() - this.getLastPublicationTime(subscriptionId);
        // log.debug("timeSinceLastPublication : " + timeSinceLastPublication + "
        // alertAfterIntervalMs: " + alertAfterIntervalMs);
        if (alertAfterIntervalMs > 0 && timeSinceLastPublication >= alertAfterIntervalMs) {
            // log.warn("publication missed for subscription id: " + subscriptionId);
            if (subscriptionListener.onError) {
                const publicationMissedException = new PublicationMissedException_1.default({
                    detailMessage: "alertAfterIntervalMs period exceeded without receiving publication",
                    subscriptionId
                });
                subscriptionListener.onError(publicationMissedException);
            }
        }
        let delayMs;
        if (timeSinceLastPublication > alertAfterIntervalMs) {
            delayMs = alertAfterIntervalMs;
        }
        else {
            delayMs = alertAfterIntervalMs - timeSinceLastPublication;
        }
        if (!this.subscriptionEnds(subscriptionId, delayMs)) {
            // log.debug("Rescheduling checkPublication with delay: " + delay_ms);
            this.publicationCheckTimerIds[subscriptionId] = LongTimer_1.default.setTimeout(this.checkPublication, delayMs, subscriptionId, alertAfterIntervalMs);
        }
    }
    calculateTtl(subscriptionQos) {
        if (subscriptionQos.expiryDateMs === SubscriptionQos_1.default.NO_EXPIRY_DATE) {
            return defaultMessagingSettings_1.default.MAX_MESSAGING_TTL_MS;
        }
        const ttl = subscriptionQos.expiryDateMs - Date.now();
        if (ttl > defaultMessagingSettings_1.default.MAX_MESSAGING_TTL_MS) {
            return defaultMessagingSettings_1.default.MAX_MESSAGING_TTL_MS;
        }
        return ttl;
    }
    storeSubscriptionRequest(settings, subscriptionRequest) {
        let onReceiveWrapper;
        if (settings.attributeType !== undefined) {
            onReceiveWrapper = (response) => {
                settings.onReceive(Typing.augmentTypes(response[0], settings.attributeType));
            };
        }
        else {
            onReceiveWrapper = (response) => {
                for (const responseKey in response) {
                    if (Object.prototype.hasOwnProperty.call(response, responseKey)) {
                        response[responseKey] = Typing.augmentTypes(response[responseKey], settings.broadcastParameter[responseKey].type);
                    }
                }
                settings.onReceive(response);
            };
        }
        this.subscriptionListeners[subscriptionRequest.subscriptionId] = new SubscriptionListener_1.default({
            onReceive: onReceiveWrapper,
            onError: settings.onError,
            onSubscribed: settings.onSubscribed
        });
        const subscriptionInfo = UtilInternal.extend({
            proxyId: settings.proxyId,
            providerDiscoveryEntry: settings.providerDiscoveryEntry,
            // eslint-disable-next-line @typescript-eslint/camelcase
            lastPublicationTime_ms: 0
        }, subscriptionRequest);
        this.subscriptionInfos[subscriptionRequest.subscriptionId] = subscriptionInfo;
        const alertAfterIntervalMs = subscriptionRequest.qos
            .alertAfterIntervalMs;
        if (alertAfterIntervalMs !== undefined && alertAfterIntervalMs > 0) {
            this.publicationCheckTimerIds[subscriptionRequest.subscriptionId] = LongTimer_1.default.setTimeout(this.checkPublication, alertAfterIntervalMs, subscriptionRequest.subscriptionId, alertAfterIntervalMs);
        }
    }
    removeRequestFromMulticastSubscribers(_multicastId, subscriptionId) {
        for (const multicastIdPattern in this.multicastSubscribers) {
            if (Object.prototype.hasOwnProperty.call(this.multicastSubscribers, multicastIdPattern)) {
                const subscribers = this.multicastSubscribers[multicastIdPattern];
                for (let i = 0; i < subscribers.length; i++) {
                    if (subscribers[i] === subscriptionId) {
                        subscribers.splice(i, 1);
                        if (subscribers.length === 0) {
                            delete this.multicastSubscribers[multicastIdPattern];
                        }
                    }
                }
            }
        }
    }
    cleanupSubscription(subscriptionId) {
        if (this.publicationCheckTimerIds[subscriptionId] !== undefined) {
            LongTimer_1.default.clearTimeout(this.publicationCheckTimerIds[subscriptionId]);
            delete this.publicationCheckTimerIds[subscriptionId];
        }
        if (this.subscriptionInfos[subscriptionId] !== undefined) {
            const subscriptionInfo = this.subscriptionInfos[subscriptionId];
            if (subscriptionInfo.multicastId !== undefined) {
                this.removeRequestFromMulticastSubscribers(subscriptionInfo.multicastId, subscriptionId);
            }
            delete this.subscriptionInfos[subscriptionId];
        }
        if (this.subscriptionListeners[subscriptionId] !== undefined) {
            delete this.subscriptionListeners[subscriptionId];
        }
        this.subscriptionReplyCallers.delete(subscriptionId);
    }
    registerSubscriptionInternal(settings, cb) {
        if (!this.isReady()) {
            cb(new Error("SubscriptionManager is already shut down"));
            return;
        }
        const subscriptionId = settings.subscriptionId || nanoid_1.nanoid();
        // log.debug("Registering Subscription Id " + subscriptionId);
        if (settings.attributeName === undefined) {
            cb(new Error(`Error: attributeName not provided in call to registerSubscription, settings = ${JSON.stringify(settings)}`));
        }
        if (settings.attributeType === undefined) {
            cb(new Error(`Error: attributeType not provided in call to registerSubscription, settings = ${JSON.stringify(settings)}`));
        }
        if (settings.onError === undefined) {
            log.warn(`Warning: subscription for attribute "${settings.attributeName}" has been done without error callback function. You will not be informed about missed publications. Please specify the "onError" parameter while subscribing!`);
        }
        if (settings.onReceive === undefined) {
            log.warn(`Warning: subscription for attribute "${settings.attributeName}" has been done without receive callback function. You will not be informed about incoming publications. Please specify the "onReceive" parameter while subscribing!`);
        }
        const subscriptionRequest = new SubscriptionRequest_1.default({
            subscriptionId,
            subscribedToName: settings.attributeName,
            qos: settings.qos
        });
        const ttl = this.calculateTtl(subscriptionRequest.qos);
        const messagingQos = new MessagingQos_1.default({ ttl });
        const timeout = LongTimer_1.default.setTimeout(() => {
            this.cleanupSubscription(subscriptionId);
            cb(new Error(`SubscriptionRequest with id ${subscriptionId} failed: tll expired`));
        }, ttl);
        this.subscriptionReplyCallers.set(subscriptionId, {
            cb: (...args) => {
                LongTimer_1.default.clearTimeout(timeout);
                cb(...args);
            }
        });
        this.storeSubscriptionRequest(settings, subscriptionRequest);
        this.dispatcher.sendSubscriptionRequest({
            from: settings.proxyId,
            toDiscoveryEntry: settings.providerDiscoveryEntry,
            messagingQos,
            subscriptionRequest
        });
    }
    addRequestToMulticastSubscribers(multicastId, subscriptionId) {
        const multicastIdPattern = this.multicastWildcardRegexFactory.createIdPattern(multicastId);
        if (this.multicastSubscribers[multicastIdPattern] === undefined) {
            this.multicastSubscribers[multicastIdPattern] = [];
        }
        const subscribers = this.multicastSubscribers[multicastIdPattern];
        for (let i = 0; i < subscribers.length; i++) {
            if (subscribers[i] === subscriptionId) {
                return;
            }
        }
        subscribers.push(subscriptionId);
    }
    createBroadcastSubscriptionRequest(parameters) {
        let request;
        if (parameters.selective) {
            request = new BroadcastSubscriptionRequest_1.default({
                subscriptionId: parameters.subscriptionId || nanoid_1.nanoid(),
                subscribedToName: parameters.broadcastName,
                qos: parameters.subscriptionQos,
                filterParameters: parameters.filterParameters
            });
        }
        else {
            request = new MulticastSubscriptionRequest_1.default({
                multicastId: SubscriptionUtil.createMulticastId(parameters.providerDiscoveryEntry.participantId, parameters.broadcastName, parameters.partitions),
                subscriptionId: parameters.subscriptionId || nanoid_1.nanoid(),
                subscribedToName: parameters.broadcastName,
                qos: parameters.subscriptionQos
            });
            this.addRequestToMulticastSubscribers(request.multicastId, request.subscriptionId);
        }
        return request;
    }
    /**
     * @name SubscriptionManager#registerBroadcastSubscription
     * @function
     * @param parameters
     * @param parameters.proxyId participantId of the sender
     * @param parameters.providerDiscoveryEntry DiscoveryEntry of the receiver
     * @param parameters.broadcastName the name of the broadcast being subscribed to
     * @param parameters.broadcastParameter the parameter meta information of the broadcast being subscribed to
     * @param [parameters.subscriptionQos] the subscriptionQos
     * @param [parameters.filterParameters] filter parameters used to indicate interest in
     *            only a subset of broadcasts that might be sent.
     * @param parameters.selective true if broadcast is selective
     * @param [parameters.partitions] partitions for multicast requests
     * @param parameters.subscriptionId optional parameter subscriptionId to reuse a
     *            pre-existing identifier for this concrete subscription request
     * @param {SubscriptionManager~onReceive} parameters.onReceive is called when a broadcast is received.
     * @param {SubscriptionManager~onError} parameters.onError is called when an error occurs with the broadcast
     * @param {SubscriptionManager~onSubscribed} parameters.onSubscribed the callback to inform once the subscription request has
     *            been delivered successfully
     * @param cb
     */
    registerBroadcastSubscriptionInternal(parameters, cb) {
        if (!this.isReady()) {
            cb(new Error("SubscriptionManager is already shut down"));
            return;
        }
        const subscriptionRequest = this.createBroadcastSubscriptionRequest(parameters);
        const subscriptionId = subscriptionRequest.subscriptionId;
        const ttl = this.calculateTtl(subscriptionRequest.qos);
        const messagingQos = new MessagingQos_1.default({ ttl });
        const timeout = LongTimer_1.default.setTimeout(() => {
            this.cleanupSubscription(subscriptionId);
            cb(new Error(`BroadcastSubscriptionRequest with id ${subscriptionId} failed: tll expired`));
        }, ttl);
        this.subscriptionReplyCallers.set(subscriptionId, {
            cb: (...args) => {
                LongTimer_1.default.clearTimeout(timeout);
                cb(...args);
            }
        });
        this.storeSubscriptionRequest(parameters, subscriptionRequest);
        this.dispatcher
            .sendBroadcastSubscriptionRequest({
            from: parameters.proxyId,
            toDiscoveryEntry: parameters.providerDiscoveryEntry,
            messagingQos,
            subscriptionRequest
        })
            .then(() => {
            const type = Typing.getObjectType(subscriptionRequest);
            if (type === "MulticastSubscriptionRequest") {
                const subscriptionReplyCaller = this.subscriptionReplyCallers.get(subscriptionId);
                const subscriptionListener = this.subscriptionListeners[subscriptionId];
                if (subscriptionReplyCaller !== undefined) {
                    subscriptionReplyCaller.cb(undefined, subscriptionId);
                }
                if (subscriptionListener !== undefined && subscriptionListener.onSubscribed !== undefined) {
                    subscriptionListener.onSubscribed(subscriptionId);
                }
                this.subscriptionReplyCallers.delete(subscriptionId);
                this.addRequestToMulticastSubscribers(subscriptionRequest.multicastId, subscriptionRequest.subscriptionId);
            }
        })
            .catch(error => {
            this.cleanupSubscription(subscriptionRequest.subscriptionId);
            if (parameters.onError) {
                parameters.onError(error);
            }
            // eslint-disable-next-line promise/no-callback-in-promise
            cb(error);
        });
    }
    /**
     * @param subscriptionReply incoming subscriptionReply
     */
    handleSubscriptionReply(subscriptionReply) {
        const subscriptionReplyCaller = this.subscriptionReplyCallers.get(subscriptionReply.subscriptionId);
        const subscriptionListener = this.subscriptionListeners[subscriptionReply.subscriptionId];
        if (subscriptionReplyCaller === undefined && subscriptionListener === undefined) {
            log.error(`error handling subscription reply, because subscriptionReplyCaller and subscriptionListener could not be found: ${JSONSerializer.stringify(subscriptionReply)}`);
            return;
        }
        try {
            if (subscriptionReply.error) {
                if (!(subscriptionReply.error instanceof Error)) {
                    subscriptionReply.error = Typing.augmentTypes(subscriptionReply.error);
                }
                if (subscriptionReplyCaller !== undefined) {
                    subscriptionReplyCaller.cb(subscriptionReply.error);
                }
                if (subscriptionListener !== undefined && subscriptionListener.onError !== undefined) {
                    subscriptionListener.onError(subscriptionReply.error);
                }
                this.cleanupSubscription(subscriptionReply.subscriptionId);
            }
            else {
                if (subscriptionReplyCaller !== undefined) {
                    subscriptionReplyCaller.cb(undefined, subscriptionReply.subscriptionId);
                }
                if (subscriptionListener !== undefined && subscriptionListener.onSubscribed !== undefined) {
                    subscriptionListener.onSubscribed(subscriptionReply.subscriptionId);
                }
                this.subscriptionReplyCallers.delete(subscriptionReply.subscriptionId);
            }
        }
        catch (e) {
            log.error(`exception thrown during handling subscription reply ${JSONSerializer.stringify(subscriptionReply)}:\n${e.stack}`);
            this.subscriptionReplyCallers.delete(subscriptionReply.subscriptionId);
        }
    }
    /**
     * @param publication incoming multicast publication
     */
    handleMulticastPublication(publication) {
        let subscribersFound = false;
        for (const multicastIdPattern in this.multicastSubscribers) {
            if (Object.prototype.hasOwnProperty.call(this.multicastSubscribers, multicastIdPattern)) {
                if (publication.multicastId.match(new RegExp(multicastIdPattern)) !== null) {
                    const subscribers = this.multicastSubscribers[multicastIdPattern];
                    if (subscribers !== undefined) {
                        subscribersFound = true;
                        for (let i = 0; i < subscribers.length; i++) {
                            const subscriptionListener = this.subscriptionListeners[subscribers[i]];
                            if (publication.error) {
                                if (subscriptionListener.onError) {
                                    subscriptionListener.onError(publication.error);
                                }
                                else {
                                    log.debug(`subscriptionListener with Id "${subscribers[i]}" has no onError callback. Skipping error publication`);
                                }
                            }
                            else if (publication.response) {
                                if (subscriptionListener.onReceive) {
                                    subscriptionListener.onReceive(publication.response);
                                }
                                else {
                                    log.debug(`subscriptionListener with Id "${subscribers[i]}" has no onReceive callback. Skipping multicast publication`);
                                }
                            }
                        }
                    }
                }
            }
        }
        if (!subscribersFound) {
            throw new Error(`${"Publication cannot be handled, as no subscription with multicastId "}${publication.multicastId} is known.`);
        }
    }
    /**
     * @param publication incoming publication
     */
    handlePublication(publication) {
        if (this.subscriptionInfos[publication.subscriptionId] === undefined) {
            throw new Error(`${"Publication cannot be handled, as no subscription with subscriptionId "}${publication.subscriptionId} is known.`);
        }
        this.setLastPublicationTime(publication.subscriptionId, Date.now());
        const subscriptionListener = this.subscriptionListeners[publication.subscriptionId];
        if (publication.error) {
            if (subscriptionListener.onError) {
                subscriptionListener.onError(publication.error);
            }
            else {
                log.debug(`subscriptionListener with Id "${publication.subscriptionId}" has no onError callback. Skipping error publication`);
            }
        }
        else if (publication.response) {
            if (subscriptionListener.onReceive) {
                subscriptionListener.onReceive(publication.response);
            }
            else {
                log.debug(`subscriptionListener with Id "${publication.subscriptionId}" has no onReceive callback. Skipping publication`);
            }
        }
    }
    /**
     * @param settings
     * @param settings.messagingQos the messaging Qos object for the ttl
     * @param settings.subscriptionId of the subscriptionId to stop
     * @returns A promise object
     */
    unregisterSubscription(settings) {
        if (!this.isReady()) {
            throw new Error("SubscriptionManager is already shut down");
        }
        const subscriptionInfo = this.subscriptionInfos[settings.subscriptionId];
        let errorMessage;
        if (subscriptionInfo === undefined) {
            errorMessage = `Cannot find subscription with id: ${settings.subscriptionId}`;
            log.error(errorMessage);
            return Promise.reject(new Error(errorMessage));
        }
        const subscriptionStop = new SubscriptionStop_1.default({
            subscriptionId: settings.subscriptionId
        });
        let promise;
        if (subscriptionInfo.multicastId !== undefined) {
            promise = this.dispatcher.sendMulticastSubscriptionStop({
                from: subscriptionInfo.proxyId,
                toDiscoveryEntry: subscriptionInfo.providerDiscoveryEntry,
                messagingQos: settings.messagingQos,
                multicastId: subscriptionInfo.multicastId,
                subscriptionStop
            });
        }
        else {
            promise = this.dispatcher.sendSubscriptionStop({
                from: subscriptionInfo.proxyId,
                toDiscoveryEntry: subscriptionInfo.providerDiscoveryEntry,
                messagingQos: settings.messagingQos,
                subscriptionStop
            });
        }
        this.cleanupSubscription(settings.subscriptionId);
        return promise;
    }
    hasMulticastSubscriptions() {
        return Object.keys(this.multicastSubscribers).length > 0;
    }
    hasOpenSubscriptions() {
        const hasSubscriptionInfos = Object.keys(this.subscriptionInfos).length > 0;
        const hasSubscriptionListeners = Object.keys(this.subscriptionListeners).length > 0;
        const hasPublicationCheckTimerIds = Object.keys(this.publicationCheckTimerIds).length > 0;
        const hasSubscriptionReplyCallers = this.subscriptionReplyCallers.size > 0;
        return (hasSubscriptionInfos ||
            hasSubscriptionListeners ||
            hasPublicationCheckTimerIds ||
            hasSubscriptionReplyCallers ||
            this.hasMulticastSubscriptions());
    }
    /**
     * This method is meant to be called by the runtime before shutdown is called.
     * It turns out that there is a necessary shutdown order and SubscriptionManager can't be shutdown first.
     *
     * @param timeoutMs timeout in ms after which this operation shall timeout. 0 defaults to no timeout.
     * @returns - resolves if subscriptionStop message has been sent for each active subscription
     * - rejects in case of any issues or timeout occurs
     */
    terminateSubscriptions(timeoutMs) {
        const logPrefix = "SubscriptionManager::terminateSubscriptions";
        log.info(`${logPrefix} ${timeoutMs}`);
        const cleanUpPromises = [];
        let activeSubscriptionId;
        for (activeSubscriptionId in this.subscriptionInfos) {
            if (Object.prototype.hasOwnProperty.call(this.subscriptionInfos, activeSubscriptionId)) {
                const promise = this.unregisterSubscription({
                    subscriptionId: activeSubscriptionId,
                    messagingQos: new MessagingQos_1.default({})
                });
                cleanUpPromises.push(promise);
            }
        }
        const cleanUpPromise = Promise.all(cleanUpPromises);
        log.info(`${logPrefix} terminating a total of ${cleanUpPromises.length} subscriptions`);
        return timeoutMs === 0 ? cleanUpPromise : UtilInternal.timeoutPromise(cleanUpPromise, timeoutMs);
    }
    /**
     * Shutdown the subscription manager
     */
    shutdown() {
        for (const subscriptionId in this.publicationCheckTimerIds) {
            if (Object.prototype.hasOwnProperty.call(this.publicationCheckTimerIds, subscriptionId)) {
                const timerId = this.publicationCheckTimerIds[subscriptionId];
                if (timerId !== undefined) {
                    LongTimer_1.default.clearTimeout(timerId);
                }
            }
        }
        this.publicationCheckTimerIds = {};
        for (const subscriptionReplyCaller of this.subscriptionReplyCallers.values()) {
            if (subscriptionReplyCaller) {
                subscriptionReplyCaller.cb(new Error("Subscription Manager is already shut down"));
            }
        }
        this.subscriptionReplyCallers.clear();
        this.started = false;
    }
}
module.exports = SubscriptionManager;
