"use strict";
/*
 * #%L
 * %%
 * Copyright (C) 2019 BMW Car IT GmbH
 * %%
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * #L%
 */
const MESSAGE_CUSTOM_HEADER_PREFIX = "c-";
const nanoid_1 = require("nanoid");
const jmBase = nanoid_1.nanoid();
let jmIndex = 0;
class JoynrMessage {
    /**
     * @param payload
     * @param type the message type as defined by JoynrMessage.JOYNRMESSAGE_TYPE_*
     * @param msgId
     */
    constructor({ payload, type, msgId }) {
        this.headers = {};
        this.payload = payload;
        this.type = type;
        this.msgId = msgId || `${jmBase}_${jmIndex++}`;
    }
    static setSigningCallback(callback) {
        JoynrMessage.prototype.signingCallback = callback;
    }
    static parseMessage(settings) {
        settings.headers = settings.headers || {};
        settings.headers.id = settings.headers.id || `${jmBase}_${jmIndex++}`;
        Object.setPrototypeOf(settings, JoynrMessage.prototype);
        return settings;
    }
    /**
     * @param customHeaders a map containing key/value pairs of headers to be set as custom
     *            headers. The keys will be added to the header field with the prefix
     *            MESSAGE_CUSTOM_HEADER_PREFIX
     * @returns joynrMessage instance for chainable calls
     */
    setCustomHeaders(customHeaders) {
        let headerKey;
        for (headerKey in customHeaders) {
            if (Object.prototype.hasOwnProperty.call(customHeaders, headerKey)) {
                this.headers[MESSAGE_CUSTOM_HEADER_PREFIX + headerKey] = customHeaders[headerKey];
            }
        }
        return this;
    }
    /**
     * @returns customHeader object containing all headers that begin with the
     *          prefix MESSAGE_CUSTOM_HEADER_PREFIX
     */
    getCustomHeaders() {
        const customHeaders = {};
        for (const headerKey in this.headers) {
            if (Object.prototype.hasOwnProperty.call(this.headers, headerKey) &&
                headerKey.substr(0, MESSAGE_CUSTOM_HEADER_PREFIX.length) === MESSAGE_CUSTOM_HEADER_PREFIX) {
                const trimmedKey = headerKey.substr(MESSAGE_CUSTOM_HEADER_PREFIX.length);
                customHeaders[trimmedKey] = this.headers[headerKey];
            }
        }
        return customHeaders;
    }
    /**
     * @param key is one of the header keys defined in JoynrMessagingDefines
     * @param value of the header
     * @returns joynrMessage instance for chainable calls
     */
    setHeader(key, value) {
        this.headers[key] = value;
        return this;
    }
    /**
     * @param key is one of the header keys defined in JoynrMessagingDefines
     * @returns value of header key
     */
    getHeader(key) {
        return this.headers[key];
    }
    get type() {
        return this.headers.t;
    }
    set type(value) {
        this.headers.t = value;
    }
    /**
     * @returns The participant id the message is to
     */
    get to() {
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        return this.recipient;
    }
    /**
     * @param value The participant id the message is to
     */
    set to(value) {
        this.recipient = value;
    }
    /**
     * @returns The participant id the message is from
     */
    get from() {
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        return this.sender;
    }
    /**
     * @param value The participant id the message is from
     */
    set from(value) {
        this.sender = value;
    }
    /**
     * @param value The expiry date of the message
     */
    set expiryDate(value) {
        this.ttlMs = value;
    }
    /**
     * @returns The expiry date of the message
     */
    get expiryDate() {
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        return this.ttlMs;
    }
    /**
     * @param value The reply channel Id to return response messages to
     */
    set replyChannelId(value) {
        this.headers.re = value;
    }
    /**
     * @returns The reply channel Id to return response messages to
     */
    get replyChannelId() {
        return this.headers.re;
    }
    set msgId(value) {
        this.headers.id = value;
    }
    get msgId() {
        return this.headers.id;
    }
    set compress(value) {
        this.isCompressed = value;
    }
    get compress() {
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        return this.isCompressed;
    }
    /**
     * @param value The effort to be expended while delivering the message
     */
    set effort(value) {
        this.headers.ef = value;
    }
    /**
     * @returns The effort to be expended while delivering the message
     */
    get effort() {
        return this.headers.ef;
    }
}
JoynrMessage.JOYNRMESSAGE_HEADER_TO_PARTICIPANT_ID = "to";
JoynrMessage.JOYNRMESSAGE_HEADER_CONTENT_TYPE = "contentType";
JoynrMessage.JOYNRMESSAGE_HEADER_REPLY_CHANNELID = "replyChannelId";
JoynrMessage.JOYNRMESSAGE_HEADER_EFFORT = "effort";
JoynrMessage.JOYNRMESSAGE_HEADER_CREATOR_USER_ID = "creator";
JoynrMessage.JOYNRMESSAGE_TYPE_SUBSCRIPTION_STOP = "sst";
JoynrMessage.JOYNRMESSAGE_TYPE_MULTICAST = "m";
JoynrMessage.JOYNRMESSAGE_TYPE_PUBLICATION = "p";
JoynrMessage.JOYNRMESSAGE_TYPE_SUBSCRIPTION_REPLY = "srp";
JoynrMessage.JOYNRMESSAGE_TYPE_BROADCAST_SUBSCRIPTION_REQUEST = "brq";
JoynrMessage.JOYNRMESSAGE_TYPE_MULTICAST_SUBSCRIPTION_REQUEST = "mrq";
JoynrMessage.JOYNRMESSAGE_TYPE_SUBSCRIPTION_REQUEST = "arq";
JoynrMessage.JOYNRMESSAGE_TYPE_REPLY = "rp";
JoynrMessage.JOYNRMESSAGE_TYPE_REQUEST = "rq";
JoynrMessage.JOYNRMESSAGE_TYPE_ONE_WAY = "o";
JoynrMessage.prototype._typeName = "joynr.JoynrMessage";
JoynrMessage.prototype.isTtlAbsolute = true;
JoynrMessage.prototype.encryptionCert = null;
JoynrMessage.prototype.signingCert = null;
JoynrMessage.prototype.signingKey = null;
JoynrMessage.prototype.isReceivedFromGlobal = false;
JoynrMessage.prototype.isLocalMessage = false;
JoynrMessage.prototype.isCompressed = false;
module.exports = JoynrMessage;
