"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
/*
 * #%L
 * %%
 * Copyright (C) 2019 BMW Car IT GmbH
 * %%
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * #L%
 */
const util_1 = __importDefault(require("util"));
const mqtt_1 = __importDefault(require("mqtt"));
const MessagingQosEffort = __importStar(require("../MessagingQosEffort"));
const Typing = __importStar(require("../../util/Typing"));
const MessageSerializer = __importStar(require("../MessageSerializer"));
const UtilInternal = __importStar(require("../../util/UtilInternal"));
const LoggingManager = require("../../system/LoggingManager");
const log = LoggingManager.getLogger("joynr/messaging/routing/MessageRouter");
class SharedMqttClient {
    /**
     * @constructor
     * @param settings
     * @param settings.address to be used to connect to mqtt broker
     */
    constructor(settings) {
        this.queuedUnsubscriptions = [];
        this.queuedSubscriptions = [];
        this.connected = false;
        this.closed = false;
        this.queuedMessages = [];
        Typing.checkProperty(settings, "Object", "settings");
        Typing.checkProperty(settings.address, "MqttAddress", "settings.address");
        this.address = settings.address;
        this.onConnectedDeferred = UtilInternal.createDeferred();
        this.qosLevel =
            settings.provisioning.qosLevel !== undefined
                ? settings.provisioning.qosLevel
                : SharedMqttClient.DEFAULT_QOS_LEVEL;
        this.onOpen = this.onOpen.bind(this);
        this.onMessage = this.onMessage.bind(this);
        this.resetConnection();
    }
    set onmessage(newCallback) {
        this.onmessageCallback = newCallback;
        if (typeof newCallback !== "function") {
            throw new Error(`onmessage callback must be a function, but instead was of type ${typeof newCallback}`);
        }
    }
    get onmessage() {
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        return this.onmessageCallback;
    }
    onMessage(topic, payload) {
        if (this.onmessageCallback !== undefined) {
            this.onmessageCallback(topic, MessageSerializer.parse(payload));
        }
    }
    onError(error) {
        log.error(`MqttClient emitted error ${error}`);
    }
    resetConnection() {
        if (this.closed) {
            return;
        }
        this.client = mqtt_1.default.connect(this.address.brokerUri);
        this.client.on("connect", this.onOpen);
        this.client.on("message", this.onMessage);
        this.client.on("error", this.onError);
    }
    onConnected() {
        return this.onConnectedDeferred.promise;
    }
    sendQueuedUnsubscriptions() {
        for (let i = 0; i < this.queuedUnsubscriptions.length; i++) {
            this.client.unsubscribe(this.queuedUnsubscriptions[i]);
        }
        this.queuedUnsubscriptions = [];
    }
    sendQueuedMessages() {
        while (this.queuedMessages.length) {
            const queued = this.queuedMessages.shift();
            try {
                this.client.publish(queued.topic, MessageSerializer.stringify(queued.message), queued.options);
                queued.resolve();
                // Error is thrown if the connection is no longer open
            }
            catch (e) {
                // so add the message back to the front of the queue
                this.queuedMessages.unshift(queued);
                throw e;
            }
        }
    }
    sendQueuedSubscriptions() {
        const subscribeObject = {};
        for (let i = 0; i < this.queuedSubscriptions.length; i++) {
            const topic = this.queuedSubscriptions[i];
            subscribeObject[topic] = { qos: this.qosLevel };
        }
        this.queuedSubscriptions = [];
        this.client.subscribe(subscribeObject, (err, _granted) => {
            if (err) {
                log.error(`Error subscribing to topics ${util_1.default.inspect(subscribeObject)}`);
                this.client.reconnect();
            }
            else {
                log.info("onConnection resolved");
                this.onConnectedDeferred.resolve();
            }
        });
    }
    /* send all queued messages, requeuing to the front in case of a problem*/
    onOpen() {
        try {
            this.connected = true;
            this.sendQueuedMessages();
            this.sendQueuedUnsubscriptions();
            this.sendQueuedSubscriptions();
        }
        catch (e) {
            log.error("Error opening MqttClient connection");
            this.resetConnection();
        }
    }
    /**
     * @param topic the topic to publish the message
     * @param joynrMessage the joynr message to transmit
     */
    send(topic, joynrMessage) {
        let sendQosLevel = this.qosLevel;
        if (MessagingQosEffort.BEST_EFFORT.value === joynrMessage.effort) {
            sendQosLevel = SharedMqttClient.BEST_EFFORT_QOS_LEVEL;
        }
        const deferred = UtilInternal.createDeferred();
        try {
            this.client.publish(topic, MessageSerializer.stringify(joynrMessage), {
                qos: sendQosLevel
            });
            deferred.resolve();
            // Error is thrown if the socket is no longer open, so requeue to the front
        }
        catch (e) {
            // add the message back to the front of the queue
            this.queuedMessages.unshift({
                message: joynrMessage,
                resolve: deferred.resolve,
                options: {
                    qos: sendQosLevel
                },
                topic
            });
            log.error(`failed to send Message ${util_1.default.inspect(joynrMessage)} due to ${e}`);
            this.client.reconnect();
        }
        return deferred.promise;
    }
    close() {
        log.info("SharedMqttClient::onClose");
        this.closed = true;
        if (this.client && this.client.end) {
            this.client.end();
        }
    }
    subscribe(topic) {
        if (this.connected) {
            this.client.subscribe(topic, { qos: this.qosLevel });
        }
        else {
            this.queuedSubscriptions.push(topic);
        }
    }
    unsubscribe(topic) {
        if (this.connected) {
            this.client.unsubscribe(topic);
        }
        else {
            this.queuedUnsubscriptions.push(topic);
        }
    }
}
SharedMqttClient.BEST_EFFORT_QOS_LEVEL = 0;
SharedMqttClient.DEFAULT_QOS_LEVEL = 1;
module.exports = SharedMqttClient;
