"use strict";
/*
 * #%L
 * %%
 * Copyright (C) 2019 BMW Car IT GmbH
 * %%
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * #L%
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
/**
 * The <code>MessageQueue</code> is a joynr internal data structure. The Message Queue caches incoming messages, which cannot be shipped
 * to the correct participant. Once a participant with the matching participantId is registered, the incoming message is forwarded to him
 */
const LoggingManager_1 = __importDefault(require("../../system/LoggingManager"));
const DiagnosticTags = __importStar(require("../../system/DiagnosticTags"));
const UtilInternal = __importStar(require("../../util/UtilInternal"));
const ParticipantQueue_1 = __importDefault(require("./ParticipantQueue"));
const log = LoggingManager_1.default.getLogger("joynr/messaging/routing/MessageQueue");
const CHECK_TTL_ON_QUEUED_MESSAGES_INTERVAL_MS = 10000; // a very loose interval because of a second check on return
class MessageQueue {
    /**
     * MessageQueue caches incoming messages, and cached messages can be retrieved in case the destination participant becomes visible
     *
     * @constructor
     *
     * @param settings the settings object holding dependencies
     * @param settings.maxQueueSizeInKBytes the maximum buffer size that shall be allocated by the message queue
     *
     * @classdesc The <code>MessageQueue</code> is a joynr internal data structure. The Message Queue caches incoming messages, which cannot be shipped
     * to the correct participant. Once a participant with the matching participantId is registered, the incoming message is forwarded to him
     */
    constructor(settings) {
        this.currentQueueSize = 0;
        this.participantQueues = {};
        this.maxQueueSizeInKBytes = MessageQueue.DEFAULT_MAX_QUEUE_SIZE_IN_KBYTES;
        if (settings.maxQueueSizeInKBytes !== undefined)
            this.maxQueueSizeInKBytes = settings.maxQueueSizeInKBytes;
        this.cleanupInterval = setInterval(() => {
            // TODO: we could call this way more lazy -> make an if and only call this if this.currentQueueSize > 100
            let newSize = 0;
            for (const id in this.participantQueues) {
                if (Object.prototype.hasOwnProperty.call(this.participantQueues, id)) {
                    this.participantQueues[id].filterExpiredMessages();
                    const size = this.participantQueues[id].getSize();
                    newSize += size;
                    if (size === 0) {
                        delete this.participantQueues[id];
                    }
                }
            }
            this.currentQueueSize = newSize;
        }, CHECK_TTL_ON_QUEUED_MESSAGES_INTERVAL_MS);
    }
    putMessage(message) {
        // drop message if maximum queue size has been reached
        if (message.payload !== undefined) {
            const messageSize = UtilInternal.getLengthInBytes(message.payload);
            if (this.currentQueueSize + messageSize <= this.maxQueueSizeInKBytes * 1024) {
                this.currentQueueSize = this.currentQueueSize + messageSize;
                if (this.participantQueues[message.to] === undefined) {
                    this.participantQueues[message.to] = new ParticipantQueue_1.default();
                }
                this.participantQueues[message.to].putMessage(message, messageSize);
            }
            else {
                log.error("message cannot be added to message queue, as the queue buffer size has been exceeded", DiagnosticTags.forJoynrMessage(message));
            }
        }
    }
    /**
     * gets the queue messages for the participant
     *
     * @param participantId
     */
    getAndRemoveMessages(participantId) {
        let result = [];
        if (this.participantQueues[participantId] !== undefined) {
            const participantQueue = this.participantQueues[participantId];
            this.currentQueueSize -= participantQueue.getSize();
            participantQueue.filterExpiredMessages();
            result = participantQueue.getMessages();
            delete this.participantQueues[participantId];
        }
        return result;
    }
    /**
     * Shutdown the message queue
     */
    shutdown() {
        this.participantQueues = {};
        this.currentQueueSize = 0;
        clearInterval(this.cleanupInterval);
    }
}
MessageQueue.DEFAULT_MAX_QUEUE_SIZE_IN_KBYTES = 10000;
module.exports = MessageQueue;
