import * as RoutingProxy from "../../../generated/joynr/system/RoutingProxy";
import JoynrMessage from "../JoynrMessage";
import MessagingStubFactory = require("../MessagingStubFactory");
import MessageQueue = require("./MessageQueue");
import Address = require("../../../generated/joynr/system/RoutingTypes/Address");
declare type RoutingTable = Record<string, Address>;
declare type MulticastSkeletons = Record<string, any>;
interface MulticastAddressCalculator {
    calculate: (joynrMessage: JoynrMessage) => Address;
}
declare namespace MessageRouter {
    interface MessageRouterSettings {
        routingTable?: RoutingTable;
        messagingStubFactory: MessagingStubFactory;
        incomingAddress?: Address;
        parentMessageRouterAddress?: Address;
        messageQueue: MessageQueue;
        initialRoutingTable?: RoutingTable;
        multicastAddressCalculator: MulticastAddressCalculator;
        multicastSkeletons: MulticastSkeletons;
    }
}
declare class MessageRouter {
    private settings;
    private started;
    private messagesWithoutReplyTo;
    private messageReplyToAddressCalculator;
    private replyToAddress;
    private multicastReceiversRegistry;
    private multicastSkeletons;
    private multicastAddressCalculator;
    private parentMessageRouterAddress?;
    private incomingAddress?;
    private routingTable;
    private queuedRemoveMulticastReceiverCalls;
    private queuedAddMulticastReceiverCalls;
    private queuedRemoveNextHopCalls;
    private queuedAddNextHopCalls;
    private messagingStub;
    private routingProxy;
    private multicastWildcardRegexFactory;
    /**
     * Message Router receives a message and forwards it to the correct endpoint, as looked up in the {@link RoutingTable} @constructor
     *
     * @param settings the settings object holding dependencies
     * @param settings.routingTable
     * @param settings.messagingStubFactory
     * @param settings.incomingAddress
     * @param settings.parentMessageRouterAddress
     * @param settings.messageQueue
     * @param settings.initialRoutingTable
     * @param settings.multicastAddressCalculator
     * @param settings.multicastSkeletons
     *
     * @classdesc The <code>MessageRouter</code> is a joynr internal interface. The Message
     * Router receives messages from Message Receivers, and forwards them along using to the
     * appropriate Address, either <code>{@link ChannelAddress}</code> for messages being
     * sent to a joynr channel via HTTP, <code>{@link BrowserAddress}</code> for messages
     * going to applications running within a seperate browser tab, or
     * <code>{@link InProcessAddress}</code> for messages going to a dispatcher running
     * within the same JavaScript scope as the MessageRouter.
     *
     * MessageRouter is part of the cluster controller, and is used for
     * internal messaging only.
     */
    constructor(settings: MessageRouter.MessageRouterSettings);
    private isReady;
    private addRoutingProxyToParentRoutingTable;
    private processQueuedRoutingProxyCalls;
    /**
     * This method is called when no address can be found in the local routing table.
     * It tries to resolve the next hop from parent router.
     */
    private resolveNextHopInternal;
    private containsAddress;
    /**
     * Get the address to which the passed in message should be sent to.
     * This is a multicast address calculated from the header content of the message.
     *
     * @param message the message for which we want to find an address to send it to.
     * @return the address to send the message to. Will not be null, because if an address can't be determined an exception is thrown.
     */
    private getAddressesForMulticast;
    private routeInternalTransmitOnError;
    /**
     * Helper function to route a message once the address is known
     */
    private routeInternal;
    private registerGlobalRoutingEntryIfRequired;
    private resolveNextHopAndRoute;
    /**
     * @param newAddress - the address to be used as replyTo  address
     *
     * @returns void
     */
    setReplyToAddress(newAddress: string): void;
    /**
     * @param participantId
     *
     * @returns promise
     */
    removeNextHop(participantId: string): Promise<any>;
    /**
     * @param participantId
     * @param isGloballyVisible
     *
     * @returns promise
     */
    private addNextHopToParentRoutingTable;
    /**
     * @param newRoutingProxy - the routing proxy to be set
     * @returns A+ promise object
     */
    setRoutingProxy(newRoutingProxy: RoutingProxy): Promise<void>;
    /**
     * Get replyToAddress from routing proxy
     * @returns A+ promise object
     */
    getReplyToAddressFromRoutingProxy(): Promise<string>;
    /**
     * Get replyToAddress via routing proxy to enable global communication in message router.
     * @returns A+ promise object
     */
    configureReplyToAddressFromRoutingProxy(): Promise<void>;
    /**
     * Looks up an Address for a given participantId (next hop)
     *
     * @param participantId
     * @returns the address of the next hop in the direction of the given
     *          participantId, or undefined if not found
     */
    resolveNextHop(participantId: string): Promise<Address>;
    /**
     * @param joynrMessage
     * @returns A+ promise object
     */
    route(joynrMessage: JoynrMessage): Promise<any>;
    /**
     * Registers the next hop with this specific participant Id
     *
     * @param participantId
     * @param address the address to register
     * @param isGloballyVisible
     * @returns A+ promise object
     */
    addNextHop(participantId: string, address: Address, isGloballyVisible: boolean): Promise<void>;
    /**
     * Adds a new receiver for the identified multicasts.
     *
     * @param parameters - object containing parameters
     * @param parameters.multicastId
     * @param parameters.subscriberParticipantId
     * @param parameters.providerParticipantId
     * @returns A+ promise object
     */
    addMulticastReceiver(parameters: {
        multicastId: string;
        subscriberParticipantId: string;
        providerParticipantId: string;
    }): Promise<void>;
    /**
     * Removes a receiver for the identified multicasts.
     *
     * @param parameters - object containing parameters
     * @param parameters.multicastId
     * @param parameters.subscriberParticipantId
     * @param parameters.providerParticipantId
     * @returns A+ promise object
     */
    removeMulticastReceiver(parameters: {
        multicastId: string;
        subscriberParticipantId: string;
        providerParticipantId: string;
    }): Promise<void>;
    /**
     * @param participantId
     *
     * @returns void
     */
    participantRegistered(participantId: string): void;
    /**
     * Tell the message router that the given participantId is known. The message router
     * checks internally if an address is already present in the routing table. If not,
     * it adds the parentMessageRouterAddress to the routing table for this participantId.
     *
     * @param participantId
     *
     * @returns void
     */
    setToKnown(participantId: string): void;
    hasMulticastReceivers(): boolean;
    /**
     * Shutdown the message router
     */
    shutdown(): void;
}
export = MessageRouter;
