"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
/* eslint-disable @typescript-eslint/no-non-null-assertion */
/*
 * #%L
 * %%
 * Copyright (C) 2019 BMW Car IT GmbH
 * %%
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * #L%
 */
/*eslint no-use-before-define: "off"*/
const util_1 = __importDefault(require("util"));
const MessageSerializer = __importStar(require("../MessageSerializer"));
const WebSocketNode = require("../../../global/WebSocketNode");
const LongTimer_1 = __importDefault(require("../../util/LongTimer"));
const LoggingManager_1 = __importDefault(require("../../system/LoggingManager"));
const log = LoggingManager_1.default.getLogger("joynr.messaging.websocket.SharedWebSocket");
/**
 * @param address
 * @param address to which messages are sent on the clustercontroller.
 * @returns a url
 */
function webSocketAddressToUrl(address) {
    let url = `${address.protocol.name.toLowerCase()}://${address.host}:${address.port}`;
    if (address.path) {
        url += address.path;
    }
    return url;
}
class SharedWebSocket {
    /**
     * @constructor
     * @param settings
     * @param settings.localAddress address used by the websocket server to contact this
     *            client. This address is used in in the init phase on the websocket, and
     *            then registered with the message router on the remote side
     * @param settings.remoteAddress to which messages are sent on the websocket server.
     * @param settings.provisioning
     * @param settings.provisioning.reconnectSleepTimeMs
     * @param settings.keychain
     */
    constructor(settings) {
        this.sendConfig = { binary: true };
        this.closed = false;
        this.queuedMessages = [];
        this.onmessageCallback = null;
        this.websocket = null;
        settings.provisioning = settings.provisioning || {};
        this.reconnectSleepTimeMs = settings.provisioning.reconnectSleepTimeMs || 1000;
        this.localAddress = settings.localAddress;
        this.remoteUrl = webSocketAddressToUrl(settings.remoteAddress);
        this.webSocketSendAsync = util_1.default.promisify((marshaledMessage, cb) => this.websocket.send(marshaledMessage, this.sendConfig, cb));
        this.keychain = settings.keychain;
        this.onOpen = this.onOpen.bind(this);
        this.onClose = this.onClose.bind(this);
        this.onError = this.onError.bind(this);
        this.resetConnection = this.resetConnection.bind(this);
        this.resetConnection();
    }
    get onmessage() {
        return this.onmessageCallback;
    }
    set onmessage(newCallback) {
        this.onmessageCallback = (data) => {
            try {
                const joynrMessage = MessageSerializer.parse(data.data);
                if (joynrMessage) {
                    newCallback(joynrMessage);
                }
            }
            catch (e) {
                log.error(`could not unmarshal joynrMessage: ${e}`);
            }
        };
        this.websocket.onmessage = this.onmessageCallback;
    }
    get numberOfQueuedMessages() {
        return this.queuedMessages.length;
    }
    sendQueuedMessages() {
        while (this.queuedMessages.length) {
            const queued = this.queuedMessages.shift();
            try {
                this.websocket.send(queued, { binary: true });
                // Error is thrown if the socket is no longer open
            }
            catch (e) {
                // so add the message back to the front of the queue
                this.queuedMessages.unshift(queued);
                throw e;
            }
        }
    }
    resetConnection() {
        this.reconnectTimer = undefined;
        if (this.closed) {
            return;
        }
        this.websocket = new WebSocketNode(this.remoteUrl, this.keychain);
        this.websocket.onopen = this.onOpen;
        this.websocket.onclose = this.onClose;
        this.websocket.onerror = this.onError;
        if (this.onmessageCallback !== null) {
            this.websocket.onmessage = this.onmessageCallback;
        }
    }
    onError(event) {
        if (this.closed) {
            return;
        }
        log.error(`error in websocket: ${util_1.default.inspect(event)}. Resetting connection`);
        if (this.reconnectTimer !== undefined) {
            LongTimer_1.default.clearTimeout(this.reconnectTimer);
        }
        if (this.websocket) {
            this.websocket.close(SharedWebSocket.EVENT_CODE_SHUTDOWN, "shutdown");
            this.websocket = null;
        }
        this.reconnectTimer = LongTimer_1.default.setTimeout(this.resetConnection, this.reconnectSleepTimeMs);
    }
    onClose(event) {
        if (this.closed) {
            return;
        }
        if (event.code !== SharedWebSocket.EVENT_CODE_SHUTDOWN) {
            log.info(`connection closed unexpectedly. code: ${event.code} reason: ${event.reason}. Trying to reconnect...`);
            if (this.reconnectTimer !== undefined) {
                LongTimer_1.default.clearTimeout(this.reconnectTimer);
            }
            if (this.websocket) {
                this.websocket.close(SharedWebSocket.EVENT_CODE_SHUTDOWN, "shutdown");
                this.websocket = null;
            }
            this.reconnectTimer = LongTimer_1.default.setTimeout(this.resetConnection, this.reconnectSleepTimeMs);
        }
        else {
            log.info(`connection closed. reason: ${event.reason}`);
            if (this.websocket) {
                this.websocket.close(SharedWebSocket.EVENT_CODE_SHUTDOWN, "shutdown");
                this.websocket = null;
            }
        }
    }
    /* send all queued messages, requeuing to the front in case of a problem*/
    onOpen() {
        try {
            log.debug("connection opened.");
            this.websocket.send(Buffer.from(JSON.stringify(this.localAddress)), this.sendConfig);
            this.sendQueuedMessages();
        }
        catch (e) {
            this.resetConnection();
        }
    }
    async sendInternal(marshaledMessage) {
        this.websocket.send(marshaledMessage, this.sendConfig);
    }
    async sendMessage(joynrMessage) {
        let marshaledMessage;
        try {
            marshaledMessage = MessageSerializer.stringify(joynrMessage);
        }
        catch (e) {
            log.error(`could not marshal joynrMessage: ${joynrMessage.msgId} ${e}`);
            return Promise.resolve();
        }
        if (this.websocket !== null && this.websocket.readyState === WebSocketNode.WSN_OPEN) {
            try {
                await this.sendInternal(marshaledMessage);
                // Error is thrown if the socket is no longer open, so requeue to the front
            }
            catch (e) {
                // add the message back to the front of the queue
                this.queuedMessages.unshift(marshaledMessage);
                log.error(`could not send joynrMessage: ${joynrMessage.msgId} requeuing message. Error: ${e}`);
            }
        }
        else {
            // push new messages onto the back of the queue
            this.queuedMessages.push(marshaledMessage);
        }
    }
    /**
     * @param joynrMessage the joynr message to transmit
     */
    send(joynrMessage) {
        log.debug(`>>> OUTGOING >>> message with ID ${joynrMessage.msgId}`);
        return this.sendMessage(joynrMessage);
    }
    /**
     * Normally the SharedWebsocket.send api automatically resolves the Promise
     * when it's called. But this doesn't mean that the data was actually
     * written out. This method is a helper for a graceful shutdown which delays
     * the resolving of the SharedWebSocket.send Promise till the data is
     * written out, to make sure that unsubscribe messages are successfully sent.
     */
    enableShutdownMode() {
        this.sendInternal = this.webSocketSendAsync;
    }
    close() {
        this.closed = true;
        if (this.reconnectTimer !== undefined) {
            LongTimer_1.default.clearTimeout(this.reconnectTimer);
            this.reconnectTimer = undefined;
        }
        if (this.websocket !== null) {
            this.websocket.close(SharedWebSocket.EVENT_CODE_SHUTDOWN, "shutdown");
            this.websocket = null;
        }
    }
}
SharedWebSocket.EVENT_CODE_SHUTDOWN = 4000;
module.exports = SharedWebSocket;
