declare type ProviderAttributeConstructor<T = ProviderAttribute> = new (...args: any[]) => T;
interface JoynrProvider {
    interfaceName: string;
}
export declare class ProviderAttribute<T = any> {
    hasNotify: boolean;
    attributeType: string;
    attributeName: string;
    /** protected property, needs to be public for tsc -d */
    implementation: {
        get?: (key: string) => any | Promise<any>;
        set?: (value: T) => void | Promise<void>;
        valueChanged?: (value: T) => void;
    };
    parent: JoynrProvider;
    hasRead: boolean;
    hasWrite: boolean;
    /** protected property, needs to be public for tsc -d */
    privateGetterFunc?: Function;
    /** protected property, needs to be public for tsc -d */
    privateSetterFunc?: (value: T) => void | Promise<void>;
    /**
     * Constructor of ProviderAttribute object that is used in the generation of provider attributes
     *
     * @constructor
     *
     * @param parent - the provider object that contains this attribute
     * @param [implementation] - the definition of attribute implementation
     * @param {Function} [implementation.set] - the getter function with the signature "function(value){}" that
     *          stores the given attribute value
     * @param {Function} [implementation.get] the getter function with the signature "function(){}" that returns the
     *          current attribute value
     * @param attributeName - the name of the attribute
     * @param attributeType - the type of the attribute
     */
    constructor(parent: JoynrProvider, implementation: {
        get?: (key: string) => T | Promise<T>;
        set?: (value: T) => void | Promise<void>;
    }, attributeName: string, attributeType: string);
    isNotifiable(): boolean;
    /**
     * Check Getter and Setter functions.
     */
    check(): boolean;
}
declare const ProviderReadAttributeImpl: {
    new (...args: any[]): {
        registerGetter(getterFunc: () => unknown): void;
        /**
         * Calls through the getter registered with registerGetter with the same arguments as this
         * function
         *
         * @returns a Promise which resolves the attribute value
         *
         * rejects {Error} if no getter function was registered before calling it
         * rejects {Error} if registered getter returns a compound type with incorrect values
         */
        get(): Promise<unknown[]>;
        hasNotify: boolean;
        attributeType: string;
        attributeName: string;
        /** protected property, needs to be public for tsc -d */
        implementation: {
            get?: ((key: string) => any) | undefined;
            set?: ((value: any) => void | Promise<void>) | undefined;
            valueChanged?: ((value: any) => void) | undefined;
        };
        parent: JoynrProvider;
        hasRead: boolean;
        hasWrite: boolean;
        /** protected property, needs to be public for tsc -d */
        privateGetterFunc?: Function | undefined;
        /** protected property, needs to be public for tsc -d */
        privateSetterFunc?: ((value: any) => void | Promise<void>) | undefined;
        isNotifiable(): boolean;
        /**
         * Check Getter and Setter functions.
         */
        check(): boolean;
    };
} & ProviderAttributeConstructor<ProviderAttribute<any>>;
export declare class ProviderReadAttribute<T> extends ProviderReadAttributeImpl {
    get: () => Promise<T[]>;
    registerGetter: (getterFunc: () => T | Promise<T>) => void;
}
declare const ProviderReadWriteAttributeImpl: {
    new (...args: any[]): {
        /**
         * Registers the setter function for this attribute
         *
         * @param setterFunc - the setter function with the signature
         *          'void setterFunc({?}value) {..}'
         * @returns fluent interface to call multiple methods
         */
        registerSetter(setterFunc: (value: unknown) => void): ProviderAttribute<any>;
        /**
         * Calls through the setter registered with registerSetter with the same arguments as this
         * function
         *
         * @param value - the new value of the attribute
         *
         * @throws {Error} if no setter function was registered before calling it
         *
         * @see ProviderAttribute#registerSetter
         */
        set(value: unknown): Promise<[]>;
        hasNotify: boolean;
        attributeType: string;
        attributeName: string;
        /** protected property, needs to be public for tsc -d */
        implementation: {
            get?: ((key: string) => any) | undefined;
            set?: ((value: any) => void | Promise<void>) | undefined;
            valueChanged?: ((value: any) => void) | undefined;
        };
        parent: JoynrProvider;
        hasRead: boolean;
        hasWrite: boolean;
        /** protected property, needs to be public for tsc -d */
        privateGetterFunc?: Function | undefined;
        /** protected property, needs to be public for tsc -d */
        privateSetterFunc?: ((value: any) => void | Promise<void>) | undefined;
        isNotifiable(): boolean;
        /**
         * Check Getter and Setter functions.
         */
        check(): boolean;
    };
} & ProviderAttributeConstructor<ProviderAttribute<any>>;
export declare class ProviderReadWriteAttribute<T> extends ProviderReadWriteAttributeImpl {
    get: () => Promise<T[]>;
    registerGetter: (getterFunc: () => T | Promise<T>) => void;
    registerSetter: (setterFunc: (value: T) => void) => ProviderReadWriteAttribute<T>;
    set: (value: T) => Promise<[]>;
}
declare const ProviderReadWriteNotifyAttributeImpl: {
    new (...args: any[]): {
        callbacks: ((value: unknown) => void)[];
        /**
         * If this attribute is changed the application should call this function with the new value,
         * whereafter the new value gets published
         *
         * @param value - the new value of the attribute
         */
        valueChanged(value: unknown): void;
        /**
         * Registers an Observer for value changes
         *
         * @param observer - the callback function with the signature "function(value){..}"
         *
         * @see ProviderAttribute#valueChanged
         * @see ProviderAttribute#unregisterObserver
         */
        registerObserver(observer: (value: unknown) => void): void;
        /**
         * Unregisters an Observer for value changes
         *
         * @param observer - the callback function with the signature "function(value){..}"
         *
         * @see ProviderAttribute#valueChanged
         * @see ProviderAttribute#registerObserver
         */
        unregisterObserver(observer: (value: unknown) => void): void;
        hasNotify: boolean;
        attributeType: string;
        attributeName: string;
        /** protected property, needs to be public for tsc -d */
        implementation: {
            get?: ((key: string) => any) | undefined;
            set?: ((value: any) => void | Promise<void>) | undefined;
            valueChanged?: ((value: any) => void) | undefined;
        };
        parent: JoynrProvider;
        hasRead: boolean;
        hasWrite: boolean;
        /** protected property, needs to be public for tsc -d */
        privateGetterFunc?: Function | undefined;
        /** protected property, needs to be public for tsc -d */
        privateSetterFunc?: ((value: any) => void | Promise<void>) | undefined;
        isNotifiable(): boolean;
        /**
         * Check Getter and Setter functions.
         */
        check(): boolean;
    };
} & ProviderAttributeConstructor<ProviderAttribute<any>>;
export declare class ProviderReadWriteNotifyAttribute<T> extends ProviderReadWriteNotifyAttributeImpl {
    get: () => Promise<T[]>;
    registerGetter: (getterFunc: () => T | Promise<T>) => void;
    registerSetter: (setterFunc: (value: T) => void) => ProviderReadWriteAttribute<T>;
    set: (value: T) => Promise<[]>;
    registerObserver: (observer: (value: T) => void) => void;
    unregisterObserver: (observer: (value: T) => void) => void;
    valueChanged: (value: T) => void;
}
declare const ProviderReadNotifyAttributeImpl: {
    new (...args: any[]): {
        callbacks: ((value: unknown) => void)[];
        /**
         * If this attribute is changed the application should call this function with the new value,
         * whereafter the new value gets published
         *
         * @param value - the new value of the attribute
         */
        valueChanged(value: unknown): void;
        /**
         * Registers an Observer for value changes
         *
         * @param observer - the callback function with the signature "function(value){..}"
         *
         * @see ProviderAttribute#valueChanged
         * @see ProviderAttribute#unregisterObserver
         */
        registerObserver(observer: (value: unknown) => void): void;
        /**
         * Unregisters an Observer for value changes
         *
         * @param observer - the callback function with the signature "function(value){..}"
         *
         * @see ProviderAttribute#valueChanged
         * @see ProviderAttribute#registerObserver
         */
        unregisterObserver(observer: (value: unknown) => void): void;
        hasNotify: boolean;
        attributeType: string;
        attributeName: string;
        /** protected property, needs to be public for tsc -d */
        implementation: {
            get?: ((key: string) => any) | undefined;
            set?: ((value: any) => void | Promise<void>) | undefined;
            valueChanged?: ((value: any) => void) | undefined;
        };
        parent: JoynrProvider;
        hasRead: boolean;
        hasWrite: boolean;
        /** protected property, needs to be public for tsc -d */
        privateGetterFunc?: Function | undefined;
        /** protected property, needs to be public for tsc -d */
        privateSetterFunc?: ((value: any) => void | Promise<void>) | undefined;
        isNotifiable(): boolean;
        /**
         * Check Getter and Setter functions.
         */
        check(): boolean;
    };
} & ProviderAttributeConstructor<ProviderAttribute<any>>;
export declare class ProviderReadNotifyAttribute<T> extends ProviderReadNotifyAttributeImpl {
    get: () => Promise<T[]>;
    registerGetter: (getterFunc: () => T | Promise<T>) => void;
    registerObserver: (observer: (value: T) => void) => void;
    unregisterObserver: (observer: (value: T) => void) => void;
    valueChanged: (value: T) => void;
}
export {};
