"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/*
 * #%L
 * %%
 * Copyright (C) 2019 BMW Car IT GmbH
 * %%
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * #L%
 */
const UtilInternal = require("../util/UtilInternal");
const Typing = require("../util/Typing");
const ProviderRuntimeException = require("../exceptions/ProviderRuntimeException");
// eslint-disable-next-line @typescript-eslint/explicit-function-return-type
function Notifiable(superclass) {
    class Notifiable extends superclass {
        constructor(...args) {
            super(...args);
            this.callbacks = [];
            this.hasNotify = true;
            if (this.implementation) {
                this.implementation.valueChanged = this.valueChanged.bind(this);
            }
        }
        /**
         * If this attribute is changed the application should call this function with the new value,
         * whereafter the new value gets published
         *
         * @param value - the new value of the attribute
         */
        valueChanged(value) {
            UtilInternal.fire(this.callbacks, [value]);
        }
        /**
         * Registers an Observer for value changes
         *
         * @param observer - the callback function with the signature "function(value){..}"
         *
         * @see ProviderAttribute#valueChanged
         * @see ProviderAttribute#unregisterObserver
         */
        registerObserver(observer) {
            this.callbacks.push(observer);
        }
        /**
         * Unregisters an Observer for value changes
         *
         * @param observer - the callback function with the signature "function(value){..}"
         *
         * @see ProviderAttribute#valueChanged
         * @see ProviderAttribute#registerObserver
         */
        unregisterObserver(observer) {
            UtilInternal.removeElementFromArray(this.callbacks, observer);
        }
    }
    return Notifiable;
}
// eslint-disable-next-line @typescript-eslint/explicit-function-return-type
function Writable(superclass) {
    return class Writable extends superclass {
        constructor(...args) {
            super(...args);
            this.hasWrite = true;
        }
        /**
         * Registers the setter function for this attribute
         *
         * @param setterFunc - the setter function with the signature
         *          'void setterFunc({?}value) {..}'
         * @returns fluent interface to call multiple methods
         */
        registerSetter(setterFunc) {
            this.privateSetterFunc = setterFunc;
            return this;
        }
        /**
         * Calls through the setter registered with registerSetter with the same arguments as this
         * function
         *
         * @param value - the new value of the attribute
         *
         * @throws {Error} if no setter function was registered before calling it
         *
         * @see ProviderAttribute#registerSetter
         */
        async set(value) {
            if (!this.privateSetterFunc) {
                throw new Error("no setter function registered for provider attribute");
            }
            if (!this.privateGetterFunc) {
                throw new Error("no getter function registered for provider attribute");
            }
            const setterParams = Typing.augmentTypes(value, this.attributeType);
            const originalValue = await this.privateGetterFunc();
            await this.privateSetterFunc(setterParams);
            if (originalValue !== value && this.hasNotify) {
                this.valueChanged(value);
            }
            return [];
        }
    };
}
function toArray(returnValue) {
    return [returnValue];
}
// eslint-disable-next-line @typescript-eslint/explicit-function-return-type
function Readable(superclass) {
    return class Readable extends superclass {
        constructor(...args) {
            super(...args);
            this.hasRead = true;
        }
        registerGetter(getterFunc) {
            this.privateGetterFunc = async () => getterFunc();
        }
        /**
         * Calls through the getter registered with registerGetter with the same arguments as this
         * function
         *
         * @returns a Promise which resolves the attribute value
         *
         * rejects {Error} if no getter function was registered before calling it
         * rejects {Error} if registered getter returns a compound type with incorrect values
         */
        get() {
            try {
                if (!this.privateGetterFunc) {
                    return Promise.reject(new Error(`no getter function registered for provider attribute: ${this.attributeName}`));
                }
                return Promise.resolve(this.privateGetterFunc())
                    .then(toArray)
                    .catch(error => {
                    if (error instanceof ProviderRuntimeException) {
                        throw error;
                    }
                    throw new ProviderRuntimeException({
                        detailMessage: `getter method for attribute ${this.attributeName} reported an error`
                    });
                });
            }
            catch (e) {
                return Promise.reject(e);
            }
        }
    };
}
class ProviderAttribute {
    /**
     * Constructor of ProviderAttribute object that is used in the generation of provider attributes
     *
     * @constructor
     *
     * @param parent - the provider object that contains this attribute
     * @param [implementation] - the definition of attribute implementation
     * @param {Function} [implementation.set] - the getter function with the signature "function(value){}" that
     *          stores the given attribute value
     * @param {Function} [implementation.get] the getter function with the signature "function(){}" that returns the
     *          current attribute value
     * @param attributeName - the name of the attribute
     * @param attributeType - the type of the attribute
     */
    constructor(parent, implementation, attributeName, attributeType) {
        this.hasNotify = false;
        this.hasRead = false;
        this.hasWrite = false;
        this.parent = parent;
        this.implementation = implementation;
        this.attributeName = attributeName;
        this.attributeType = attributeType;
        // place these functions after the forwarding we don't want them public
        if (this.implementation && typeof this.implementation.get === "function") {
            this.privateGetterFunc = this.implementation.get;
        }
        if (this.implementation && typeof this.implementation.set === "function") {
            this.privateSetterFunc = this.implementation.set;
        }
    }
    isNotifiable() {
        return this.hasNotify;
    }
    /**
     * Check Getter and Setter functions.
     */
    check() {
        return ((!this.hasRead || typeof this.privateGetterFunc === "function") &&
            (!this.hasWrite || typeof this.privateSetterFunc === "function"));
    }
}
exports.ProviderAttribute = ProviderAttribute;
/*
    The empty container classes below are declared for typing purposes.
    Unfortunately declarations need to be repeated since it's impossible to give template parameters
    to mixin classes. See https://github.com/Microsoft/TypeScript/issues/26154
 */
const ProviderReadAttributeImpl = Readable(ProviderAttribute);
class ProviderReadAttribute extends ProviderReadAttributeImpl {
}
exports.ProviderReadAttribute = ProviderReadAttribute;
const ProviderReadWriteAttributeImpl = Writable(ProviderReadAttribute);
class ProviderReadWriteAttribute extends ProviderReadWriteAttributeImpl {
}
exports.ProviderReadWriteAttribute = ProviderReadWriteAttribute;
const ProviderReadWriteNotifyAttributeImpl = Notifiable(ProviderReadWriteAttribute);
class ProviderReadWriteNotifyAttribute extends ProviderReadWriteNotifyAttributeImpl {
}
exports.ProviderReadWriteNotifyAttribute = ProviderReadWriteNotifyAttribute;
const ProviderReadNotifyAttributeImpl = Notifiable(ProviderReadAttribute);
class ProviderReadNotifyAttribute extends ProviderReadNotifyAttributeImpl {
}
exports.ProviderReadNotifyAttribute = ProviderReadNotifyAttribute;
