"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const UtilInternal = __importStar(require("../util/UtilInternal"));
const Request = __importStar(require("../dispatching/types/Request"));
const MessagingQos_1 = __importDefault(require("../messaging/MessagingQos"));
const Typing = __importStar(require("../util/Typing"));
// eslint-disable-next-line @typescript-eslint/explicit-function-return-type
function asRead(superclass) {
    return class Readable extends superclass {
        /**
         * Getter for attribute
         *
         * @param [settings] the settings object for this function call
         * @returns returns an A+ promise object
         */
        get(settings) {
            // ensure settings variable holds a valid object and initialize
            // deferred object
            settings = settings || {};
            const request = Request.create({
                methodName: `get${UtilInternal.firstUpper(this.attributeName)}`
            });
            return this.executeRequest(request, settings);
        }
    };
}
// eslint-disable-next-line @typescript-eslint/explicit-function-return-type
function asWrite(superclass) {
    return class WriteAble extends superclass {
        /**
         * Setter for attribute
         *
         * @param settings - the settings object for this function call
         * @param settings.value - the attribute value to set
         * @returns returns an A+ promise
         */
        set(settings) {
            // ensure settings variable holds a valid object and initialize deferred
            // object
            settings = settings || {};
            try {
                settings.value = Typing.augmentTypes(settings.value);
            }
            catch (e) {
                return Promise.reject(new Error(`error setting attribute: ${this.attributeName}: ${e.toString()}`));
            }
            const request = Request.create({
                methodName: `set${UtilInternal.firstUpper(this.attributeName)}`,
                paramDatatypes: [this.attributeType],
                params: [settings.value]
            });
            return this.executeRequest(request, settings);
        }
    };
}
// eslint-disable-next-line @typescript-eslint/explicit-function-return-type
function asNotify(superclass) {
    return class Notifiable extends superclass {
        /**
         * Subscription to isOn attribute
         *
         * @param settings the settings object for this function call
         * @param settings.subscriptionQos - the subscription quality of service object
         * @param settings.onReceive this function is called if the attribute has
         *            been published successfully, method signature:
         *            "void onReceive({?}value)"
         * @param settings.onError - this function is called if a publication of the attribute value was
         *          missed, method signature: "void onError({Error} error)"
         * @param settings.onSubscribed - the callback to inform once the subscription request has been
         *          delivered successfully
         * @param [settings.subscriptionId] - optional subscriptionId to be used for the new subscription
         * @returns returns an A+ promise object
         */
        subscribe(settings) {
            // return promise to caller
            return this.settings.dependencies.subscriptionManager.registerSubscription({
                proxyId: this.parent.proxyParticipantId,
                providerDiscoveryEntry: this.parent.providerDiscoveryEntry,
                attributeName: this.attributeName,
                attributeType: this.attributeType,
                qos: settings.subscriptionQos,
                subscriptionId: settings.subscriptionId,
                onReceive: settings.onReceive,
                onError: settings.onError,
                onSubscribed: settings.onSubscribed
            });
        }
        /**
         * Unsubscribe from the attribute
         *
         * @param settings - the settings object for this function call
         * @param settings.subscriptionId - the subscription id retrieved from the subscribe function
         * @returns returns an A+ promise object
         *
         * @see ProxyAttribute#subscribe
         */
        unsubscribe(settings) {
            // passed in (right-most) messagingQos have precedence; undefined values are
            // ignored
            const messagingQos = new MessagingQos_1.default(UtilInternal.extend({}, this.parent.messagingQos, this.settings.messagingQos, settings.messagingQos));
            // return promise to caller
            return this.settings.dependencies.subscriptionManager.unregisterSubscription({
                messagingQos,
                subscriptionId: settings.subscriptionId
            });
        }
    };
}
function sendRequestOnSuccess(settings) {
    const response = settings.response;
    const attributeType = settings.settings;
    return Typing.augmentTypes(response[0], attributeType);
}
class ProxyAttribute {
    /**
     * Constructor of ProxyAttribute object that is used in the generation of proxy objects
     *
     * @param parent - the proxy object that contains this attribute
     * @param parent.proxyParticipantId - participantId of the proxy itself
     * @param parent.providerDiscoveryEntry.participantId - participantId of the provider being addressed
     * @param settings - the settings object for this function call
     * @param settings.dependencies - the dependencies object for this function call
     * @param settings.dependencies.requestReplyManager
     * @param settings.dependencies.subscriptionManager
     * @param settings.discoveryQos - the Quality of Service parameters for arbitration
     * @param settings.messagingQos - the Quality of Service parameters for messaging
     * @param attributeName - the name of the attribute
     * @param attributeType - the type of the attribute
     */
    constructor(parent, attributeName, attributeType) {
        this.parent = parent;
        this.settings = parent.settings;
        this.attributeName = attributeName;
        this.attributeType = attributeType;
    }
    /**
     * protected property, but public due to tsc -d
     *
     * @param request
     * @param requestSettings
     * @param requestSettings.messagingQos
     * @returns an A+ promise
     */
    executeRequest(request, requestSettings) {
        // passed in (right-most) messagingQos have precedence; undefined values are ignored
        const messagingQos = UtilInternal.extend(new MessagingQos_1.default(), this.parent.messagingQos, this.settings.messagingQos, requestSettings.messagingQos);
        // return promise to caller
        return this.settings.dependencies.requestReplyManager.sendRequest({
            toDiscoveryEntry: this.parent.providerDiscoveryEntry,
            from: this.parent.proxyParticipantId,
            messagingQos,
            request
        }, this.attributeType).then(sendRequestOnSuccess);
    }
}
/*
    The empty container classes below are declared for typing purposes.
    Unfortunately declarations need to be repeated since it's impossible to give template parameters
    to mixin classes. See https://github.com/Microsoft/TypeScript/issues/26154
 */
const ProxyReadAttributeImpl = asRead(ProxyAttribute);
class ProxyReadAttribute extends ProxyReadAttributeImpl {
}
exports.ProxyReadAttribute = ProxyReadAttribute;
const ProxyReadWriteAttributeImpl = asWrite(ProxyReadAttribute);
class ProxyReadWriteAttribute extends ProxyReadWriteAttributeImpl {
}
exports.ProxyReadWriteAttribute = ProxyReadWriteAttribute;
const ProxyReadWriteNotifyAttributeImpl = asNotify(ProxyReadWriteAttribute);
class ProxyReadWriteNotifyAttribute extends ProxyReadWriteNotifyAttributeImpl {
}
exports.ProxyReadWriteNotifyAttribute = ProxyReadWriteNotifyAttribute;
const ProxyReadNotifyAttributeImpl = asNotify(ProxyReadAttribute);
class ProxyReadNotifyAttribute extends ProxyReadNotifyAttributeImpl {
}
exports.ProxyReadNotifyAttribute = ProxyReadNotifyAttribute;
