import Address from "../../generated/joynr/system/RoutingTypes/Address";
import DiscoveryQos from "./DiscoveryQos";
import MessagingQos from "../messaging/MessagingQos";
import Arbitrator from "../capabilities/arbitration/Arbitrator";
import TypeRegistry from "../start/TypeRegistry";
import RequestReplyManager from "../dispatching/RequestReplyManager";
import SubscriptionManager from "../dispatching/subscription/SubscriptionManager";
import PublicationManager from "../dispatching/subscription/PublicationManager";
import MessageRouter from "../messaging/routing/MessageRouter";
interface JoynrProxy {
    proxyParticipantId: string;
    domain: string;
    interfaceName: string;
    providerDiscoveryEntry: any;
}
interface JoynrProxyConstructor<T extends JoynrProxy> {
    new (...args: any[]): T;
    getUsedJoynrtypes(): any[];
    MAJOR_VERSION: number;
    MINOR_VERSION: number;
}
declare class ProxyBuilder {
    private messageRouter;
    private libjoynrMessagingAddress;
    private proxyDependencies;
    private arbitrator;
    /**
     * @constructor
     *
     * @param proxyDependencies injected for the children of the proxyBuilder and its children
     * @param proxyDependencies.arbitrator used by the proxyBuilder to find the correct provider
     * @param proxyDependencies.requestReplyManager passed on to proxyAttribute and
     *            proxyOperation
     * @param proxyDependencies.subscriptionManager passed on to proxyAttribute and
     *            proxyOperation
     * @param proxyDependencies.publicationManager passed on to proxyAttribute and
     *            proxyOperation
     * @param dependencies injected for the proxyBuilder
     * @param dependencies.messageRouter the message router
     * @param dependencies.libjoynrMessagingAddress address to this libjoynr's message receiver
     * @param dependencies.typeRegistry the typeRegistry being able to augment raw objects with
     *            type information
     */
    constructor(proxyDependencies: {
        arbitrator: Arbitrator;
        requestReplyManager: RequestReplyManager;
        subscriptionManager: SubscriptionManager;
        publicationManager: PublicationManager;
    }, dependencies: {
        messageRouter: MessageRouter;
        libjoynrMessagingAddress: Address;
        typeRegistry: TypeRegistry;
    });
    /**
     * A function that constructs, arbitrates a object and provides the result and error using
     * an A+ promise
     *
     * @param ProxyConstructor - the constructor function of the generated Proxy that
     *            creates a new proxy instance
     * @param settings - the settings object that is passed to the constructor when building
     *            the object
     * @param settings.domain - the domain on which the provider should be looked for
     * @param settings.discoveryQos - the settings object determining arbitration
     *            parameters
     * @param settings.messagingQos - the settings object determining messaging quality of
     *            service parameters
     * @param settings.loggingContext - optional logging context will be appended to logging
     *            messages created in the name of this proxy
     * @param settings.staticArbitration - true if staticArbitration shall be used
     * @param settings.gbids - optional global backend identifiers of backends to lookup capabilities
     *
     * @returns Promise
     *  - resolved with the created proxy
     *  - rejected with either DiscoveryException or NoCompatibleProviderFoundException
     */
    build<T extends JoynrProxy>(ProxyConstructor: JoynrProxyConstructor<T>, settings: {
        domain: string;
        discoveryQos?: DiscoveryQos | Partial<DiscoveryQos.Settings>;
        messagingQos?: MessagingQos | Partial<MessagingQos.Settings>;
        loggingContext?: Record<string, any>;
        staticArbitration?: boolean;
        gbids?: string[];
    }): Promise<T>;
}
export = ProxyBuilder;
