"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
const nanoid_1 = require("nanoid");
const DiscoveryQos_1 = __importDefault(require("./DiscoveryQos"));
const JoynrRuntimeException_1 = __importDefault(require("../exceptions/JoynrRuntimeException"));
const MessagingQos_1 = __importDefault(require("../messaging/MessagingQos"));
const TypeRegistrySingleton_1 = __importDefault(require("../../joynr/types/TypeRegistrySingleton"));
const Version_1 = __importDefault(require("../../generated/joynr/types/Version"));
const Typing = __importStar(require("../util/Typing"));
const LoggingManager_1 = __importDefault(require("../system/LoggingManager"));
const log = LoggingManager_1.default.getLogger("joynr.proxy.ProxyBuilder");
const typeRegistry = TypeRegistrySingleton_1.default.getInstance();
class ProxyBuilder {
    /**
     * @constructor
     *
     * @param proxyDependencies injected for the children of the proxyBuilder and its children
     * @param proxyDependencies.arbitrator used by the proxyBuilder to find the correct provider
     * @param proxyDependencies.requestReplyManager passed on to proxyAttribute and
     *            proxyOperation
     * @param proxyDependencies.subscriptionManager passed on to proxyAttribute and
     *            proxyOperation
     * @param proxyDependencies.publicationManager passed on to proxyAttribute and
     *            proxyOperation
     * @param dependencies injected for the proxyBuilder
     * @param dependencies.messageRouter the message router
     * @param dependencies.libjoynrMessagingAddress address to this libjoynr's message receiver
     * @param dependencies.typeRegistry the typeRegistry being able to augment raw objects with
     *            type information
     */
    constructor(proxyDependencies, dependencies) {
        this.arbitrator = proxyDependencies.arbitrator;
        this.proxyDependencies = proxyDependencies;
        this.messageRouter = dependencies.messageRouter;
        this.libjoynrMessagingAddress = dependencies.libjoynrMessagingAddress;
    }
    /**
     * A function that constructs, arbitrates a object and provides the result and error using
     * an A+ promise
     *
     * @param ProxyConstructor - the constructor function of the generated Proxy that
     *            creates a new proxy instance
     * @param settings - the settings object that is passed to the constructor when building
     *            the object
     * @param settings.domain - the domain on which the provider should be looked for
     * @param settings.discoveryQos - the settings object determining arbitration
     *            parameters
     * @param settings.messagingQos - the settings object determining messaging quality of
     *            service parameters
     * @param settings.loggingContext - optional logging context will be appended to logging
     *            messages created in the name of this proxy
     * @param settings.staticArbitration - true if staticArbitration shall be used
     * @param settings.gbids - optional global backend identifiers of backends to lookup capabilities
     *
     * @returns Promise
     *  - resolved with the created proxy
     *  - rejected with either DiscoveryException or NoCompatibleProviderFoundException
     */
    async build(ProxyConstructor, settings) {
        // eslint-disable-next-line prefer-const
        let { domain, discoveryQos, messagingQos, loggingContext, staticArbitration, gbids } = settings;
        // augment Qos objects if they're missing
        discoveryQos = new DiscoveryQos_1.default(discoveryQos);
        messagingQos = new MessagingQos_1.default(messagingQos);
        // check if objects are there and of correct type
        Typing.checkProperty(domain, "String", "settings.domain");
        Typing.checkProperty(discoveryQos, DiscoveryQos_1.default, "settings.discoveryQos");
        Typing.checkProperty(messagingQos, MessagingQos_1.default, "settings.messagingQos");
        const proxyParticipantId = nanoid_1.nanoid();
        const proxy = new ProxyConstructor({
            domain,
            discoveryQos,
            messagingQos,
            loggingContext,
            proxyParticipantId,
            dependencies: this.proxyDependencies
        });
        ProxyConstructor.getUsedJoynrtypes().forEach(joynrType => {
            typeRegistry.addType(joynrType);
        });
        const proxyVersion = new Version_1.default({
            majorVersion: ProxyConstructor.MAJOR_VERSION,
            minorVersion: ProxyConstructor.MINOR_VERSION
        });
        const arbitratedCaps = await this.arbitrator.startArbitration({
            domains: [proxy.domain],
            interfaceName: proxy.interfaceName,
            // typecast may be removed after Arbitrator typings are done
            discoveryQos: discoveryQos,
            staticArbitration: staticArbitration,
            proxyVersion,
            gbids
        });
        if (settings.loggingContext !== undefined) {
            log.warn("loggingContext is currently not supported");
        }
        let isGloballyVisible = false;
        if (arbitratedCaps && arbitratedCaps.length > 0) {
            proxy.providerDiscoveryEntry = arbitratedCaps[0];
            if (!arbitratedCaps[0].isLocal) {
                isGloballyVisible = true;
            }
        }
        return this.messageRouter
            .addNextHop(proxy.proxyParticipantId, this.libjoynrMessagingAddress, isGloballyVisible)
            .then(() => {
            this.messageRouter.setToKnown(proxy.providerDiscoveryEntry.participantId);
            log.info(`Proxy created, proxy participantId: ${proxy.proxyParticipantId}, provider discoveryEntry: ${JSON.stringify(proxy.providerDiscoveryEntry)}`);
            return proxy;
        })
            .catch((error) => {
            const errorMsg = `Proxy creation for domain ${proxy.domain}, interface ${proxy.interfaceName}, major version ${ProxyConstructor.MAJOR_VERSION}, proxyParticipantId ${proxy.proxyParticipantId} failed. Exception occurred while registering the address to MessageRouter. Error: ${error.stack}`;
            log.debug(errorMsg);
            throw new JoynrRuntimeException_1.default({ detailMessage: errorMsg });
        });
    }
}
module.exports = ProxyBuilder;
