import BroadcastFilterParameters from "./BroadcastFilterParameters";
import MessagingQos from "../messaging/MessagingQos";
import SubscriptionManager from "../dispatching/subscription/SubscriptionManager";
import SubscriptionQos from "./SubscriptionQos";
import DiscoveryQos = require("./DiscoveryQos");
declare class ProxyEvent {
    private parent;
    private settings;
    /**
     * Constructor of ProxyEvent object that is used in the generation of proxy objects
     *
     * @constructor
     * @param parent is the proxy object that contains this attribute
     * @param settings the settings for this broadcast proxy
     * @param settings.discoveryQos the Quality of Service parameters for arbitration
     * @param settings.messagingQos the Quality of Service parameters for messaging
     * @param settings.dependencies the dependencies object for this function call
     * @param settings.dependencies.subscriptionManager
     * @param settings.selective true if the broadcast is selective
     * @param settings.broadcastName the name of the broadcast as modelled in Franca
     * @param settings.broadcastParameter the parameter meta information of the broadcast being subscribed to
     * @param settings.filterParameters the filter parameters of the broadcast
     * @returns {ProxyEvent} */
    constructor(parent: any, settings: {
        discoveryQos: DiscoveryQos;
        messagingQos: MessagingQos;
        dependencies: {
            subscriptionManager: SubscriptionManager;
        };
        selective?: boolean;
        broadcastName: string;
        broadcastParameter: {
            name: string;
            type: string;
        }[];
        filterParameters?: Record<string, any>;
    });
    /**
     * @param subscribeParameters the settings object for this function call
     * @param subscribeParameters.subscriptionQos the subscription quality of service object
     * @param [subscribeParameters.subscriptionId] optional subscriptionId. Used to refresh or
     *            reinstate an existing subscription.
     * @param [subscribeParameters.partitions] optional parameter for multicast subscriptions.
     *            This parameter becomes relevant for non selective broadcasts and specifies the interested partitions
     *            of publications. It is interpreted hierarchically.
     * @param subscribeParameters.onReceive this function is called when an event as been
     *            received. method signature: "void onReceive({?}value)"
     * @param subscribeParameters.onError this function is called when an error occurs with
     *            a subscribed event. method signature: "void onError({Error} error)"
     * @param subscribeParameters.onSubscribed the callback to inform once the subscription request has
     *            been delivered successfully
     * @returns returns a promise that is resolved with the subscriptionId, which is to
     *          be used to unsubscribe from this subscription later.
     * @throws {Error} if subscribeParameters.partitions contains invalid characters
     */
    subscribe(subscribeParameters: {
        subscriptionQos: SubscriptionQos;
        subscriptionId?: string;
        partitions?: string[];
        onReceive: Function;
        onError?: (error: any) => void;
        onSubscribed?: Function;
        filterParameters?: any;
    }): Promise<string>;
    createFilterParameters(): BroadcastFilterParameters;
    /**
     * @param unsubscribeParameters the settings object for this function call
     * @param unsubscribeParameters.subscriptionId the subscription token retrieved from the
     *            subscribe function
     * @returns returns a promise that is resolved when unsubscribe has been executed.
     * @see ProxyEvent#subscribe
     */
    unsubscribe(unsubscribeParameters: {
        subscriptionId: string;
    }): Promise<void>;
}
export = ProxyEvent;
