import * as DiscoveryEntryWithMetaInfo from "../../generated/joynr/types/DiscoveryEntryWithMetaInfo";
import Arbitrator from "../capabilities/arbitration/Arbitrator";
import { DiscoveryStub } from "../capabilities/interface/DiscoveryStub";
import ProviderBuilder from "../provider/ProviderBuilder";
import ProxyBuilder from "../proxy/ProxyBuilder";
import CapabilitiesRegistrar from "../capabilities/CapabilitiesRegistrar";
import ParticipantIdStorage from "../capabilities/ParticipantIdStorage";
import RequestReplyManager from "../dispatching/RequestReplyManager";
import PublicationManager from "../dispatching/subscription/PublicationManager";
import SubscriptionManager from "../dispatching/subscription/SubscriptionManager";
import Dispatcher from "../dispatching/Dispatcher";
import MessageRouter, { MessageRouterSettings } from "../messaging/routing/MessageRouter";
import InProcessMessagingStub from "../messaging/inprocess/InProcessMessagingStub";
import { Provisioning, ShutdownSettings } from "./interface/Provisioning";
import LoggingManager = require("../system/LoggingManager");
import TypeRegistry = require("./TypeRegistry");
import { Persistency } from "../../global/interface/Persistency";
import JoynrStates = require("./JoynrStates");
import JoynrRuntimeException from "../exceptions/JoynrRuntimeException";
declare type Omit<T, K> = Pick<T, Exclude<keyof T, K>>;
declare class JoynrRuntime<T extends Provisioning> {
    protected provisioning: T;
    protected shutdownSettings?: ShutdownSettings;
    protected discovery: DiscoveryStub;
    protected persistencyConfig: any;
    protected persistency: Persistency;
    protected joynrState: typeof JoynrStates[keyof typeof JoynrStates];
    protected multicastSkeletons: Record<string, any>;
    protected dispatcher: Dispatcher;
    protected onFatalRuntimeError: (error: JoynrRuntimeException) => void;
    protected subscriptionManager: SubscriptionManager;
    protected publicationManager: PublicationManager;
    protected requestReplyManager: RequestReplyManager;
    protected messageRouter: MessageRouter;
    protected arbitrator: Arbitrator;
    /**
     * @name JoynrRuntime#logging
     * @type LoggingManager
     */
    logging: LoggingManager;
    /**
     * @name JoynrRuntime#participantIdStorage
     * @type ParticipantIdStorage
     */
    participantIdStorage?: ParticipantIdStorage;
    /**
     * @name JoynrRuntime#proxyBuilder
     * @type ProxyBuilder
     */
    proxyBuilder?: ProxyBuilder;
    /**
     * @name JoynrRuntime#providerBuilder
     * @type ProviderBuilder
     */
    providerBuilder?: ProviderBuilder;
    /**
     * @name JoynrRuntime#registration
     * @type CapabilitiesRegistrar
     */
    registration?: CapabilitiesRegistrar;
    /**
     * @name JoynrRuntime#typeRegistry
     * @type TypeRegistry
     */
    typeRegistry: TypeRegistry;
    constructor(onFatalRuntimeError: (error: JoynrRuntimeException) => void);
    protected initializePersistency(provisioning: T): Promise<void>;
    protected createMessageRouter(provisioning: T, messageRouterSettings: Omit<MessageRouterSettings, "multicastSkeletons" | "messageQueue">): void;
    protected initializeComponents(provisioning: T, discovery: DiscoveryStub, externalMessagingStub: InProcessMessagingStub, typedCapabilities?: DiscoveryEntryWithMetaInfo[]): void;
    /**
     * Starts up the libjoynr instance
     *
     * @returns an A+ promise object, reporting when libjoynr startup is
     *          completed or has failed
     * @throws {Error} if libjoynr is not in SHUTDOWN state
     */
    start(provisioning: T): void;
    /**
     *  Sends subscriptionStop messages for all active subscriptions.
     *
     *  @param timeout {number} optional timeout defaulting to 0 = no timeout
     *  @returns - resolved after all SubscriptionStop messages are sent.
     *  - rejected in case of any issues or timeout occurs.
     */
    terminateAllSubscriptions(timeout?: number): Promise<any>;
    /**
     * Shuts down libjoynr
     * @param settings.clearSubscriptionsTimeoutMs {number} time in ms till clearSubscriptionsPromise will be rejected
     *  if it's not resolved yet
     * @param settings.clearSubscriptionsEnabled {boolean} clear all subscriptions before shutting down.
     *  Set this to false in process.exit handler as this is not synchronous.
     *
     * @returns - resolved after successful shutdown
     * - rejected in case of any issues
     */
    shutdown(settings?: ShutdownSettings): Promise<void>;
}
export = JoynrRuntime;
