"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
const Arbitrator_1 = __importDefault(require("../capabilities/arbitration/Arbitrator"));
const ProviderBuilder_1 = __importDefault(require("../provider/ProviderBuilder"));
const ProxyBuilder_1 = __importDefault(require("../proxy/ProxyBuilder"));
const CapabilitiesRegistrar_1 = __importDefault(require("../capabilities/CapabilitiesRegistrar"));
const ParticipantIdStorage_1 = __importDefault(require("../capabilities/ParticipantIdStorage"));
const RequestReplyManager_1 = __importDefault(require("../dispatching/RequestReplyManager"));
const PublicationManager_1 = __importDefault(require("../dispatching/subscription/PublicationManager"));
const SubscriptionManager_1 = __importDefault(require("../dispatching/subscription/SubscriptionManager"));
const Dispatcher_1 = __importDefault(require("../dispatching/Dispatcher"));
const PlatformSecurityManagerNode_1 = __importDefault(require("../security/PlatformSecurityManagerNode"));
const MessageRouter_1 = __importDefault(require("../messaging/routing/MessageRouter"));
const MessageQueue_1 = __importDefault(require("../messaging/routing/MessageQueue"));
const InProcessMessagingSkeleton_1 = __importDefault(require("../messaging/inprocess/InProcessMessagingSkeleton"));
const InProcessAddress_1 = __importDefault(require("../messaging/inprocess/InProcessAddress"));
const DiscoveryQos_1 = __importDefault(require("../proxy/DiscoveryQos"));
const TypeRegistrySingleton_1 = __importDefault(require("../types/TypeRegistrySingleton"));
const nanoid_1 = require("nanoid");
const LoggingManager_1 = __importDefault(require("../system/LoggingManager"));
const defaultLibjoynrSettings_1 = __importDefault(require("./settings/defaultLibjoynrSettings"));
const LocalStorageNode_1 = __importDefault(require("../../global/LocalStorageNode"));
const MemoryStorage_1 = __importDefault(require("../../global/MemoryStorage"));
const JoynrStates = require("./JoynrStates");
const UtilInternal = __importStar(require("../util/UtilInternal"));
const log = LoggingManager_1.default.getLogger("joynr.start.JoynrRuntime");
class JoynrRuntime {
    constructor(onFatalRuntimeError) {
        this.multicastSkeletons = {};
        // all methods apart from start need to be bound here.
        this.shutdown = this.shutdown.bind(this);
        this.terminateAllSubscriptions = this.terminateAllSubscriptions.bind(this);
        this.onFatalRuntimeError = onFatalRuntimeError;
        this.typeRegistry = TypeRegistrySingleton_1.default.getInstance();
        this.logging = LoggingManager_1.default;
        this.joynrState = JoynrStates.SHUTDOWN;
    }
    initializePersistency(provisioning) {
        const persistencyProvisioning = Object.assign({}, defaultLibjoynrSettings_1.default.persistencySettings, provisioning.persistency);
        let persistencyPromise;
        if (persistencyProvisioning.capabilities) {
            this.persistency = new LocalStorageNode_1.default({
                clearPersistency: persistencyProvisioning.clearPersistency,
                location: persistencyProvisioning.location
            });
            persistencyPromise = this.persistency.init();
        }
        else {
            persistencyPromise = Promise.resolve();
        }
        this.persistencyConfig = {
            capabilities: persistencyProvisioning.capabilities ? this.persistency : new MemoryStorage_1.default()
        };
        return persistencyPromise;
    }
    createMessageRouter(provisioning, messageRouterSettings) {
        if (UtilInternal.checkNullUndefined(this.persistencyConfig)) {
            throw new Error("Call initializePersistency before createMessageRouter.");
        }
        const messageQueueSettings = {};
        if (provisioning.messaging !== undefined && provisioning.messaging.maxQueueSizeInKBytes !== undefined) {
            messageQueueSettings.maxQueueSizeInKBytes = provisioning.messaging.maxQueueSizeInKBytes;
        }
        messageRouterSettings.multicastSkeletons = this.multicastSkeletons;
        messageRouterSettings.messageQueue = new MessageQueue_1.default(messageQueueSettings);
        this.messageRouter = new MessageRouter_1.default(messageRouterSettings);
    }
    initializeComponents(provisioning, discovery, externalMessagingStub, typedCapabilities) {
        if (UtilInternal.checkNullUndefined(this.messageRouter)) {
            throw new Error("Call createMessageRouter before initializePersistency.");
        }
        this.discovery = discovery;
        const ttlUpLiftMs = provisioning.messaging && provisioning.messaging.TTL_UPLIFT ? provisioning.messaging.TTL_UPLIFT : undefined;
        this.dispatcher = new Dispatcher_1.default(externalMessagingStub, new PlatformSecurityManagerNode_1.default(), ttlUpLiftMs);
        const libjoynrMessagingSkeleton = new InProcessMessagingSkeleton_1.default();
        libjoynrMessagingSkeleton.registerListener(this.dispatcher.receive);
        this.requestReplyManager = new RequestReplyManager_1.default(this.dispatcher);
        this.subscriptionManager = new SubscriptionManager_1.default(this.dispatcher);
        this.publicationManager = new PublicationManager_1.default(this.dispatcher);
        this.dispatcher.registerRequestReplyManager(this.requestReplyManager);
        this.dispatcher.registerSubscriptionManager(this.subscriptionManager);
        this.dispatcher.registerPublicationManager(this.publicationManager);
        this.dispatcher.registerMessageRouter(this.messageRouter);
        this.participantIdStorage = new ParticipantIdStorage_1.default(this.persistencyConfig.capabilities, nanoid_1.nanoid);
        this.registration = new CapabilitiesRegistrar_1.default({
            discoveryStub: this.discovery,
            messageRouter: this.messageRouter,
            requestReplyManager: this.requestReplyManager,
            publicationManager: this.publicationManager,
            libjoynrMessagingAddress: new InProcessAddress_1.default(libjoynrMessagingSkeleton),
            participantIdStorage: this.participantIdStorage
        });
        // typedCapabilites can be undefined in case of InProcessRuntime
        this.arbitrator = new Arbitrator_1.default(this.discovery, typedCapabilities);
        this.providerBuilder = new ProviderBuilder_1.default({
            typeRegistry: this.typeRegistry
        });
        this.proxyBuilder = new ProxyBuilder_1.default({
            arbitrator: this.arbitrator,
            requestReplyManager: this.requestReplyManager,
            subscriptionManager: this.subscriptionManager,
            publicationManager: this.publicationManager
        }, {
            messageRouter: this.messageRouter,
            libjoynrMessagingAddress: new InProcessAddress_1.default(libjoynrMessagingSkeleton),
            typeRegistry: this.typeRegistry
        });
    }
    /**
     * Starts up the libjoynr instance
     *
     * @returns an A+ promise object, reporting when libjoynr startup is
     *          completed or has failed
     * @throws {Error} if libjoynr is not in SHUTDOWN state
     */
    start(provisioning) {
        if (this.joynrState !== JoynrStates.SHUTDOWN) {
            throw new Error(`Cannot start libjoynr because it's currently "${this.joynrState}"`);
        }
        this.joynrState = JoynrStates.STARTING;
        if (!provisioning) {
            throw new Error("Constructor has been invoked without provisioning");
        }
        if (provisioning.logging) {
            this.logging.configure(provisioning.logging);
        }
        this.shutdownSettings = provisioning.shutdownSettings;
        this.provisioning = provisioning;
        if (provisioning.discoveryQos) {
            const discoveryQos = provisioning.discoveryQos;
            if (discoveryQos.discoveryExpiryIntervalMs) {
                CapabilitiesRegistrar_1.default.setDefaultExpiryIntervalMs(discoveryQos.discoveryExpiryIntervalMs);
            }
            const discoveryQosSettings = {};
            if (discoveryQos.discoveryRetryDelayMs) {
                discoveryQosSettings.discoveryRetryDelayMs = discoveryQos.discoveryRetryDelayMs;
            }
            if (discoveryQos.discoveryTimeoutMs) {
                discoveryQosSettings.discoveryTimeoutMs = discoveryQos.discoveryTimeoutMs;
            }
            DiscoveryQos_1.default.setDefaultSettings(discoveryQosSettings);
        }
    }
    /**
     *  Sends subscriptionStop messages for all active subscriptions.
     *
     *  @param timeout {number} optional timeout defaulting to 0 = no timeout
     *  @returns - resolved after all SubscriptionStop messages are sent.
     *  - rejected in case of any issues or timeout occurs.
     */
    terminateAllSubscriptions(timeout = 0) {
        return this.subscriptionManager.terminateSubscriptions(timeout);
    }
    /**
     * Shuts down libjoynr
     * @param settings.clearSubscriptionsTimeoutMs {number} time in ms till clearSubscriptionsPromise will be rejected
     *  if it's not resolved yet
     * @param settings.clearSubscriptionsEnabled {boolean} clear all subscriptions before shutting down.
     *  Set this to false in process.exit handler as this is not synchronous.
     *
     * @returns - resolved after successful shutdown
     * - rejected in case of any issues
     */
    async shutdown(settings) {
        if (this.joynrState !== JoynrStates.STARTED && this.joynrState !== JoynrStates.STARTING) {
            throw new Error(`Cannot shutdown libjoynr because it's currently "${this.joynrState}"`);
        }
        this.joynrState = JoynrStates.SHUTTINGDOWN;
        const shutdownSettings = Object.assign({}, defaultLibjoynrSettings_1.default.shutdownSettings, this.shutdownSettings, settings || {});
        if (shutdownSettings.clearSubscriptionsEnabled) {
            await this.subscriptionManager
                .terminateSubscriptions(shutdownSettings.clearSubscriptionsTimeoutMs)
                .catch((e) => {
                log.error(`could not shutdown joynr in time due to ${e}`);
            });
        }
        [
            "registration",
            "arbitrator",
            "messageRouter",
            "requestReplyManager",
            "publicationManager",
            "subscriptionManager",
            "dispatcher",
            "typeRegistry"
        ].forEach(component => {
            if (this[component]) {
                // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                this[component].shutdown();
            }
        });
        if (this.persistency !== undefined)
            await this.persistency.shutdown();
        this.joynrState = JoynrStates.SHUTDOWN;
        log.debug("joynr shut down");
    }
}
module.exports = JoynrRuntime;
