"use strict";
/*
 * #%L
 * %%
 * Copyright (C) 2019 BMW Car IT GmbH
 * %%
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * #L%
 */
/* istanbul ignore file */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
const ApplicationException_1 = __importDefault(require("../exceptions/ApplicationException"));
const DiscoveryException_1 = __importDefault(require("../exceptions/DiscoveryException"));
const IllegalAccessException_1 = __importDefault(require("../exceptions/IllegalAccessException"));
const JoynrException_1 = __importDefault(require("../exceptions/JoynrException"));
const JoynrRuntimeException_1 = __importDefault(require("../exceptions/JoynrRuntimeException"));
const MethodInvocationException_1 = __importDefault(require("../exceptions/MethodInvocationException"));
const NoCompatibleProviderFoundException_1 = __importDefault(require("../exceptions/NoCompatibleProviderFoundException"));
const ProviderRuntimeException_1 = __importDefault(require("../exceptions/ProviderRuntimeException"));
const PublicationMissedException_1 = __importDefault(require("../exceptions/PublicationMissedException"));
const SubscriptionException_1 = __importDefault(require("../exceptions/SubscriptionException"));
const JoynrEnum = require("../types/JoynrEnum");
class TypeRegistry {
    /**
     * The <code>TypeRegistry</code> contains a mapping of type names (which are sent on the wire
     * through joynr) to the corresponding
     * JavaScript type constructor.
     *
     * This class is used during deserialization, ie mapping a received request/response to the
     * corresponding JavaScript call with the
     * correct parameters.
     *
     * @constructor
     */
    constructor() {
        this.registryPromise = {};
        this.enumRegistry = {};
        this.registry = {};
        this.addType(ApplicationException_1.default)
            .addType(DiscoveryException_1.default)
            .addType(IllegalAccessException_1.default)
            .addType(JoynrException_1.default)
            .addType(JoynrRuntimeException_1.default)
            .addType(MethodInvocationException_1.default)
            .addType(NoCompatibleProviderFoundException_1.default)
            .addType(ProviderRuntimeException_1.default)
            .addType(PublicationMissedException_1.default)
            .addType(SubscriptionException_1.default);
    }
    /**
     * Adds a typeName to constructor entry in the type registry.
     *
     * @param typeConstructor - the corresponding JavaScript constructor for this type.
     * @returns typeRegistry for chained calls.
     */
    addType(typeConstructor) {
        const typeName = typeConstructor._typeName;
        if (typeName === undefined) {
            throw new Error(`Trying to add invalid type: ${typeConstructor}`);
        }
        if (typeConstructor.prototype instanceof JoynrEnum) {
            this.enumRegistry[typeName] = typeConstructor;
        }
        this.registry[typeName] = typeConstructor;
        if (this.registryPromise[typeName]) {
            this.registryPromise[typeName].resolve(typeConstructor);
            clearTimeout(this.registryPromise[typeName].timeoutTimer);
        }
        return this;
    }
    /**
     * Detects if asked joynr type is an enumeration type
     *
     * @param joynrTypeName - the joynr type name that is sent/received on the wire.
     * @returns true if the asked joynr type is an enumeration type
     */
    isEnumType(joynrTypeName) {
        return this.enumRegistry[joynrTypeName] !== undefined;
    }
    /**
     * Retrieves the constructor for a given type.
     *
     * @param joynrTypeName - the joynr type name that is sent/received on the wire.
     * @returns the constructor function for the specified type name
     */
    getConstructor(joynrTypeName) {
        return this.registry[joynrTypeName];
    }
    /**
     * Shutdown the type registry
     */
    shutdown() {
        let typeName;
        for (typeName in this.registryPromise) {
            if (Object.prototype.hasOwnProperty.call(this.registryPromise, typeName)) {
                if (this.registryPromise[typeName].timeoutTimer !== undefined) {
                    clearTimeout(this.registryPromise[typeName].timeoutTimer);
                }
            }
        }
    }
}
module.exports = TypeRegistry;
