"use strict";
/*
 * #%L
 * %%
 * Copyright (C) 2020 BMW Car IT GmbH
 * %%
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * #L%
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
const JoynrRuntime_1 = __importDefault(require("./JoynrRuntime"));
const UdsClient_1 = __importDefault(require("../messaging/uds/UdsClient"));
const InProcessMessagingSkeleton_1 = __importDefault(require("../messaging/inprocess/InProcessMessagingSkeleton"));
const InProcessMessagingStub_1 = __importDefault(require("../messaging/inprocess/InProcessMessagingStub"));
const DiagnosticTags = __importStar(require("../system/DiagnosticTags"));
const RoutingProxy_1 = __importDefault(require("../../generated/joynr/system/RoutingProxy"));
const DiscoveryQos_1 = __importDefault(require("../proxy/DiscoveryQos"));
const DiscoveryScope_1 = __importDefault(require("../../generated/joynr/types/DiscoveryScope"));
const DiscoveryProxy_1 = __importDefault(require("../../generated/joynr/system/DiscoveryProxy"));
const MessageReplyToAddressCalculator_1 = __importDefault(require("../messaging/MessageReplyToAddressCalculator"));
const DiscoveryEntryWithMetaInfo_1 = __importDefault(require("../../generated/joynr/types/DiscoveryEntryWithMetaInfo"));
const defaultLibjoynrSettings_1 = __importDefault(require("./settings/defaultLibjoynrSettings"));
const nanoid_1 = require("nanoid");
//import InProcessAddress from "../messaging/inprocess/InProcessAddress";
//import InProcessMessagingStubFactory from "../messaging/inprocess/InProcessMessagingStubFactory";
const MessagingStubFactory_1 = __importDefault(require("../messaging/MessagingStubFactory"));
const MessagingQos_1 = __importDefault(require("../messaging/MessagingQos"));
const UdsAddress_1 = __importDefault(require("../../generated/joynr/system/RoutingTypes/UdsAddress"));
const UdsClientAddress_1 = __importDefault(require("../../generated/joynr/system/RoutingTypes/UdsClientAddress"));
const LocalDiscoveryAggregator = require("../capabilities/discovery/LocalDiscoveryAggregator");
const UdsMulticastAddressCalculator_1 = __importDefault(require("../messaging/uds/UdsMulticastAddressCalculator"));
const JoynrStates = require("./JoynrStates");
const LoggingManager_1 = __importDefault(require("../system/LoggingManager"));
const log = LoggingManager_1.default.getLogger("joynr.start.UdsLibJoynrRuntime");
/**
 * The UdsLibJoynrRuntime is the version of the libjoynr-js runtime that communicates with
 * a cluster controller via a unix domain socket (UDS). The cluster controller is the UDS Server, and the
 * libjoynr connects to it with a single socket for all communication.
 *
 * @name UdsLibJoynrRuntime
 * @constructor
 */
class UdsLibJoynrRuntime extends JoynrRuntime_1.default {
    constructor(onFatalRuntimeError) {
        super(onFatalRuntimeError);
        this.messageReplyToAddressCalculator = new MessageReplyToAddressCalculator_1.default({});
    }
    /**
     * Starts up the libjoynr instance
     *
     * @param {UdsLibJoynrProvisioning} provisioning
     * @returns an A+ promise object, reporting when libjoynr startup is
     *          completed or has failed
     * @throws {Error} if libjoynr is not in SHUTDOWN state
     */
    async start(provisioning) {
        super.start(provisioning);
        const udsProvisioning = provisioning.uds || {};
        udsProvisioning.socketPath = udsProvisioning.socketPath || "/var/run/joynr/cluster-controller.sock";
        udsProvisioning.clientId = udsProvisioning.clientId || nanoid_1.nanoid();
        udsProvisioning.connectSleepTimeMs = udsProvisioning.connectSleepTimeMs || 500;
        // routing table is still required for addMulticastReceiver and removeMulticastReceiver
        const initialRoutingTable = {};
        let untypedCapabilities = this.provisioning.capabilities || [];
        const defaultCapabilities = defaultLibjoynrSettings_1.default.capabilities || [];
        untypedCapabilities = untypedCapabilities.concat(defaultCapabilities);
        const ccAddress = new UdsAddress_1.default({
            path: udsProvisioning.socketPath
        });
        // provisioned capabilities for Arbitrator
        const typedCapabilities = [];
        for (let i = 0; i < untypedCapabilities.length; i++) {
            const capability = new DiscoveryEntryWithMetaInfo_1.default(untypedCapabilities[i]);
            initialRoutingTable[capability.participantId] = ccAddress;
            typedCapabilities.push(capability);
        }
        const localAddress = new UdsClientAddress_1.default({
            id: udsProvisioning.clientId
        });
        // TODO
        // MessagingStubFactory is only required for the constructor of MessageRouter.
        // It is used there for routing which is not required in UdsLibJoynrRuntime.
        // This will be refactored and optimized later.
        const messagingStubFactories = {};
        //messagingStubFactories[InProcessAddress._typeName] = new InProcessMessagingStubFactory();
        const messagingStubFactory = new MessagingStubFactory_1.default({
            messagingStubFactories
        });
        // TODO
        // MulticastAddressCalculator is only required for the constructor of MessageRouter.
        // It is used there for routing which is not required in UdsLibJoynrRuntime.
        // This will be refactored and optimized later.
        const messageRouterSettings = {
            initialRoutingTable,
            messagingStubFactory,
            incomingAddress: localAddress,
            parentMessageRouterAddress: ccAddress,
            multicastAddressCalculator: new UdsMulticastAddressCalculator_1.default({
                globalAddress: ccAddress
            })
        };
        const localDiscoveryAggregator = new LocalDiscoveryAggregator();
        await super.initializePersistency(provisioning);
        super.createMessageRouter(provisioning, messageRouterSettings);
        // InProcessMessagingSkeleton and InProcessMessagingStub used for direct handling of outgoing messages
        // from the dispatcher to the UDS client
        const clusterControllerMessagingSkeleton = new InProcessMessagingSkeleton_1.default();
        const clusterControllerMessagingStub = new InProcessMessagingStub_1.default(clusterControllerMessagingSkeleton);
        super.initializeComponents(provisioning, localDiscoveryAggregator, clusterControllerMessagingStub, typedCapabilities);
        // Forward incoming messages directly to Dispatcher.receive() (no routing)
        const onUdsClientMessageCallback = (joynrMessage) => {
            log.debug(`<<< INCOMING <<< message with ID ${joynrMessage.msgId}`);
            this.dispatcher.receive(joynrMessage);
        };
        this.udsClient = new UdsClient_1.default({
            socketPath: udsProvisioning.socketPath,
            clientId: udsProvisioning.clientId,
            connectSleepTimeMs: udsProvisioning.connectSleepTimeMs,
            onMessageCallback: onUdsClientMessageCallback,
            onFatalRuntimeError: this.onFatalRuntimeError
        });
        // Forward all outgoing messages directly to udsClient.send() (no routing)
        const onMessageSend = (joynrMessage) => {
            if (!joynrMessage.isLocalMessage) {
                try {
                    this.messageReplyToAddressCalculator.setReplyTo(joynrMessage);
                }
                catch (error) {
                    log.warn(`replyTo address could not be set: ${error}. Dropping message.`, DiagnosticTags.forJoynrMessage(joynrMessage));
                    return Promise.resolve();
                }
            }
            return this.udsClient.send(joynrMessage);
        };
        clusterControllerMessagingSkeleton.registerListener(onMessageSend);
        const internalMessagingQos = new MessagingQos_1.default({ ttl: MessagingQos_1.default.DEFAULT_TTL + 10000 });
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        return this.proxyBuilder.build(RoutingProxy_1.default, {
            domain: "io.joynr",
            messagingQos: internalMessagingQos,
            discoveryQos: new DiscoveryQos_1.default({
                discoveryScope: DiscoveryScope_1.default.LOCAL_ONLY
            }),
            staticArbitration: true
        })
            .then(newRoutingProxy => {
            return this.messageRouter.setRoutingProxy(newRoutingProxy);
        })
            .catch((error) => {
            throw new Error(`Failed to create routing proxy: ${error}`);
        })
            .then(() => {
            return this.messageRouter.getReplyToAddressFromRoutingProxy();
        })
            .catch((error) => {
            throw new Error(`Failed to initialize replyToAddress: ${error}`);
        })
            .then(address => {
            this.messageReplyToAddressCalculator.setReplyToAddress(address);
            // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
            return this.proxyBuilder.build(DiscoveryProxy_1.default, {
                domain: "io.joynr",
                messagingQos: internalMessagingQos,
                discoveryQos: new DiscoveryQos_1.default({
                    discoveryScope: DiscoveryScope_1.default.LOCAL_ONLY
                }),
                staticArbitration: true
            });
        })
            .then((newDiscoveryProxy) => {
            localDiscoveryAggregator.setDiscoveryProxy(newDiscoveryProxy);
            this.joynrState = JoynrStates.STARTED;
            log.debug("UdsLibJoynrRuntime initialized");
        })
            .catch(async (error) => {
            log.error(`error starting up joynr: ${error}`);
            await this.shutdown();
            throw error;
        });
    }
    /**
     *  Sends subscriptionStop messages for all active subscriptions.
     *
     *  @param timeout {number} optional timeout defaulting to 0 = no timeout
     *  @returns - resolved after all SubscriptionStop messages are sent.
     *  - rejected in case of any issues or timeout occurs.
     */
    terminateAllSubscriptions(timeout = 0) {
        this.udsClient.enableShutdownMode();
        return super.terminateAllSubscriptions(timeout);
    }
    /**
     * Shuts down libjoynr
     * @param settings.clearSubscriptionsTimeoutMs {number} time in ms till clearSubscriptionsPromise will be rejected
     *  if it's not resolved yet
     * @param settings.clearSubscriptionsEnabled {boolean} clear all subscriptions before shutting down.
     *  Set this to false in process.exit handler as this is not synchronous.
     *
     * @returns - resolved after successful shutdown
     * - rejected in case of any issues
     */
    async shutdown(settings) {
        if (this.udsClient) {
            this.udsClient.enableShutdownMode();
        }
        await super.shutdown(settings);
        if (this.udsClient) {
            this.udsClient.shutdown();
        }
    }
}
module.exports = UdsLibJoynrRuntime;
