"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ArbitrationConstants_1 = require("./ArbitrationConstants");
/**
 * The ArbitrationStrategyCollection.Nothing just returns the passed array
 *
 * @param capabilities the array of capability entries
 * @param capabilities.array the array of DiscoveryEntries
 *
 * @returns an array of capabilities as it came in through the
 *          capabilities parameter
 */
function Nothing(capabilities) {
    return capabilities;
}
exports.Nothing = Nothing;
/**
 * The ArbitrationStrategyCollection.HighestPriority favors higher priorities
 *
 * @param capabilities the array of capability entries
 *
 * @returns an array of capabilities sorted by the highest priority
 */
function HighestPriority(capabilities) {
    if (!Array.isArray(capabilities)) {
        throw new Error("provided argument capabilities is not of type Array");
    }
    // sort with descending priority
    return capabilities.sort((a, b) => {
        return b.qos.priority - a.qos.priority;
    });
}
exports.HighestPriority = HighestPriority;
/**
 * The ArbitrationStrategyCollection.Keyword searches for the given keyword in the
 * "keyword" qos parameter and returns those.
 * NOTE: to use this function, you must bind your keyword as follows:
 * arbitrationStrategy = Keyword.bind(undefined, "myKeyword")
 *
 * @param capabilities the array of capability entries
 * @param keyword the keyword to search for
 * @returns an array of capabilities sorted by the highest priority
 */
function Keyword(keyword, capabilities) {
    const keywordCaps = [];
    if (!Array.isArray(capabilities)) {
        throw new Error("provided argument capabilities is not of type Array");
    }
    for (const capId in capabilities) {
        if (Object.prototype.hasOwnProperty.call(capabilities, capId)) {
            const cap = capabilities[capId];
            if (cap.qos.customParameters && Array.isArray(cap.qos.customParameters)) {
                for (const qosId in cap.qos.customParameters) {
                    if (Object.prototype.hasOwnProperty.call(cap.qos.customParameters, qosId)) {
                        const qosParam = cap.qos.customParameters[qosId];
                        if (qosParam &&
                            qosParam.value &&
                            qosParam.name === ArbitrationConstants_1.KEYWORD_PARAMETER &&
                            qosParam.value === keyword) {
                            keywordCaps.push(cap);
                        }
                    }
                }
            }
        }
    }
    return keywordCaps;
}
exports.Keyword = Keyword;
/**
 * The ArbitrationStrategyCollection.LastSeen favors latest lastSeenDateMs
 *
 * @param capabilities the array of capability entries
 * @param capabilities.array the array of capability entries
 *
 * @returns an array of capabilities sorted by the lastSeenDateMs
 */
function LastSeen(capabilities) {
    if (!Array.isArray(capabilities)) {
        throw new Error("provided argument capabilities is not of type Array");
    }
    // sort with descending priority
    return capabilities.sort((a, b) => {
        return b.lastSeenDateMs - a.lastSeenDateMs;
    });
}
exports.LastSeen = LastSeen;
/**
 * The ArbitrationStrategyCollection.FixedParticipant returns an array of one entry with the FixedParticipantId
 *
 * @param capabilities the array of capability entries
 * @param capabilities.array the array of capability entries
 *
 * @returns an array of one and solely capability which is the one of FixedParticipantId
 */
function FixedParticipant(capabilities) {
    if (!Array.isArray(capabilities)) {
        throw new Error("provided argument capabilities is not of type Array");
    }
    if (capabilities.length > 1) {
        throw new Error("multiple providers found for the given fixed participantId");
    }
    return capabilities;
}
exports.FixedParticipant = FixedParticipant;
