"use strict";
/*
 * #%L
 * %%
 * Copyright (C) 2019 BMW Car IT GmbH
 * %%
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * #L%
 */
// max value for timeout, see
// http://stackoverflow.com/questions/3468607/why-does-settimeout-break-for-large-millisecond-delay-values
/**
 * Implementation for long (>2^31-1 ms) timer functions that only allows timeouts or intervals
 * of 2^31-1 ms (because of SignedInt32 representation)
 */
class LongTimer {
    constructor() {
        this.maxTime = Math.pow(2, 31) - 1;
        this.idPrefix = "lt";
        this.timeoutMap = {};
        this.highestTimeoutId = 0;
        this.highestIntervalId = -1;
        this.intervalMap = {};
        this.timeoutPortion = (timeoutId) => {
            // retrieve timeout object
            const timeoutObj = this.timeoutMap[timeoutId];
            // timeout has been cancelled
            if (timeoutObj === undefined) {
                return;
            }
            // if timeout elapsed, remove timeout object and call function
            if (timeoutObj.remainingTimeout === 0) {
                delete this.timeoutMap[timeoutId];
                timeoutObj.func.apply(this, timeoutObj.args);
                return;
            }
            // recalculate remaining timeout and start next portion of timeout
            const timeToWait = Math.min(this.maxTime, timeoutObj.remainingTimeout);
            timeoutObj.remainingTimeout -= timeToWait;
            timeoutObj.currentTimeout = setTimeout(this.timeoutPortion, timeToWait, timeoutId);
        };
    }
    /**
     * Implementation for long (>2^31-1 ms) window.setTimeout only allows intervals of 2^31-1 ms
     * (because of SignedInt32 representation)
     *
     * @param func timout handler
     * @param timeout in ms for function to be called
     * @param args extra parameters for func
     * @returns timeout identifier
     * @throws {Error} if parameters are nullable or not of documented type
     */
    setTimeout(func, timeout, ...args) {
        if (timeout <= this.maxTime) {
            return setTimeout(func, timeout, ...args);
        }
        // get next timeout id and prefix it to avoid possible collisions in environments
        // where setTimeout returns a number (e.g. in browsers).
        this.highestTimeoutId++;
        const timeoutId = this.idPrefix + this.highestTimeoutId;
        // put timeout object into map
        this.timeoutMap[timeoutId] = {
            func,
            remainingTimeout: timeout,
            args
        };
        this.timeoutPortion(timeoutId);
        return timeoutId;
    }
    /**
     * Clears the timeout
     *
     * @param timeoutId the timeout id given by setTimeout
     * @throws {Error} if parameters are nullable or not of documented type
     */
    clearTimeout(timeoutId) {
        // retrieve timeout object
        const timeoutObj = this.timeoutMap[timeoutId];
        // timeout has run out, been cancelled already or was less than maxTime
        if (timeoutObj === undefined) {
            return clearTimeout(timeoutId);
        }
        // stop javascript interval and remove timeout object
        clearTimeout(timeoutObj.currentTimeout);
        this.timeoutMap[timeoutId] = undefined;
    }
    /**
     * Implementation for long (>2^31-1 ms) window.setInterval only allows intervals of 2^31-1 ms
     * (because of SignedInt32 representation)
     *
     * @param func
     * @param interval
     * @returns interval identifier
     * @throws {Error} if parameters are nullable or not of documented type
     */
    setInterval(func, interval) {
        // get next interval id
        const intervalId = ++this.highestIntervalId;
        const intervalPortion = () => {
            // retrieve timeout object
            const intervalObj = this.intervalMap[intervalId];
            // interval has been cancelled
            if (intervalObj === undefined) {
                return;
            }
            // if interval elapsed, call function and reset remaining interval time
            if (intervalObj.remainingInterval === 0) {
                intervalObj.func();
                intervalObj.remainingInterval = intervalObj.interval;
            }
            // recalculate remaining timeout and start next portion of interval
            const timeToWait = Math.min(this.maxTime, intervalObj.remainingInterval);
            intervalObj.remainingInterval -= timeToWait;
            intervalObj.currentTimeout = setTimeout(intervalPortion, timeToWait);
        };
        // put interval object into map
        this.intervalMap[intervalId] = {
            func,
            remainingInterval: interval,
            interval
        };
        // start interval
        intervalPortion();
        // return id to
        return intervalId;
    }
    /**
     * Clears the interval
     *
     * @param intervalId the interval id given by setInterval
     * @throws {Error} if parameters are nullable or not of documented type
     */
    clearInterval(intervalId) {
        // retrieve interval object
        const interval = this.intervalMap[intervalId];
        // interval has run out or been cancelled already
        if (interval === undefined) {
            return;
        }
        // stop javascript timeout and remove interval object
        clearTimeout(interval.currentTimeout);
        delete this.intervalMap[intervalId];
    }
}
const longTimer = new LongTimer();
module.exports = longTimer;
